@@ -1,4 +1,4 @@
-/*  $Revision: 1.23 $
+/*  $Revision: 1.18 $
 **
 **  Transmit articles to remote site.
 **  Modified for NNTP streaming: 3Jan96 Jerry Aguirre
@@ -20,7 +20,7 @@
 #include <sys/uio.h>
 #include "nntp.h"
 #include "paths.h"
-#include "logging.h"
+#include <syslog.h> 
 #include "libinn.h"
 #include "qio.h"
 #include "dbz.h"
@@ -104,7 +104,6 @@ STATIC BOOL		AlwaysRewrite;
 STATIC BOOL		Debug;
 STATIC BOOL		DoRequeue = TRUE;
 STATIC BOOL		Purging;
-STATIC BOOL		Slavish;
 STATIC BOOL		STATprint;
 STATIC BOOL		Mime;
 STATIC MIMEXFERTYPE	MimeArticle = MTnotmime;
@@ -136,43 +135,51 @@ STATIC char		*AltPath;
 **  Find the history file entry for the Message-ID and return a file
 **  positioned at the third field.
 */
-STATIC FILE *
-HistorySeek(MessageID)
-    char		*MessageID;
+STATIC FILE *HistorySeek(char *MessageID)
 {
-    static char		History[] = _PATH_HISTORY;
+    static char		*History = NULL;
     static FILE		*F;
-    register char	*p;
-    register char	*q;
-    register int	i;
-    datum		key;
-    datum		val;
     OFFSET_T		offset;
+#ifndef	DO_TAGGED_HASH
+    idxrec		ionevalue;
+    idxrecext		iextvalue;
+#endif
 
     /* Open the history file. */
     if (F == NULL) {
-	if (dbminit(History) < 0) {
+	if (History == NULL)
+	    History = COPY(cpcatpath(innconf->pathdb, _PATH_HISTORY));
+	if (!dbzinit(History)) {
 	    (void)fprintf(stderr, "Can't set up \"%s\" database, %s\n",
 		    History, strerror(errno));
+	    SMshutdown();
 	    exit(1);
 	}
 	if ((F = fopen(History, "r")) == NULL) {
 	    (void)fprintf(stderr, "Can't open \"%s\" for reading, %s\n",
 		    History, strerror(errno));
+	    SMshutdown();
 	    exit(1);
 	}
     }
 
     /* Do the lookup. */
-    key.dsize = strlen(MessageID) + 1;
-    key.dptr = MessageID;
-    val = dbzfetch(key);
-    if (val.dptr == NULL || val.dsize != sizeof offset)
+#ifdef	DO_TAGGED_HASH
+    if ((offset = dbzfetch(HashMessageID(MessageID))) < 0)
 	return NULL;
+#else
+    if (innconf->extendeddbz) {
+	if (!dbzfetch(HashMessageID(MessageID), &iextvalue))
+	    return NULL;
+	offset = iextvalue.offset[HISTOFFSET];
+    } else {
+	if (!dbzfetch(HashMessageID(MessageID), &ionevalue))
+	    return NULL;
+	offset = ionevalue.offset;
+    }
+#endif
 
     /* Get the seek offset, and seek. */
-    for (p = val.dptr, q = (char *)&offset, i = sizeof offset; --i >= 0; )
-	*q++ = *p++;
     if (fseek(F, offset, SEEK_SET) == -1)
 	return NULL;
     return F;
@@ -336,7 +343,6 @@ REMwrite(p, i, escdot)
     static char		HDR[] = "Content-Transfer-Encoding:";
     static char		COD[] =
 		"Content-Transfer-Encoding: quoted-printable\r\n";
-    register char	*dest;
     int			size;
 
     /* Buffer too full? */
@@ -362,15 +368,8 @@ REMwrite(p, i, escdot)
     /* Dot escape, text of the line, line terminator. */
     if (escdot && (*p == '.'))
 	*REMbuffptr++ = '.';
-    if (i > MEMCPY_THRESHOLD) {
-	(void)memcpy((POINTER)REMbuffptr, (POINTER)p, (SIZE_T)i);
-	REMbuffptr += i;
-    }
-    else {
-	for (dest = REMbuffptr, i++; --i > 0; )
-	    *dest++ = *p++;
-	REMbuffptr = dest;
-    }
+    (void)memcpy((POINTER)REMbuffptr, (POINTER)p, (SIZE_T)i);
+    REMbuffptr += i;
     *REMbuffptr++ = '\r';
     *REMbuffptr++ = '\n';
 
@@ -480,6 +479,7 @@ ExitWithStats(x)
 	(void)fprintf(stderr, "Can't remove \"%s\", %s\n",
 		BATCHtemp, strerror(errno));
     (void)sleep(1);
+    SMshutdown();
     exit(x);
     /* NOTREACHED */
 }
@@ -856,7 +856,8 @@ GetMessageID(qp)
     register QIOSTATE	*qp;
 {
     static char		HDR[] = "Message-ID:";
-    static char		buff[DBZMAXKEY + 1];
+    static char		*buff;
+    static int          buffsize = 0;
     register char	*p;
 
     while ((p = QIOread(qp)) != NULL)
@@ -865,9 +866,16 @@ GetMessageID(qp)
 	    /* Found the header -- skip whitespace. */
 	    for (p += STRLEN(HDR); ISWHITE(*p); p++)
 		continue;
-	    if (*p == '\0' || (int)strlen(p) > DBZMAXKEY)
-		/* Header is empty or too long. */
+	    if (*p == '\0')
+		/* Header is empty*/
 		break;
+	    if (strlen(p) >= buffsize) {
+		if (buffsize)
+		    buff = RENEW(buff, char, strlen(p) + 1);
+		else
+		    buff = NEW(char, strlen(p) + 1);
+		
+	    }
 	    (void)strcpy(buff, p);
 	    return buff;
 	}
@@ -993,11 +1001,11 @@ takethis(i)
 
     if (!stbuf[i].st_qp) { /* should already be open but ... */
 	/* Open the article. */
-	if (!(stbuf[i].st_qp = QIOopen(stbuf[i].st_fname, QIO_BUFFER))) {
+	if (!(stbuf[i].st_qp = QIOopen(stbuf[i].st_fname))) {
 	    /* can not open it. Should check AltPath */
 	    if (AltPath && (*(stbuf[i].st_fname) != '/')) {
 		(void)sprintf(AltPath, "%s/%s", AltSpool, stbuf[i].st_fname);
-		stbuf[i].st_qp = QIOopen(AltPath, QIO_BUFFER);
+		stbuf[i].st_qp = QIOopen(AltPath);
 	    }
 	    if (!(stbuf[i].st_qp)) {
 		strel(i);
@@ -1132,22 +1140,17 @@ STATIC NORETURN
 Usage()
 {
     (void)fprintf(stderr,
-	"Usage: innxmit [-a] [-c] [-d] [-M] [-p] [-r] [-s] [-S] [-t#] [-T#] host file\n");
+	"Usage: innxmit [-a] [-c] [-d] [-M] [-p] [-r] [-s] [-t#] [-T#] host file\n");
     exit(1);
 }
 
 
-int
-main(ac, av)
-    int			ac;
-    char		*av[];
+int main(int ac, char *av[])
 {
-    static char		SPOOL[] = _PATH_SPOOL;
-    static char		BATCHDIR[] = _PATH_BATCHDIR;
     static char		SKIPPING[] = "Skipping \"%s\" --%s?\n";
-    register int	i;
-    register char	*p;
-    register QIOSTATE	*qp;
+    int	                i;
+    char	        *p;
+    QIOSTATE	        *qp;
     TIMEINFO		Now;
     FILE		*From;
     FILE		*To;
@@ -1159,16 +1162,19 @@ main(ac, av)
     SIGHANDLER		(*old)();
     unsigned int	ConnectTimeout;
     unsigned int	TotalTimeout;
-    extern char		*nntp_port;
+    int                 port = NNTP_PORT;
 
     /* Set defaults. */
+    if (ReadInnConf() < 0) exit(1);
+
     ConnectTimeout = 0;
     TotalTimeout = 0;
     AltSpool = NULL;
+    
     (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "lA:acdMprsSt:T:vP:")) != EOF)
+    while ((i = getopt(ac, av, "lA:acdMprst:T:vP:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -1178,7 +1184,7 @@ main(ac, av)
 	    AltPath = NEW(char, SPOOLNAMEBUFF + strlen(AltSpool));
 	    break;
 	case 'P':
-	    nntp_port = COPY(optarg);
+	    port = atoi(COPY(optarg));
 	    break;
 	case 'a':
 	    AlwaysRewrite = TRUE;
@@ -1205,10 +1211,6 @@ main(ac, av)
 	case 's':
 	    TryStream = FALSE;
 	    break;
-	case 'S':
-	    Slavish = TRUE;
-	    TryStream = FALSE; /* streaming does not support xreplic */
-	    break;
 	case 't':
 	    ConnectTimeout = atoi(optarg);
 	    break;
@@ -1228,32 +1230,49 @@ main(ac, av)
     REMhost = av[0];
     BATCHname = av[1];
 
-    if (chdir(SPOOL) < 0) {
+    if (chdir(innconf->patharticles) < 0) {
 	(void)fprintf(stderr, "Can't cd to \"%s\", %s\n",
-		SPOOL, strerror(errno));
+		innconf->patharticles, strerror(errno));
 	exit(1);
     }
 
     (void)openlog("innxmit", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
 
+    if (innconf->storageapi) {
+	BOOL val;
+	val = TRUE;
+	if (!SMsetup(SM_PREOPEN,(void *)&val)) {
+	    fprintf(stderr, "Can't setup the storage manager\n");
+	    exit(1);
+	}
+	if (!SMinit()) {
+	    fprintf(stderr, "Can't initialize the storage manager: %s\n", SMerrorstr);
+	    exit(1);
+	}
+    }
+
     /* Open the batch file and lock others out. */
     if (BATCHname[0] != '/') {
-	BATCHname = NEW(char, STRLEN(BATCHDIR) + 1 + strlen(av[1]) + 1);
-	(void)sprintf(BATCHname, "%s/%s", BATCHDIR, av[1]);
+	BATCHname = NEW(char, strlen(innconf->pathoutgoing) + 1 +
+						strlen(av[1]) + 1);
+	(void)sprintf(BATCHname, "%s/%s", innconf->pathoutgoing, av[1]);
     }
-    if ((i = open(BATCHname, O_RDWR)) < 0
-     || (BATCHqp = QIOfdopen(i, QIO_BUFFER)) == NULL) {
+    if (((i = open(BATCHname, O_RDWR)) < 0) || ((BATCHqp = QIOfdopen(i)) == NULL)) {
 	(void)fprintf(stderr, "Can't open \"%s\", %s\n",
 		BATCHname, strerror(errno));
+	SMshutdown();
 	exit(1);
     }
     if (LockFile(QIOfileno(BATCHqp), TRUE) < 0) {
 #if	defined(EWOULDBLOCK)
-	if (errno == EWOULDBLOCK)
+	if (errno == EWOULDBLOCK) {
+	    SMshutdown();
 	    exit(0);
+	}
 #endif	/* defined(EWOULDBLOCK) */
 	(void)fprintf(stderr, "Can't lock \"%s\", %s\n",
 		BATCHname, strerror(errno));
+	SMshutdown();
 	exit(1);
     }
 
@@ -1272,6 +1291,7 @@ main(ac, av)
     /* Start timing. */
     if (GetTimeInfo(&Now) < 0) {
 	(void)fprintf(stderr, "Can't get time, %s\n", strerror(errno));
+	SMshutdown();
 	exit(1);
     }
     STATbegin = TIMEINFOasDOUBLE(Now);
@@ -1286,18 +1306,20 @@ main(ac, av)
 	    if (setjmp(JMPwhere)) {
 		(void)fprintf(stderr, "Can't connect to %s, timed out\n",
 			REMhost);
+		SMshutdown();
 		exit(1);
 	    }
 	}
-	if (NNTPconnect(REMhost, &From, &To, buff) < 0 || GotAlarm) {
+	if (NNTPconnect(REMhost, port, &From, &To, buff) < 0 || GotAlarm) {
 	    i = errno;
 	    (void)fprintf(stderr, "Can't connect to %s, %s\n",
 		    REMhost, buff[0] ? REMclean(buff) : strerror(errno));
 	    if (GotAlarm)
 		syslog(L_NOTICE, CANT_CONNECT, REMhost, "timeout");
-	    else
+	    else 
 		syslog(L_NOTICE, CANT_CONNECT, REMhost,
 		    buff[0] ? REMclean(buff) : strerror(i));
+	    SMshutdown();
 	    exit(1);
 	}
 	if (Debug)
@@ -1309,6 +1331,7 @@ main(ac, av)
 	    syslog(L_ERROR, CANT_AUTHENTICATE,
 		REMhost, GotAlarm ? "timeout" : strerror(i));
 	    /* Don't send quit; we want the remote to print a message. */
+	    SMshutdown();
 	    exit(1);
 	}
 	if (ConnectTimeout) {
@@ -1412,19 +1435,17 @@ main(ac, av)
 
 	/* Split the line into possibly two fields. */
 	if (Article[0] == '/'
-	 && Article[STRLEN(SPOOL)] == '/'
-	 && EQn(Article, SPOOL, STRLEN(SPOOL)))
-	    Article += STRLEN(SPOOL) + 1;
+	 && Article[strlen(innconf->patharticles)] == '/'
+	 && EQn(Article, innconf->patharticles, strlen(innconf->patharticles)))
+	    Article += strlen(innconf->patharticles) + 1;
 	if ((MessageID = strchr(Article, ' ')) != NULL) {
 	    *MessageID++ = '\0';
-	    if (!Slavish) {
-		if (*MessageID != '<'
-		 || (p = strrchr(MessageID, '>')) == NULL
-		 || *++p != '\0') {
-		    (void)fprintf(stderr, "Ignoring line \"%s %s...\"\n",
-			Article, MessageID);
-		    continue;
-		}
+	    if (*MessageID != '<'
+		|| (p = strrchr(MessageID, '>')) == NULL
+		|| *++p != '\0') {
+		(void)fprintf(stderr, "Ignoring line \"%s %s...\"\n",
+			      Article, MessageID);
+		continue;
 	    }
 	}
 
@@ -1444,15 +1465,11 @@ main(ac, av)
 	}
 
         /*
-         * If the command ("IHAVE" or "XREPLIC") plus the "message-id",
-         * separating space and trailing CR-NL will exceed the maximum command
-         * length permitted by the RFC (i.e. NNTP_STRLEN), then reject the
-         * article and continue to avoid overrunning buffers and throwing the
-         * server on the recieving end a blow from behind.  This really comes
-         * into play with XREPLIC feeds to slaved servers where cross-posting
-         * really causes the "message-ID" to get too long really quickly.  Note
-         * that the value of 12 is the length of "XREPLIC " plus 2 for the
-         * CR-NL, plus 2 for a safety buffer.
+         * If the IHAVE plus the "message-id", separating space and trailing
+	 * CR-NL will exceed the maximum command length permitted by the RFC
+	 * (i.e. NNTP_STRLEN), then reject the article and continue to avoid
+	 * overrunning buffers and throwing the server on the recieving end a
+	 * blow from behind.  
          */
         if (MessageID != NULL && (strlen(MessageID) > NNTP_STRLEN - 12)) {
             (void)fprintf(stderr, "Dropping article in \"%s\" - long message id \"%s\"\n",
@@ -1461,14 +1478,23 @@ main(ac, av)
         }
 
 	/* Open the article. */
-	if ((qp = QIOopen(Article, QIO_BUFFER)) == NULL
+	if ((qp = QIOopen(Article)) == NULL
 	 && AltSpool
 	 && *Article != '/') {
 	    (void)sprintf(AltPath, "%s/%s", AltSpool, Article);
-	    qp = QIOopen(AltPath, QIO_BUFFER);
+	    qp = QIOopen(AltPath);
 	}
 
 	if (qp == NULL) {
+	    if (IsToken(Article)) {
+		if ((SMerrno == SMERR_NOENT) || (SMerrno == SMERR_UNINIT))
+		    continue;
+		else {
+		    (void)fprintf(stderr, "Requeue \"%s\", %s\n",
+			Article, SMerrorstr);
+		    Requeue(Article, MessageID);
+		}
+	    }
 	    switch (errno) {
 	    default:
 		(void)fprintf(stderr, "Requeue \"%s\", %s\n",
@@ -1593,7 +1619,7 @@ main(ac, av)
 	    }
 	    continue; /* next article */
 	}
-	(void)sprintf(buff, "%s %s", Slavish ? "xreplic" : "ihave", MessageID);
+	(void)sprintf(buff, "ihave %s", MessageID);
 	if (!REMwrite(buff, (int)strlen(buff), FALSE)) {
 	    (void)fprintf(stderr, "Can't offer article, %s\n",
 		    strerror(errno));
