@@ -1,5 +1,5 @@
 #! /bin/sh
-##  $Revision: 1.5 $
+##  $Revision: 1.1.1.1 $
 ##
 ##  Prepare a manpage for installation, and install it.  Usage:
 ##	putman <style> "<installitflags>" <source> <dest-dir>
@@ -37,7 +37,7 @@ XNROFF-PACK-SCO)
     rm -f $T
     exit
     ;;
-XNROFF-NET2)
+XBSD4.4)
     T=${TMPDIR-/tmp}/man$$
     nroff -man ${SRC} >$T
     DEST2=`echo ${DEST} | sed -e 's/\..$/.0/'`
