@@ -1,4 +1,4 @@
-/*  $Revision: 1.10 $
+/*  $Revision: 1.15 $
 **
 **  Expire overview database.
 */
@@ -16,6 +16,7 @@
 #include "libinn.h"
 #include "macros.h"
 #include "paths.h"
+#include <syslog.h>  
 
 
 #define	START_LIST_SIZE	128
@@ -31,7 +32,6 @@ typedef struct _LINE {
     int		Offset;
 } LINE;
 
-
 /*
 **  A list of articles; re-uses space.
 */
@@ -56,12 +56,12 @@ typedef struct _BUFFER {
 **  Information about the schema of the news overview files.
 */
 typedef struct _ARTOVERFIELD {
-    char	*Header;
-    int		Length;
-    BOOL	HasHeader;
+    char       *Header;
+    int                Length;
+    BOOL       HasHeader;
 } ARTOVERFIELD;
 
-
+    
 /*
 **  Append an article to an LIST.
 */
@@ -78,10 +78,11 @@ typedef struct _ARTOVERFIELD {
 /*
 **  Global variables.
 */
-STATIC char		SPOOL[] = _PATH_SPOOL;
-STATIC char		*SCHEMA = _PATH_SCHEMA;
+STATIC char		*SPOOL = NULL;
+STATIC char		*SCHEMA;
 STATIC BOOL		InSpoolDir;
 STATIC BOOL		Verbose;
+STATIC BOOL		Quiet;
 STATIC BOOL		DoNothing;
 STATIC ARTOVERFIELD	*ARTfields;
 STATIC int		ARTfieldsize;
@@ -90,10 +91,7 @@ STATIC int		ARTfieldsize;
 /*
 **  Sorting predicate for qsort to put articles in numeric order.
 */
-STATIC int
-LISTcompare(p1, p2)
-    CPOINTER *p1;
-    CPOINTER *p2;
+STATIC int LISTcompare(CPOINTER p1, CPOINTER p2)
 {
     ARTNUM	*ip1;
     ARTNUM	*ip2;
@@ -107,12 +105,10 @@ LISTcompare(p1, p2)
 /*
 **  If list is big enough, and out of order, sort it.
 */
-STATIC void
-LISTsort(lp)
-    LIST	*lp;
+STATIC void LISTsort(LIST *lp)
 {
-    register int	i;
-    register ARTNUM	*ap;
+    int	                i;
+    ARTNUM	        *ap;
 
     for (ap = lp->Articles, i = lp->Used - 1; --i >= 0; ap++)
 	if (ap[0] >= ap[1]) {
@@ -122,14 +118,54 @@ LISTsort(lp)
 	}
 }
 
+/*
+**  Read the overview schema.
+*/
+static void ARTreadschema(void)
+{
+    FILE		        *F;
+    char		        *p;
+    ARTOVERFIELD	        *fp;
+    int		                i;
+    char			buff[SMBUF];
+
+    /* Open file, count lines. */
+    if ((F = fopen(SCHEMA, "r")) == NULL) {
+	(void)fprintf(stderr, "Can't open %s, %s\n", SCHEMA, strerror(errno));
+	exit(1);
+    }
+    for (i = 0; fgets(buff, sizeof buff, F) != NULL; i++)
+	continue;
+    (void)fseek(F, (OFFSET_T)0, SEEK_SET);
+    ARTfields = NEW(ARTOVERFIELD, i + 1);
+
+    /* Parse each field. */
+    for (fp = ARTfields; fgets(buff, sizeof buff, F) != NULL; ) {
+	/* Ignore blank and comment lines. */
+	if ((p = strchr(buff, '\n')) != NULL)
+	    *p = '\0';
+	if ((p = strchr(buff, COMMENT_CHAR)) != NULL)
+	    *p = '\0';
+	if (buff[0] == '\0')
+	    continue;
+	if ((p = strchr(buff, ':')) != NULL) {
+	    *p++ = '\0';
+	    fp->HasHeader = EQ(p, "full");
+	}
+	else
+	    fp->HasHeader = FALSE;
+	fp->Header = COPY(buff);
+	fp->Length = strlen(buff);
+	fp++;
+    }
+    ARTfieldsize = fp - ARTfields;
+    (void)fclose(F);
+}
 
 /*
 **  Unlock the group.
 */
-STATIC void
-UnlockGroup(lfd, lockfile)
-    int		lfd;
-    char	*lockfile;
+STATIC void UnlockGroup(int lfd, char *lockfile)
 {
     if (lfd > 0) {
 	if (unlink(lockfile) < 0 && errno != ENOENT)
@@ -146,10 +182,7 @@ UnlockGroup(lfd, lockfile)
 /*
 **  Sorting predicate to put lines in numeric order.
 */
-STATIC int
-LINEcompare(p1, p2)
-    CPOINTER *p1;
-    CPOINTER *p2;
+STATIC int LINEcompare(CPOINTER p1, CPOINTER p2)
 {
     LINE	*lp1;
     LINE	*lp2;
@@ -159,26 +192,22 @@ LINEcompare(p1, p2)
     return lp1->Article - lp2->Article;
 }
 
-
 /*
 **  Take in a sorted list of count article numbers in group, and delete
 **  them from the overview file.
 */
-STATIC void
-RemoveLines(group, Deletes)
-    char			*group;
-    LIST			*Deletes;
+STATIC void RemoveLines(char *group, LIST *Deletes)
 {
     static BUFFER		B;
     static LINE			*Lines;
     static int			LineSize;
-    register struct iovec	*vp;
-    register LINE		*lp;
-    register LINE		*end;
-    register char		*p;
-    register char		*next;
-    register ARTNUM		*ap;
-    register int		i;
+    struct iovec	        *vp;
+    LINE		        *lp;
+    LINE		        *end;
+    char		        *p;
+    char		        *next;
+    ARTNUM		        *ap;
+    int		                i;
     struct stat			Sb;
     struct iovec		iov[8];
     char			file[SPOOLNAMEBUFF];
@@ -186,6 +215,9 @@ RemoveLines(group, Deletes)
     int				fd;
     int				count;
     int				lfd;
+    int				offset;
+    
+    offset = 0;
 
     if (Verbose) {
 	for (ap = Deletes->Articles, i = Deletes->Used; --i >= 0; ap++)
@@ -195,16 +227,17 @@ RemoveLines(group, Deletes)
     }
 
     /* Lock the group. */
-    (void)sprintf(lockfile, "%s/.LCK%s", group, _PATH_OVERVIEW);
+    (void)sprintf(lockfile, "%s/.LCK%s", group, innconf->overviewname);
     lfd = open(lockfile, O_WRONLY | O_TRUNC | O_CREAT, ARTFILE_MODE);
     if (lfd < 0) {
-	(void)fprintf(stderr, "Can't open %s, %s\n", lockfile, strerror(errno));
+        if (!Quiet)
+	    (void)fprintf(stderr, "Can't open %s, %s\n", lockfile, strerror(errno));
 	return;
     }
 
     /* Open file, lock it. */
-    (void)sprintf(file, "%s/%s", group, _PATH_OVERVIEW);
-    for ( ; ; ) {
+    (void)sprintf(file, "%s/%s", group, innconf->overviewname);
+    for (i = 0; i < 15; i++) {
 	if ((fd = open(file, O_RDWR)) < 0) {
 	    (void)fprintf(stderr, "Can't open %s, %s\n", file, strerror(errno));
 	    UnlockGroup(lfd, lockfile);
@@ -215,6 +248,12 @@ RemoveLines(group, Deletes)
 	/* Wait for lock; close file -- might be unlinked -- and try again. */
 	(void)LockFile(fd, TRUE);
 	(void)close(fd);
+	sleep(i);
+    }
+    if (i >= 15) {
+	fprintf(stderr, "Can't open/lock %s, %s\n", file, strerror(errno));
+	close(fd);
+	return;
     }
 
     if (fstat(fd, &Sb) < 0) {
@@ -303,6 +342,7 @@ RemoveLines(group, Deletes)
 	    if (((char *) vp->iov_base) + vp->iov_len == lp->Start) {
 		/* Contiguous. */
 		vp->iov_len += lp->Length;
+		offset += lp->Length;
 		continue;
 	    }
 
@@ -322,6 +362,7 @@ RemoveLines(group, Deletes)
 	/* Start new element. */
 	vp->iov_base = lp->Start;
 	vp->iov_len = lp->Length;
+	offset += lp->Length;
     }
 
     /* Write out remaining. */
@@ -334,7 +375,7 @@ RemoveLines(group, Deletes)
 	(void)close(fd);
 	return;
     }
-
+    
     if (rename(lockfile, file) < 0)
 	(void)fprintf(stderr, "Can't rename %s, %s\n",
 		lockfile, strerror(errno));
@@ -349,79 +390,32 @@ RemoveLines(group, Deletes)
 }
 
 
-/*
-**  Read the overview schema.
-*/
-void
-ARTreadschema()
-{
-    register FILE		*F;
-    register char		*p;
-    register ARTOVERFIELD	*fp;
-    register int		i;
-    char			buff[SMBUF];
-
-    /* Open file, count lines. */
-    if ((F = fopen(SCHEMA, "r")) == NULL) {
-	(void)fprintf(stderr, "Can't open %s, %s\n", SCHEMA, strerror(errno));
-	exit(1);
-    }
-    for (i = 0; fgets(buff, sizeof buff, F) != NULL; i++)
-	continue;
-    (void)fseek(F, (OFFSET_T)0, SEEK_SET);
-    ARTfields = NEW(ARTOVERFIELD, i + 1);
-
-    /* Parse each field. */
-    for (fp = ARTfields; fgets(buff, sizeof buff, F) != NULL; ) {
-	/* Ignore blank and comment lines. */
-	if ((p = strchr(buff, '\n')) != NULL)
-	    *p = '\0';
-	if ((p = strchr(buff, COMMENT_CHAR)) != NULL)
-	    *p = '\0';
-	if (buff[0] == '\0')
-	    continue;
-	if ((p = strchr(buff, ':')) != NULL) {
-	    *p++ = '\0';
-	    fp->HasHeader = EQ(p, "full");
-	}
-	else
-	    fp->HasHeader = FALSE;
-	fp->Header = COPY(buff);
-	fp->Length = strlen(buff);
-	fp++;
-    }
-    ARTfieldsize = fp - ARTfields;
-    (void)fclose(F);
-}
-
 
 /*
 **  Read an article and create an overview line without the trailing
 **  newline.  Returns pointer to static space or NULL on error.
 */
-STATIC char *
-OVERgen(name)
-    char			*name;
+STATIC char *OVERgen(char *name)
 {
     static ARTOVERFIELD		*Headers;
     static BUFFER		B;
-    register ARTOVERFIELD	*fp;
-    register ARTOVERFIELD	*hp;
-    register ARTOVERFIELD	*lasthp = 0;
-    register QIOSTATE		*qp;
-    register char		*colon;
-    register char		*line;
-    register char		*p;
-    register int		i;
-    register int		size;
-    register int		ov_size;
-    register long		lines;
+    ARTOVERFIELD	        *fp;
+    ARTOVERFIELD	 	*hp;
+    ARTOVERFIELD		*lasthp = 0;
+    QIOSTATE			*qp;
+    char			*colon;
+    char			*line;
+    char			*p;
+    int		        	i;
+    int		        	size;
+    int		        	ov_size;
+    long			lines;
     struct stat			Sb;
     long			t;
     char			value[10];
 
     /* Open article. */
-    if ((qp = QIOopen(name, QIO_BUFFER)) == NULL)
+    if ((qp = QIOopen(name)) == NULL)
 	return NULL;
     if ((p = strrchr(name, '/')) != NULL)
 	name = p + 1;
@@ -431,6 +425,19 @@ OVERgen(name)
 	Headers = NEW(ARTOVERFIELD, ARTfieldsize);
 	for (hp = Headers, i = ARTfieldsize; --i >= 0; hp++)
 	    hp->Length = 0;
+    } else {
+	/* This disposes from the previous call.  This simplifies
+	   handling later on.  We trade off this readable code
+	   for the problem that nothing DISPOSEs() the last
+	   caller's use.  mibsoft 8/22/97
+	 */
+	for (hp = Headers, i = ARTfieldsize; --i >= 0; hp++) {
+	    if (hp->Length) {
+		DISPOSE(hp->Header);
+		hp->Header = 0;
+	    }
+	    hp->Length = 0;
+	}
     }
     for (hp = Headers, i = ARTfieldsize; --i >= 0; hp++)
 	hp->HasHeader = FALSE;
@@ -468,31 +475,34 @@ OVERgen(name)
 		for (p = colon; *++p && ISWHITE(*p); )
 		    continue;
 	    size = strlen(p);
-	    if (hp->Length == 0) {
-		hp->Length = size;
-		hp->Header = NEW(char, hp->Length + 1);
-	    }
-	    else if (hp->Length < size) {
-		hp->Length = size;
-		RENEW(hp->Header, char, hp->Length + 1);
+	    if (!size) { /* Ignore empty headers 11/5/97 due to pmb1@york.ac.uk */
+	        i = -1; /* Abort */
+	        lasthp = 0;
+	        break;
 	    }
+	     
+            hp->Length = size;
+            hp->Header = NEW(char, hp->Length + 1);
 	    (void)strcpy(hp->Header, p);
 	    for (p = hp->Header; *p; p++)
 		if (*p == '\t' || *p == '\n' || *p == '\r')
 		    *p = ' ';
 	    hp->HasHeader = TRUE;
             lasthp = hp;
+            break ;             /* the first one is used */
 	}
         /* handle multi-line headers -- kondou@uxd.fc.nec.co.jp */
-	if (lasthp && ISWHITE(*line)) {
-	    register int oldsize = lasthp->Length;
-	    size = strlen(line);
-	    lasthp->Length += size + 1;
-	    RENEW(lasthp->Header, char, lasthp->Length + 1);
-	    for (p = line; *p; p++)
-		if (*p == '\t' || *p == '\n' || *p == '\r')
-		    *p = ' ';
-	    strcat(lasthp->Header, line);
+        if (i < 0) {
+            if (lasthp && ISWHITE(*line)) {
+                lasthp->Length += strlen(line);
+                RENEW(lasthp->Header, char, lasthp->Length + 1);
+                for (p = line; *p; p++)
+                    if (*p == '\t' || *p == '\n' || *p == '\r')
+                        *p = ' ';
+                strcat(lasthp->Header, line);
+            } else {
+                lasthp = 0 ;
+            }
         }
     }
 
@@ -562,20 +572,17 @@ OVERgen(name)
 **  Take in a sorted list of count article numbers in group, and add
 **  them them to the overview file.
 */
-STATIC void
-AddLines(group, Adds)
-    char			*group;
-    LIST			*Adds;
+STATIC void AddLines(char *group, LIST *Adds)
 {
     static BUFFER		New;
     static BUFFER		B;
     static LINE			*Lines;
     static int			LineSize;
-    register LINE		*lp;
-    register char		*next;
-    register int		i;
-    register struct iovec	*vp;
-    register ARTNUM		*ap;
+    LINE		        *lp;
+    char		        *next;
+    int		                i;
+    struct iovec	        *vp;
+    ARTNUM		        *ap;
     LINE			*end;
     struct iovec		iov[8];
     struct stat			Sb;
@@ -585,6 +592,9 @@ AddLines(group, Adds)
     int				LineUsed;
     int				fd;
     int				lfd;
+    int				offset;
+
+    offset = 0;
 
     if (Verbose) {
 	for (ap = Adds->Articles, i = Adds->Used; --i >= 0; ap++)
@@ -635,7 +645,7 @@ AddLines(group, Adds)
 	lp->Start = New.Data + lp->Offset;
 
     /* Lock the group. */
-    (void)sprintf(lockfile, "%s/.LCK%s", group, _PATH_OVERVIEW);
+    (void)sprintf(lockfile, "%s/.LCK%s", group, innconf->overviewname);
     lfd = open(lockfile, O_WRONLY | O_TRUNC | O_CREAT, ARTFILE_MODE);
     if (lfd < 0) {
 	(void)fprintf(stderr, "Can't open %s, %s\n", lockfile, strerror(errno));
@@ -643,7 +653,7 @@ AddLines(group, Adds)
     }
 
     /* Open file, lock it. */
-    (void)sprintf(file, "%s/%s", group, _PATH_OVERVIEW);
+    (void)sprintf(file, "%s/%s", group, innconf->overviewname);
     for ( ; ; ) {
 	if ((fd = open(file, O_RDWR | O_CREAT, ARTFILE_MODE)) < 0) {
 	    (void)fprintf(stderr, "Can't open %s, %s\n", file, strerror(errno));
@@ -722,6 +732,7 @@ AddLines(group, Adds)
 	    if (((char *) vp->iov_base) + vp->iov_len == lp->Start) {
 		/* Contiguous. */
 		vp->iov_len += lp->Length;
+		offset += lp->Length;
 		continue;
 	    }
 
@@ -741,6 +752,7 @@ AddLines(group, Adds)
 	/* Start new element. */
 	vp->iov_base = lp->Start;
 	vp->iov_len = lp->Length;
+	offset += lp->Length;
     }
 
     if (vp->iov_len)
@@ -767,19 +779,17 @@ AddLines(group, Adds)
 	(void)fprintf(stderr, "expireover cant close unlinked %s %s\n",
 		file, strerror(errno));
 }
+
 
 
 /*
 **  Expire by batch, or line at a time.
 */
-STATIC void
-Expire(SortedInput, qp)
-    BOOL		SortedInput;
-    register QIOSTATE	*qp;
+STATIC void Expire(BOOL SortedInput, QIOSTATE *qp)
 {
     static LIST		List;
-    register char	*line;
-    register char	*p;
+    char	        *line;
+    char	        *p;
     char		group[SPOOLNAMEBUFF];
 
     if (List.Articles == NULL) {
@@ -849,18 +859,18 @@ Expire(SortedInput, qp)
 /*
 **  Read the overview file, return sorted list of all articles in it.
 */
-STATIC LIST *
-GetOverviewList(group)
-    char		*group;
+STATIC LIST *GetOverviewList(char *group, int *numentries)
 {
     static LIST		List;
-    register QIOSTATE	*qp;
-    register char	*p;
+    QIOSTATE	        *qp;
+    char	        *p;
     char		file[SPOOLNAMEBUFF];
 
+    *numentries = 0;
+
     /* Open the file. */
-    (void)sprintf(file, "%s/%s", group, _PATH_OVERVIEW);
-    if ((qp = QIOopen(file, QIO_BUFFER)) == NULL)
+    (void)sprintf(file, "%s/%s", group, innconf->overviewname);
+    if ((qp = QIOopen(file)) == NULL)
 	return NULL;
 
     /* Setup the article list. */
@@ -884,6 +894,7 @@ GetOverviewList(group)
 	    break;
 	}
 	LISTappend(List, atol(p));
+	*numentries++;
     }
     QIOclose(qp);
 
@@ -897,14 +908,12 @@ GetOverviewList(group)
 /*
 **  Read spool directory and return sorted list of articles or NULL on error.
 */
-STATIC LIST *
-GetSpoolList(group)
-    char		*group;
+STATIC LIST *GetSpoolList(char *group)
 {
     static LIST		List;
-    register DIR	*dp;
-    register DIRENTRY	*ep;
-    register char	*p;
+    DIR	                *dp;
+    DIRENTRY	        *ep;
+    char	        *p;
     char		buff[SPOOLNAMEBUFF];
 
     /* Open directory. */
@@ -943,16 +952,13 @@ GetSpoolList(group)
 **  the Spool list.  Both lists are sorted.  See SpoolUpdate for an
 **  explanation of the names.
 */
-STATIC LIST *
-GetNotIn(Over, Spool)
-    register LIST	*Over;
-    register LIST	*Spool;
+STATIC LIST *GetNotIn(LIST *Over, LIST *Spool)
 {
     static LIST		List;
-    register ARTNUM	*oEnd;
-    register ARTNUM	*sEnd;
-    register ARTNUM	*o;
-    register ARTNUM	*s;
+    ARTNUM	        *oEnd;
+    ARTNUM	        *sEnd;
+    ARTNUM	        *o;
+    ARTNUM	        *s;
 
     /* Setup the list. */
     if (List.Articles == NULL) {
@@ -989,9 +995,7 @@ GetNotIn(Over, Spool)
 /*
 **  Try to make one directory.  Return FALSE on error.
 */
-STATIC BOOL
-MakeDir(Name)
-    char	*Name;
+STATIC BOOL MakeDir(char *Name)
 {
     struct stat	Sb;
 
@@ -1007,11 +1011,9 @@ MakeDir(Name)
 **  Given a directory, comp/foo/bar, create that directory and all
 **  intermediate directories needed.  Return FALSE on error.
 */
-BOOL
-MakeOverDir(Name)
-    register char	*Name;
+static BOOL MakeOverDir(char *Name)
 {
-    register char	*p;
+    char	        *p;
     BOOL		made;
 
     /* Optimize common case -- parent almost always exists. */
@@ -1035,22 +1037,22 @@ MakeOverDir(Name)
 /*
 **  Update using the News Spool.  Either add or delete entries.
 */
-STATIC void
-SpoolUpdate(AddEntries, Name)
-    BOOL		AddEntries;
-    char		*Name;
+STATIC void SpoolUpdate(BOOL AddEntries, char *Name)
 {
-    register QIOSTATE	*qp;
-    register char	*line;
-    register char	*p;
+    QIOSTATE 		*qp;
+    char		*line;
+    char		*p;
     LIST		*Over;
     LIST		*Spool;
     LIST		*Missing;
+    int			processed;
+    int			overentries;
+    int			oidxentries;
 
     /* Open file. */
     if (EQ(Name, "-"))
-	qp = QIOfdopen(STDIN, QIO_BUFFER);
-    else if ((qp = QIOopen(Name, QIO_BUFFER)) == NULL) {
+	qp = QIOfdopen(STDIN);
+    else if ((qp = QIOopen(Name)) == NULL) {
 	(void)fprintf(stderr, "Can't open %s, %s\n", Name, strerror(errno));
 	exit(1);
     }
@@ -1075,32 +1077,49 @@ SpoolUpdate(AddEntries, Name)
 	*p = '\0';
 
 	Spool = GetSpoolList(line);
-	Over = GetOverviewList(line);
+	Over = GetOverviewList(line, &overentries);
+	processed = 0;
 
 	if (AddEntries && (Spool != NULL)) {
 	    if (Over != NULL) {
-		if ((Missing = GetNotIn(Spool, Over)) != NULL)
+		if ((Missing = GetNotIn(Spool, Over)) != NULL) {
 		    AddLines(line, Missing);
+		    processed++;
+		}
 	    }
 	    else if (!InSpoolDir) {
-		if (MakeOverDir(line))
+		if (MakeOverDir(line)) {
 		    AddLines(line, Spool);
-		else
+		    processed++;
+		} else
 		    (void)fprintf(stderr, "expireover: cant mkdir %s, %s\n",
 			    line, strerror(errno));
 	    }
-	    else
+	    else {
 		AddLines(line, Spool);
+		processed++;
+	    }
 	}
 
-	if (Over == NULL)
-	    continue;
-	if (Spool != NULL) {
-	    if ((Missing = GetNotIn(Over, Spool)) != NULL)
-		RemoveLines(line, Missing);
+	if (Over != NULL) {
+    	    if (Spool != NULL) {
+	        if ((Missing = GetNotIn(Over, Spool)) != NULL) {
+		    RemoveLines(line, Missing);
+		    processed++;
+		}
+	    }
+	    else {
+	        RemoveLines(line, Over);
+	        processed++;
+	    }
+	}
+	if (!processed) {
+	    if (Over) {
+	        if (Over->Used == 0)
+	            continue;
+	    } else
+	        continue;
 	}
-	else
-	    RemoveLines(line, Over);
     }
 
     QIOclose(qp);
@@ -1112,20 +1131,16 @@ SpoolUpdate(AddEntries, Name)
 /*
 **  Print usage message and exit.
 */
-STATIC NORETURN
-Usage()
+STATIC NORETURN Usage(void)
 {
     (void)fprintf(stderr, "Usage:  expireover [flags] [file...]\n");
     exit(1);
 }
 
 
-int
-main(ac, av)
-    int			ac;
-    char		*av[];
+int main(int ac, char *av[])
 {
-    register int	i;
+    int	                i;
     QIOSTATE		*qp;
     BOOL		AddEntries;
     BOOL		ReadSpool;
@@ -1133,16 +1148,25 @@ main(ac, av)
     char		*Dir;
     char		*Name;
 
+    /* First thing, set up logging and our identity. */
+    openlog("expireover", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);     
+
     /* Set defaults. */
-    Dir = _PATH_OVERVIEWDIR;
-    Name = _PATH_ACTIVE;
     AddEntries = FALSE;
     ReadSpool = FALSE;
     SortedInput = FALSE;
+
+    if (ReadInnConf() < 0) exit(1); 
+
     (void)umask(NEWSUMASK);
 
+    Dir = innconf->pathoverview;
+    Name = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVE));
+    SCHEMA = COPY(cpcatpath(innconf->pathetc, _PATH_SCHEMA));
+    SPOOL = innconf->patharticles;
+
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "aD:f:nO:svz")) != EOF)
+    while ((i = getopt(ac, av, "aD:f:nO:qsvz")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -1163,6 +1187,9 @@ main(ac, av)
 	case 'O':
 	    SCHEMA = optarg;
 	    break;
+	case 'q':
+	    Quiet = TRUE;
+	    break;
 	case 's':
 	    ReadSpool = TRUE;
 	    break;
@@ -1189,12 +1216,12 @@ main(ac, av)
     if (ReadSpool)
 	SpoolUpdate(AddEntries, Name);
     if (ac == 0)
-	Expire(SortedInput, QIOfdopen(STDIN, QIO_BUFFER));
+	Expire(SortedInput, QIOfdopen(STDIN));
     else {
 	for ( ; *av; av++)
 	    if (EQ(*av, "-"))
-		Expire(SortedInput, QIOfdopen(STDIN, QIO_BUFFER));
-	    else if ((qp = QIOopen(*av, QIO_BUFFER)) == NULL)
+		Expire(SortedInput, QIOfdopen(STDIN));
+	    else if ((qp = QIOopen(*av)) == NULL)
 		(void)fprintf(stderr, "Can't open %s, %s\n",
 			*av, strerror(errno));
 	    else
