@@ -1,4 +1,4 @@
-/*  $Revision: 1.9 $
+/*  $Revision: 1.14 $
 **
 **  Get data from history database.
 */
@@ -17,20 +17,16 @@
 #include "libinn.h"
 #include "dbz.h"
 #include "macros.h"
+#include <syslog.h>  
 
 
 /*
 **  Get the next filename from the history file.
 */
-STATIC BOOL
-GetName(F, buff, Againp)
-    register FILE	*F;
-    register char	*buff;
-    BOOL		*Againp;
+STATIC BOOL GetName(FILE *F, char *buff, BOOL *Againp)
 {
-    static char		SPOOL[] = _PATH_SPOOL;
-    register int	c;
-    register char	*p;
+    int	                c;
+    char	        *p;
 
     /* Skip whitespace before filename. */
     while ((c = getc(F)) == ' ')
@@ -38,34 +34,30 @@ GetName(F, buff, Againp)
     if (c == EOF || c == '\n')
 	return FALSE;
 
-    (void)strcpy(buff, SPOOL);
-    p = &buff[STRLEN(SPOOL)];
+    (void)strcpy(buff, innconf->patharticles);
+    p = &buff[strlen(innconf->patharticles)];
     *p++ = '/';
     *p++ = (char)c;
     while ((c = getc(F)) != EOF && c != ' ' && c != '\n')
 	*p++ = (char)(c == '.' ? '/' : c);
     *p = '\0';
     *Againp = c != EOF && c != '\n';
+    p = &buff[strlen(innconf->patharticles) + 1];
+    if (IsToken(p))
+	memmove(buff, p, strlen(p) + 1);
     return TRUE;
 }
 
 /*
 **  Given a DBZ value, seek to the right spot.
 */
-STATIC BOOL
-HistorySeek(F, p)
-    register FILE	*F;
-    register char	*p;
+STATIC BOOL HistorySeek(FILE *F, OFFSET_T offset)
 {
-    register char	*dest;
-    OFFSET_T		l;
-    register int	c;
-    register int	i;
+    int	c;
+    int	i;
 
-    for (dest = (char *)&l, i = sizeof l; --i >= 0; )
-	*dest++ = *p++;
-    if (fseek(F, l, SEEK_SET) == -1) {
-	(void)fprintf(stderr, "Can't seek to %ld, %s\n", l, strerror(errno));
+    if (fseek(F, offset, SEEK_SET) == -1) {
+	(void)fprintf(stderr, "Can't seek to %ld, %s\n", offset, strerror(errno));
 	return FALSE;
     }
 
@@ -87,20 +79,12 @@ HistorySeek(F, p)
 /*
 **  Print the full line from the history file.
 */
-STATIC void
-FullLine(F, p)
-    register FILE	*F;
-    register char	*p;
+STATIC void FullLine(FILE *F, OFFSET_T offset)
 {
-    register char	*dest;
-    OFFSET_T		l;
-    register int	c;
-    register int	i;
+    int	                c;
 
-    for (dest = (char *)&l, i = sizeof l; --i >= 0; )
-	*dest++ = *p++;
-    if (fseek(F, l, SEEK_SET) == -1) {
-	(void)fprintf(stderr, "Can't seek to %ld, %s\n", l, strerror(errno));
+    if (fseek(F, offset, SEEK_SET) == -1) {
+	(void)fprintf(stderr, "Can't seek to %ld, %s\n", offset, strerror(errno));
 	exit(1);
     }
 
@@ -115,22 +99,24 @@ FullLine(F, p)
 **  don't have.  Or, output list of files for ones we DO have.
 */
 STATIC void
-IhaveSendme(History, What)
-    STRING		History;
-    register char	What;
+IhaveSendme(STRING History, char What)
 {
-    register FILE	*F;
-    register char	*p;
-    register char	*q;
-    datum		key;
-    datum		value;
+    FILE		*F;
+    char		*p;
+    char		*q;
+    HASH		key;
+    OFFSET_T            offset;
     struct stat		Sb;
     BOOL		More;
     char		buff[BUFSIZ];
     char		Name[SPOOLNAMEBUFF];
+#ifndef DO_TAGGED_HASH
+    idxrec		ionevalue;
+    idxrecext		iextvalue;
+#endif
 
     /* Open history. */
-    if (dbminit(History) < 0) {
+    if (!dbzinit(History)) {
 	(void)fprintf(stderr, "Can't open history database, %s\n",
 		strerror(errno));
 	exit(1);
@@ -151,22 +137,40 @@ IhaveSendme(History, What)
 	if (*q != '>')
 	    continue;
 	*++q = '\0';
-	key.dptr = p;
-	key.dsize = q - key.dptr + 1;
-	value = dbzfetch(key);
+
+	key = HashMessageID(p);
+#ifdef	DO_TAGGED_HASH
+	offset = dbzfetch(key);
+#else
+	if (innconf->extendeddbz) {
+	    if (!dbzfetch(key, &iextvalue)) {
+		if (What == 'i')
+		    (void)printf("%s\n", p);
+		continue;
+	    }
+	    offset = iextvalue.offset[HISTOFFSET];
+	} else {
+	    if (!dbzfetch(key, &ionevalue)) {
+		if (What == 'i')
+		    (void)printf("%s\n", p);
+		continue;
+	    }
+	    offset = ionevalue.offset;
+	}
+#endif
 
 	/* Ihave -- say if we want it, and continue. */
 	if (What == 'i') {
-	    if (value.dptr == NULL)
+	    if (offset < 0)
 		(void)printf("%s\n", p);
 	    continue;
 	}
 
 	/* Sendme -- print a filename for the message. */
-	if (value.dptr == NULL)
+	if (offset < 0)
 	    /* Doesn't exist. */
 	    continue;
-	if (HistorySeek(F, value.dptr))
+	if (HistorySeek(F, offset))
 	    while (GetName(F, Name, &More)) {
 		if (stat(Name, &Sb) >= 0) {
 		    (void)printf("%s\n", Name);
@@ -183,7 +187,7 @@ IhaveSendme(History, What)
 **  Print a usage message and exit.
 */
 STATIC NORETURN
-Usage()
+Usage(void)
 {
     (void)fprintf(stderr, "Usage: grephistory [flags] MessageID\n");
     exit(1);
@@ -191,27 +195,43 @@ Usage()
 
 
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(int ac, char *av[])
 {
-    register int	i;
-    register char	*p;
-    register FILE	*F;
+    int			i;
+    FILE		*F;
     STRING		History;
-    datum		key;
-    datum		value;
+    char                *keystr;
+    HASH		key;
+    OFFSET_T		offset;
     struct stat		Sb;
     BOOL		More;
     char		What;
     char		Name[SPOOLNAMEBUFF];
+#ifndef DO_TAGGED_HASH
+    BOOL		val;
+    idxrec		ionevalue;
+    idxrecext		iextvalue;
+    TOKEN		token;
+    int			len;
+    char		*p, *q;
+#endif
+
+    /* First thing, set up logging and our identity. */
+    openlog("grephistory", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);     
 
     /* Set defaults. */
-    History = _PATH_HISTORY;
+    if (ReadInnConf() < 0) exit(1);
+
+    History = COPY(cpcatpath(innconf->pathdb, _PATH_HISTORY));
+
     What = '?';
 
     /* Parse JCL. */
+#ifdef	DO_TAGGED_HASH
     while ((i = getopt(ac, av, "f:eiltnqs")) != EOF)
+#else
+    while ((i = getopt(ac, av, "f:eiloTtnqs")) != EOF)
+#endif
 	switch (i) {
 	default:
 	    Usage();
@@ -226,8 +246,16 @@ main(ac, av)
 	case 'n':
 	case 'q':
 	case 's':
+#ifndef	DO_TAGGED_HASH
+	case 'o':
+	case 'T':
+#endif
 	    if (What != '?') {
+#ifdef	DO_TAGGED_HASH
 		(void)fprintf(stderr, "Only one [eiltnqs] flag allowed.\n");
+#else
+		(void)fprintf(stderr, "Only one [eiloTtnqs] flag allowed.\n");
+#endif
 		exit(1);
 	    }
 	    What = (char)i;
@@ -253,29 +281,48 @@ main(ac, av)
 	Usage();
 
     /* Open the history file, do the lookup. */
-    if (dbminit(History) < 0) {
+    if (!dbzinit(History)) {
 	(void)fprintf(stderr, "Can't open history database, %s\n",
 		strerror(errno));
 	exit(1);
     }
-    key.dptr = av[0];
-    if (*key.dptr != '<') {
-	/* Add optional braces. */
-	key.dptr = NEW(char, 1 + strlen(av[0]) + 1 + 1);
-	(void)sprintf(key.dptr, "<%s>", av[0]);
+    
+    keystr = av[0];
+    if (*av[0] == '[') {
+	key = TextToHash(&av[0][1]);
+    } else {
+	if (*av[0] != '<') {
+	    /* Add optional braces. */
+	    keystr = NEW(char, 1 + strlen(av[0]) + 1 + 1);
+	    (void)sprintf(keystr, "<%s>", av[0]);
+	}
+	key = HashMessageID(keystr);
     }
-    for (p = key.dptr; *p; p++)
-	if (*p == HIS_FIELDSEP || *p == '\n')
-	    *p = HIS_BADCHAR;
-    key.dsize = p - key.dptr + 1;
-    value = dbzfetch(key);
 
     /* Not found. */
-    if (value.dptr == NULL) {
+#ifdef	DO_TAGGED_HASH
+    if ((offset = dbzfetch(key)) < 0) {
 	if (What == 'n')
 	    (void)fprintf(stderr, "Not found.\n");
 	exit(1);
     }
+#else
+    if (innconf->extendeddbz) {
+	if (!dbzfetch(key, &iextvalue)) {
+	    if (What == 'n')
+		(void)fprintf(stderr, "Not found.\n");
+	    exit(1);
+	}
+	offset = iextvalue.offset[HISTOFFSET];
+    } else {
+	if (!dbzfetch(key, &ionevalue)) {
+	    if (What == 'n')
+		(void)fprintf(stderr, "Not found.\n");
+	    exit(1);
+	}
+	offset = ionevalue.offset;
+    }
+#endif
 
     /* Simple case? */
     if (What == 'q')
@@ -283,10 +330,54 @@ main(ac, av)
 
     /* Just give offset into history file */
     if (What == 't') {
-      (void)printf("%lu\n", *(long *) value.dptr);
+      (void)printf("%lu\n", offset);
       exit(0);
     }
 
+#ifndef	DO_TAGGED_HASH
+    /* Just give offset into overview */
+    if (What == 'T') {
+      if (innconf->extendeddbz)
+        (void)printf("%lu (%lu)\n", iextvalue.offset[OVEROFFSET], iextvalue.overindex);
+      else
+        (void)printf("Overview offset is not available.\n");
+      exit(0);
+    }
+
+    /* Get overview */
+    if (What == 'o') {
+      if (innconf->extendeddbz) {
+	if (innconf->overviewmmap)
+	  val = TRUE;
+	else
+	  val = FALSE;
+	if (!OVERsetup(OVER_MMAP, (void *)&val)) {
+	  fprintf(stderr, "Can't setup unified overview mmap\n");
+	  exit(1);    
+	}
+	if (!OVERsetup(OVER_MODE, "r")) {
+	  fprintf(stderr, "Can't setup unified overview mode\n");
+	  exit(1);
+	}
+	if (!OVERinit()) {
+	  fprintf(stderr, "Can't initialize unified overview mode\n");
+	  exit(1);
+	}
+	OVERmaketoken(&token, iextvalue.offset[OVEROFFSET], iextvalue.overindex, iextvalue.overlen);
+	if ((p = OVERretrieve(&token, &len)) == NULL) {
+	  fprintf(stderr, "Can't retrieve overview\n");
+	  exit(1);
+	}
+	q = NEW(char, len + 1);
+	strncpy(q, p, len);
+	(void)printf("%s\n", q);
+      } else {
+	(void)printf("Overview offset is not available.\n");
+      }
+      exit(0);
+    }
+#endif
+
     /* Open the text file, go to the entry. */
     if ((F = fopen(History, "r")) == NULL) {
 	(void)fprintf(stderr, "Can't open \"%s\", %s\n",
@@ -294,14 +385,14 @@ main(ac, av)
 	exit(1);
     }
     if (What == 'l') {
-	FullLine(F, value.dptr);
+	FullLine(F, offset);
 	exit(0);
     }
 
     /* Loop until we find an existing file. */
-    if (HistorySeek(F, value.dptr))
+    if (HistorySeek(F, offset))
 	while (GetName(F, Name, &More)) {
-	    if (stat(Name, &Sb) >= 0) {
+	    if (IsToken(Name) || (stat(Name, &Sb) >= 0)) {
 		(void)printf("%s\n", Name);
 		exit(0);
 	    }
