@@ -3,38 +3,103 @@
 **  Build an active file from either an old copy or by calling find
 **  to get the directory names.
 */
-#include "configdata.h"
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
+#include <ctype.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
 #include <errno.h>
 #include "paths.h"
 #include "libinn.h"
-#include "clibrary.h"
 #include "mydir.h"
 #include "macros.h"
+#include <syslog.h>  
 
 
-STATIC char	ACTIVE[] = _PATH_ACTIVE;
+STATIC char	*ACTIVE = NULL;
+STATIC char	*SPOOL = NULL;
+STATIC char	*OVERVIEWDIR = NULL;
+STATIC BOOL	StorageAPI;
+STATIC BOOL	OVERmmap;
 
+/*
+**  read from overview index
+*/
+STATIC void
+ReadOverviewIndex(char *name, long *lomark, long *himark)
+{
+    char	*p;
+    FILE	*fi;
+    struct stat	sb;
+    char	(*mapped)[][OVERINDEXPACKSIZE];
+    int		count;
+    int		i;
+    OVERINDEX	index;
+    char	packed[OVERINDEXPACKSIZE];
+
+    p = NEW(char, strlen(OVERVIEWDIR) + strlen(name) + strlen(innconf->overviewname) + 32);
+    sprintf(p, "%s/%s/%s.index", OVERVIEWDIR, name, innconf->overviewname);
+    if ((fi = fopen(p, "r")) == NULL) {
+	return;
+    }
+    DISPOSE(p);
+    if (OVERmmap) {
+	if (fstat(fileno(fi), &sb) < 0) {
+	    fclose(fi);
+	    return;
+	}
+	count = sb.st_size / OVERINDEXPACKSIZE;
+	if (count == 0) {
+	    fclose(fi);
+	    return;
+	}
+	if ((mapped = (char (*)[][OVERINDEXPACKSIZE])mmap((MMAP_PTR)0, count * OVERINDEXPACKSIZE, 
+	    PROT_READ, MAP__ARG, fileno(fi), 0)) == (char (*)[][OVERINDEXPACKSIZE])-1) {
+	    fclose(fi);
+	    return;
+	}
+	fclose(fi);
+	for (i = 0; i < count; i++) {
+	    UnpackOverIndex((*mapped)[i], &index);
+	    if (index.artnum < *lomark)
+		*lomark = index.artnum;
+	    if (index.artnum > *himark)
+		*himark = index.artnum;
+	}
+	(void)munmap((MMAP_PTR)mapped, count * OVERINDEXPACKSIZE);
+    } else {
+	while (fread(&packed, OVERINDEXPACKSIZE, 1, fi) == 1) {
+	    UnpackOverIndex(packed, &index);
+	    if (index.artnum < *lomark)
+		*lomark = index.artnum;
+	    if (index.artnum > *himark)
+		*himark = index.artnum;
+	}
+	fclose(fi);
+    }
+}
 
 /*
 **  Given an newsgroup name, write the active file entry.
 */
 STATIC BOOL
-MakeEntry(name, rest, oldhimark, oldlomark, ComputeMarks)
-    char		*name;
-    char		*rest;
-    long		oldhimark;
-    long		oldlomark;
-    BOOL		ComputeMarks;
+MakeEntry(char *name, char *rest, long oldhimark, long oldlomark, BOOL ComputeMarks)
 {
-    register long	himark;
-    register long	lomark;
-    register DIR	*dp;
-    register DIRENTRY	*ep;
-    register long	j;
-    register char	*p;
+    long	himark;
+    long	lomark;
+    DIR		*dp;
+    DIRENTRY	*ep;
+    long	j;
+    char	*p;
+    FILE	*fi;
+    struct stat	sb;
+    char	(*mapped)[][OVERINDEXPACKSIZE];
+    int		count;
+    int		i;
+    OVERINDEX	index;
+    char	packed[OVERINDEXPACKSIZE];
 
     /* Turn group name into directory name. */
     for (p = name; *p; p++)
@@ -51,20 +116,23 @@ MakeEntry(name, rest, oldhimark, oldlomark, ComputeMarks)
 	lomark = oldlomark;
     }
 
-
-    if ((dp = opendir(name)) != NULL) {
-	/* Scan through all entries in the directory. */
-	while ((ep = readdir(dp)) != NULL) {
-	    if (!CTYPE(isdigit, ep->d_name[0])
-	     || strspn(ep->d_name, "0123456789") != strlen(ep->d_name)
-	     || (j = atol(ep->d_name)) == 0)
-		continue;
-	    if (lomark == 0 || j < lomark)
-		lomark = j;
-	    if (j > himark)
-		himark = j;
+    if (StorageAPI) {
+	ReadOverviewIndex(name, &lomark, &himark);
+    } else {
+        if ((dp = opendir(name)) != NULL) {
+	    /* Scan through all entries in the directory. */
+	    while ((ep = readdir(dp)) != NULL) {
+	        p = ep->d_name;
+	        if (!CTYPE(isdigit, p[0]) || strspn(p, "0123456789") != strlen(p)
+	         || (j = atol(p)) == 0)
+		    continue;
+	        if (lomark == 0 || j < lomark)
+		    lomark = j;
+	        if (j > himark)
+		    himark = j;
+	    }
+	    (void)closedir(dp);
 	}
-	(void)closedir(dp);
     }
     if (lomark == 0 || lomark - 1 > himark)
 	lomark = himark + 1;
@@ -78,7 +146,7 @@ MakeEntry(name, rest, oldhimark, oldlomark, ComputeMarks)
     for (p = name; *p; p++)
 	if (*p == '/')
 	    *p = '.';
-    if (printf("%s %010.10ld %010.10ld %s\n",
+    if (printf("%s %010ld %010ld %s\n",
 	    name, himark, lomark, rest) == EOF
      || fflush(stdout) == EOF
      || ferror(stdout)) {
@@ -94,11 +162,9 @@ MakeEntry(name, rest, oldhimark, oldlomark, ComputeMarks)
 **  See if a line is too long to be a newsgroup name, return TRUE if so.
 */
 STATIC BOOL
-TooLong(buff, i)
-    char		*buff;
-    int			i;
+TooLong(char *buff, int i)
 {
-    register char	*p;
+    char	*p;
 
     if ((p = strchr(buff, '\n')) == NULL) {
 	(void)fprintf(stderr, "Line %d is too long:  \"%.40s\"...\n",
@@ -119,19 +185,18 @@ TooLong(buff, i)
 **  Renumber the active file based on the old active file.
 */
 STATIC BOOL
-RebuildFromOld(ComputeMarks)
-    BOOL		ComputeMarks;
+RebuildFromOld(BOOL ComputeMarks)
 {
-    register FILE	*F;
-    register char	*p;
-    register int	i;
-    register BOOL	Ok;
-    char		buff[BUFSIZ];
-    STRING		rest;
-    long		lomark;
-    long		himark;
-    char		*save1;
-    char		*save2;
+    FILE	*F;
+    char	*p;
+    int		i;
+    BOOL	Ok;
+    char	buff[BUFSIZ];
+    STRING	rest;
+    long	lomark;
+    long	himark;
+    char	*save1;
+    char	*save2;
 
     /* Open the file. */
     if ((F = fopen(ACTIVE, "r")) == NULL) {
@@ -168,7 +233,7 @@ RebuildFromOld(ComputeMarks)
 	    }
 	}
 
-	if (!MakeEntry(buff, rest, himark, lomark, ComputeMarks)) {
+	if (!MakeEntry(buff, (char *)rest, himark, lomark, ComputeMarks)) {
 	    Ok = FALSE;
 	    break;
 	}
@@ -180,20 +245,20 @@ RebuildFromOld(ComputeMarks)
 
 
 STATIC BOOL
-RebuildFromFind()
+RebuildFromFind(void)
 {
-    register int	i;
-    register char	*p;
-    register FILE	*F;
-    register BOOL	Ok;
+    int	i;
+    char	*p;
+    FILE	*F;
+    BOOL	Ok;
     char		buff[BUFSIZ];
 
     /* Start getting a list of the directories. */
-#if	defined(DO_HAVE_SYMLINKS)
+#if	defined(HAVE_SYMLINK)
     F = popen("exec find . -follow -type d -print", "r");
 #else
     F = popen("exec find . -type d -print", "r");
-#endif	/* defined(DO_HAVE_SYMLINKS) */
+#endif	/* defined(HAVE_SYMLINK) */
     if (F == NULL) {
 	(void)fprintf(stderr, "Can't start find, %s\n", strerror(errno));
 	exit(1);
@@ -230,7 +295,7 @@ RebuildFromFind()
 **  Print a usage message and exit.
 */
 STATIC NORETURN
-Usage()
+Usage(void)
 {
     (void)fprintf(stderr, "Usage: makeactive [-o [-m] ] >output\n");
     exit(1);
@@ -239,19 +304,26 @@ Usage()
 
 
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(int ac, char *av[])
 {
     BOOL		Ok;
-    register int	i;
+    int	i;
     BOOL		OldFile;
     BOOL		ComputeMarks;
 
+    /* First thing, set up logging and our identity. */
+    openlog("makeactive", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);     
+
     /* Set defaults. */
     OldFile = FALSE;
     ComputeMarks = FALSE;
 
+    if (ReadInnConf() < 0) exit(1);
+
+    (void)umask(NEWSUMASK);
+
+    ACTIVE = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVE));
+
     /* Parse JCL. */
     while ((i = getopt(ac, av, "mo")) != EOF)
 	switch (i) {
@@ -270,8 +342,12 @@ main(ac, av)
     if (ac || (ComputeMarks && !OldFile))
 	Usage();
 
+    StorageAPI = innconf->storageapi;
+    OVERmmap = innconf->overviewmmap;
     /* Go to where the articles are. */
-    if (chdir(_PATH_SPOOL) < 0) {
+    SPOOL = innconf->patharticles;
+    OVERVIEWDIR = innconf->pathoverview;
+    if (chdir(StorageAPI ? OVERVIEWDIR : SPOOL) < 0) {
 	(void)fprintf(stderr, "Can't change to spool directory, %s\n",
 		strerror(errno));
 	exit(1);
