@@ -1,4 +1,4 @@
-/*  $Revision: 1.31 $
+/*  $Revision: 1.14 $
 **
 **  A front-end for InterNetNews.
 **  Read UUCP batches and offer them up NNTP-style.  Because we may end
@@ -19,7 +19,7 @@
 #include <errno.h>
 #include "nntp.h"
 #include "paths.h"
-#include "logging.h"
+#include <syslog.h> 
 #include "libinn.h"
 #include "mydir.h"
 #include "macros.h"
@@ -34,8 +34,7 @@ typedef struct _HEADER {
 STATIC BOOL	Verbose;
 STATIC char	*InputFile = "stdin";
 STATIC char	*UUCPHost;
-STATIC char	SPOOLNEWS[] = _PATH_SPOOLNEWS;
-STATIC char	SPOOLTEMP[] = _PATH_SPOOLTEMP;
+STATIC char	*PATHBADNEWS = NULL;
 STATIC FILE	*FromServer;
 STATIC FILE	*ToServer;
 STATIC char	UNPACK[] = "compress";
@@ -86,6 +85,7 @@ StartChild(fd, path, argv)
     int		pan[2];
     int		i;
     PID_T	pid;
+    char	*p;
 
     /* Create a pipe. */
     if (pipe(pan) < 0) {
@@ -95,7 +95,7 @@ StartChild(fd, path, argv)
 
     /* Get a child. */
     for (i = 0; (pid = FORK()) < 0; i++) {
-	if (i == MAX_FORKS) {
+	if (i == innconf->maxforks) {
 	    syslog(L_ERROR, "cant fork %s %m -- spooling", path);
 	    return -1;
 	}
@@ -163,9 +163,7 @@ WaitForChildren(i)
 /*
 **  Clean up the NNTP escapes from a line.
 */
-STATIC char *
-REMclean(buff)
-    char	*buff;
+STATIC char *REMclean(char *buff)
 {
     char	*p;
 
@@ -182,11 +180,7 @@ REMclean(buff)
 /*
 **  Write an article to the rejected directory.
 */
-STATIC void
-Reject(article, reason, arg)
-    char	*article;
-    char	*reason;
-    char	*arg;
+STATIC void Reject(const char *article, const char *reason, const char *arg)
 {
 #if	defined(DO_RNEWS_SAVE_BAD)
     char	buff[SMBUF];
@@ -202,7 +196,7 @@ Reject(article, reason, arg)
     }
 
 #if	defined(DO_RNEWS_SAVE_BAD)
-    TempName(_PATH_BADNEWS, buff);
+    TempName(PATHBADNEWS, buff);
     if ((F = fopen(buff, "w")) == NULL) {
 	syslog(L_ERROR, "cant fopen %s %m", buff);
 	return;
@@ -221,12 +215,10 @@ Reject(article, reason, arg)
 **  whole batch needs to be saved (such as when the server goes down or if
 **  the file is corrupted).
 */
-STATIC BOOL
-Process(article)
-    char		*article;
+STATIC BOOL Process(char *article)
 {
-    register HEADER	*hp;
-    register char	*p;
+    HEADER	        *hp;
+    char	        *p;
     char		*id;
     char		buff[SMBUF];
 #if	defined(FILE_RNEWS_LOG_DUPS)
@@ -242,7 +234,7 @@ Process(article)
 
     /* Make sure that all the headers are there, note the ID. */
     for (hp = RequiredHeaders; hp < ENDOF(RequiredHeaders); hp++) {
-	if ((p = HeaderFind(article, hp->Name, hp->size)) == NULL) {
+	if ((p = (char *)HeaderFindMem(article, strlen(article), hp->Name, hp->size)) == NULL) {
 	    Reject(article, "bad_article missing %s", hp->Name);
 	    return FALSE;
 	}
@@ -296,7 +288,7 @@ Process(article)
 #if	defined(FILE_RNEWS_LOG_DUPS)
 	if ((F = fopen(_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
 	    *p = '\0';
-	    (void)fprintf(F, "duplicate %s %s", id, path);
+	    (void)fprintf(F, "duplicate %s %s\n", id, path);
 	    (void)fclose(F);
 	}
 #endif	/* defined(FILE_RNEWS_LOG_DUPS) */
@@ -480,7 +472,7 @@ UnpackOne(fdp, countp)
     int		*countp;
 {
 #if	defined(DO_RNEWSPROGS)
-    char	path[sizeof _PATH_RNEWSPROGS + 1 + SMBUF + 1];
+    char	path[(SMBUF * 2) + 1];
     char	*p;
 #endif	/* defined(DO_RNEWSPROGS) */
     char	buff[SMBUF];
@@ -490,6 +482,7 @@ UnpackOne(fdp, countp)
     int		gzip = 0;
     BOOL	HadCount;
     BOOL	SawCunbatch;
+    int		len;
 
     *countp = 0;
     for (SawCunbatch = FALSE, HadCount = FALSE; ; ) {
@@ -578,8 +571,15 @@ UnpackOne(fdp, countp)
 	    p++;
 	else
 	    p = &buff[3];
-	(void)sprintf(path, "%s/%s", _PATH_RNEWSPROGS, p);
-	for (p = &path[sizeof _PATH_RNEWSPROGS]; *p; p++)
+	if (strchr(_PATH_RNEWSPROGS, '/') == NULL) {
+	    (void)sprintf(path, "%s/%s/%s", innconf->pathbin,
+					_PATH_RNEWSPROGS, p);
+	    len = strlen(innconf->pathbin) + 1 + sizeof _PATH_RNEWSPROGS;
+	} else {
+	    (void)sprintf(path, "%s/%s", _PATH_RNEWSPROGS, p);
+	    len = sizeof _PATH_RNEWSPROGS;
+	}
+	for (p = &path[len]; *p; p++)
 	    if (ISWHITE(*p)) {
 		*p = '\0';
 		break;
@@ -616,9 +616,9 @@ Unspool()
     int			i;
 
     /* Go to the spool directory, get ready to scan it. */
-    if (chdir(SPOOLNEWS) < 0) {
-	xperror(SPOOLNEWS);
-	syslog(L_FATAL, "cant cd %s %m", SPOOLNEWS);
+    if (chdir(innconf->pathincoming) < 0) {
+	xperror(innconf->pathincoming);
+	syslog(L_FATAL, "cant cd %s %m", innconf->pathincoming);
 	exit(1);
     }
     if ((dp = opendir(".")) == NULL) {
@@ -662,7 +662,7 @@ Unspool()
 	WaitForChildren(i);
 
 	if (!ok) {
-	    TempName(_PATH_BADNEWS, buff);
+	    TempName(PATHBADNEWS, buff);
 	    (void)fprintf(stderr, "Unspooling failed saving to %s\n", buff);
 	    syslog(L_ERROR, "cant unspool saving to %s", buff);
 	    if (rename(InputFile, buff) < 0) {
@@ -702,7 +702,7 @@ Spool(fd, mode)
 
     if(mode == 'N')
 	exit(9);
-    TempName(SPOOLTEMP, temp);
+    TempName(innconf->pathtmp, temp);
     (void)umask(0);
     if ((spfd = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) < 0) {
 	syslog(L_FATAL, "cant open %s %m", temp);
@@ -733,7 +733,7 @@ Spool(fd, mode)
     }
 
     /* Move temp file into the spool area, and exit appropriately. */
-    TempName(SPOOLNEWS, buff);
+    TempName(innconf->pathincoming, buff);
     if (rename(temp, buff) < 0) {
 	syslog(L_FATAL, "cant rename %s to %s %m", temp, buff);
 	status++;
@@ -747,18 +747,15 @@ Spool(fd, mode)
 **  Try to read the password file and open a connection to a remote
 **  NNTP server.
 */
-STATIC BOOL
-OpenRemote(server, buff)
-    char	*server;
-    char	*buff;
+STATIC BOOL OpenRemote(char *server, int port, char *buff)
 {
     int		i;
 
     /* Open the remote connection. */
     if (server)
-	i = NNTPconnect(server, &FromServer, &ToServer, buff);
+	i = NNTPconnect(server, port, &FromServer, &ToServer, buff);
     else
-	i = NNTPremoteopen(&FromServer, &ToServer, buff);
+	i = NNTPremoteopen(port, &FromServer, &ToServer, buff);
     if (i < 0)
 	return FALSE;
 
@@ -804,20 +801,17 @@ Usage()
 }
 
 
-int
-main(ac, av)
-    int		ac;
-    char	*av[];
+int main(int ac, char *av[])
 {
     int		fd;
     int		i;
     int		mode;
     char	buff[SMBUF];
-    char	*Slave;
+    int         port = NNTP_PORT;
 
     /* First thing, set up logging and our identity. */
     openlog("rnews", L_OPENLOG_FLAGS, LOG_INN_PROG);
-#if !defined(__FreeBSD__) && !defined(__bsdi__)
+#if !defined(__FreeBSD__) && !defined(__bsdi__) && !defined (__NetBSD__)
     if (setgid(getegid()) < 0) {
 	syslog(L_FATAL, "cant setgid to %d %m", getegid());
 	exit(1);
@@ -827,14 +821,16 @@ main(ac, av)
 	exit(1);
     }
 #endif
+     if (ReadInnConf() < 0) exit(1);
      UUCPHost = getenv(_ENV_UUCPHOST);
+     PATHBADNEWS = cpcatpath(innconf->pathincoming, _PATH_BADNEWS);
+     
     (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
     fd = STDIN;
     mode = '\0';
-    Slave = NULL;
-    while ((i = getopt(ac, av, "h:S:NUv")) != EOF)
+    while ((i = getopt(ac, av, "h:P:NUv")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -842,13 +838,13 @@ main(ac, av)
 	case 'h':
 	    UUCPHost = *optarg ? optarg : NULL;
 	    break;
-	case 'S':
-	    Slave = optarg;
-	    break;
 	case 'N':
 	case 'U':
 	    mode = i;
 	    break;
+	case 'P':
+	    port = atoi(optarg);
+	    break;
 	case 'v':
 	    Verbose = TRUE;
 	    break;
@@ -876,8 +872,8 @@ main(ac, av)
     }
 
     /* Open the link to the server. */
-    if (Slave) {
-	if (!OpenRemote(Slave, buff))
+    if (innconf->nnrpdposthost != NULL) {
+	if (!OpenRemote(innconf->nnrpdposthost, port, buff))
 	    CantConnect(buff, mode, fd);
     }
     else {
@@ -886,11 +882,11 @@ main(ac, av)
 	    /* If server rejected us, no point in continuing. */
 	    if (buff[0])
 		CantConnect(buff, mode, fd);
-	    if (!OpenRemote((char *)NULL, buff))
+	    if (!OpenRemote((char *)NULL, port, buff))
 		CantConnect(buff, mode, fd);
 	}
 #else
-	if (!OpenRemote((char *)NULL, buff))
+	if (!OpenRemote((char *)NULL, port, buff))
 	    CantConnect(buff, mode, fd);
 #endif	/* defined(DO_RNEWSLOCALCONNECT) */
     }
