@@ -1,4 +1,4 @@
-/*  $Revision: 1.53 $
+/*  $Id: cc.c,v 1.40 1998/05/26 14:41:28 kondou Exp $
 **
 **  Routines for the control channel.  Create a Unix-domain datagram socket
 **  that processes on the local server send messages to.  The control
@@ -11,13 +11,14 @@
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #include "innd.h"
 #include "inndcomm.h"
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
 #include <sys/un.h>
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	
 
 
 /*
@@ -49,7 +50,6 @@ STATIC STRING	CCnewgroup();
 STATIC STRING	CCparam();
 STATIC STRING	CCpause();
 STATIC STRING	CCreaders();
-STATIC STRING	CCrefile();
 STATIC STRING	CCreject();
 STATIC STRING	CCreload();
 STATIC STRING	CCrenumber();
@@ -57,7 +57,9 @@ STATIC STRING	CCrmgroup();
 STATIC STRING	CCsend();
 STATIC STRING	CCshutdown();
 STATIC STRING	CCsignal();
+STATIC STRING	CCstatus();
 STATIC STRING	CCthrottle();
+STATIC STRING	CCtimer();
 STATIC STRING	CCtrace();
 STATIC STRING	CCxabort();
 STATIC STRING	CCxexec();
@@ -67,8 +69,9 @@ STATIC STRING	CCfilter();
 #if defined(DO_PERL)
 STATIC STRING	CCperl();
 #endif /* defined(DO_PERL) */
+STATIC STRING	CClowmark();
 
-STATIC char		CCpath[] = _PATH_NEWSCONTROL;
+STATIC char		*CCpath = NULL;
 STATIC char		**CCargv;
 STATIC char		CCnosite[] = "1 No such site";
 STATIC char		CCwrongtype[] = "1 Wrong site type";
@@ -106,7 +109,6 @@ STATIC CCDISPATCH	CCcommands[] = {
     {	SC_PARAM,	2, CCparam	},
     {	SC_PAUSE,	1, CCpause	},
     {	SC_READERS,	2, CCreaders	},
-    {	SC_REFILE,	2, CCrefile	},
     {	SC_REJECT,	1, CCreject	},
     {	SC_RENUMBER,	1, CCrenumber	},
     {	SC_RELOAD,	2, CCreload	},
@@ -115,9 +117,12 @@ STATIC CCDISPATCH	CCcommands[] = {
     {	SC_SEND,	2, CCsend	},
     {	SC_SHUTDOWN,	1, CCshutdown	},
     {	SC_SIGNAL,	2, CCsignal	},
+    {	SC_STATUS,	1, CCstatus	},
     {	SC_THROTTLE,	1, CCthrottle	},
+    {   SC_TIMER,       1, CCtimer      },
     {	SC_TRACE,	2, CCtrace	},
     {	SC_XABORT,	1, CCxabort	},
+    {	SC_LOWMARK,	1, CClowmark	},
     {	SC_XEXEC,	1, CCxexec	}
 };
 
@@ -136,8 +141,12 @@ CCcopyargv(av)
 	continue;
 
     /* Get the vector, copy each element. */
-    for (v = CCargv = NEW(char*, i + 1); *av; av++)
+    for (v = CCargv = NEW(char*, i + 1); *av; av++) {
+	/* not to renumber */
+	if (strncmp(*av, "-r", 2) == 0)
+	    continue;
 	*v++ = COPY(*av);
+    }
     *v = NULL;
 }
 
@@ -228,12 +237,58 @@ CCaddhist(av)
     ARTDATA		Data;
     STRING		p;
     BOOL		ok;
+    HASH                hash;
+    int			i;
+    TOKEN		token;
+    OFFSET_T		offset;
+    unsigned char	index;
+    unsigned short	len;
 
-    /* Check the fields. */
-    if ((p = CCgetid(av[0], &Data.MessageID)) != NULL)
-	return p;
-    if (HIShavearticle(Data.MessageID))
+    /* Check to see if we were passed a hash first. */
+    i = strlen(av[0]);
+    if (av[0][0]=='[' && av[0][i-1] == ']') {
+	if (i != ((sizeof(HASH) * 2) + 2))
+	    return "1 Bad Hash";
+	index = OVER_NONE;
+	offset = 0;
+	len = 0;
+	OVERsetoffset(&token, &offset, &index, &len);
+	if (av[4] != NULL && *av[4] != '\0') {
+	    if (!IsToken(av[4]))
+		return "1 Bad Token";
+	    else
+		token = TextToToken(av[4]);
+	}
+        hash = TextToHash(&av[0][1]);
+	/* Put something bogus in here.  This should never be referred
+	   to unless someone tries to add a [msgidhash].... history
+	   entry to an innd set with 'storageapi' off, which isn't a
+	   very sensible thing to do. */
+	Data.MessageID = "<bogus@messsageid>";
+    } else {
+        /* Try to take what we were given as a <messageid> */
+        if ((p = CCgetid(av[0], &Data.MessageID)) != NULL)
+	  return p;
+	hash = HashMessageID(Data.MessageID);
+    }
+
+    /* If paused, don't try to use the history database since expire may be
+       running */
+    if (Mode == OMpaused)
+	return "1 Server paused";
+
+    /* If throttled by admin, briefly open the history database. */
+    if (Mode != OMrunning) {
+	if (ThrottledbyIOError)
+	    return "1 Server throttled";
+	HISsetup();
+    }
+
+    if (HIShavearticle(hash)) {
+	if (Mode != OMrunning) HISclose();
 	return "1 Duplicate";
+    }
+    if (Mode != OMrunning) HISclose();
     if (strspn(av[1], DIGITS) != strlen(av[1]))
 	return "1 Bad arrival date";
     Data.Arrived = atol(av[1]);
@@ -245,11 +300,11 @@ CCaddhist(av)
     Data.Posted = atol(av[3]);
 
     if (Mode == OMrunning)
-	ok = HISwrite(&Data, av[4]);
+	ok = HISwrite(&Data, hash, av[4], &token);
     else {
 	/* Possible race condition, but documented in ctlinnd manpage. */
 	HISsetup();
-	ok = HISwrite(&Data, av[4]);
+	ok = HISwrite(&Data, hash, av[4], &token);
 	HISclose();
     }
     return ok ? NULL : "1 Write failed";
@@ -410,22 +465,28 @@ CCcancel(av)
     ARTDATA	Data;
     STRING	p;
 
-    Data.Posted = Now.time;
+    Data.Posted = Data.Arrived = Now.time;
     Data.Expires = 0;
     Data.Feedsite = "?";
     if ((p = CCgetid(av[0], &Data.MessageID)) != NULL)
 	return p;
+
     if (Mode == OMrunning)
 	ARTcancel(&Data, Data.MessageID, TRUE);
     else {
+	/* If paused, don't try to use the history database since expire may be
+	   running */
+	if (Mode == OMpaused)
+	    return "1 Server paused";
+	if (ThrottledbyIOError)
+	    return "1 Server throttled";
 	/* Possible race condition, but documented in ctlinnd manpage. */
 	HISsetup();
 	ARTcancel(&Data, Data.MessageID, TRUE);
 	HISclose();
     }
-#if	defined(DO_LOG_CANCEL_COMMANDS)
-    syslog(L_NOTICE, "%s cancelled %s", LogName, Data.MessageID);
-#endif	/* defined(DO_LOG_CANCEL_COMMANDS) */
+    if (innconf->logcancelcomm)
+	syslog(L_NOTICE, "%s cancelled %s", LogName, Data.MessageID);
     return NULL;
 }
 
@@ -461,10 +522,13 @@ CCcheckfile(av)
     if (errors == 0)
 	return NULL;
 
-    if (CCreply.Size == 0) {
+    if (CCreply.Data == NULL) {
 	/* If we got the "-s" flag, then CCsetup hasn't been called yet. */
 	CCreply.Size = SMBUF;
 	CCreply.Data = NEW(char, CCreply.Size);
+    } else if (CCreply.Size < SMBUF) {
+	CCreply.Size = SMBUF;
+	RENEW(CCreply.Data, char, CCreply.Size);
     }
 
     (void)sprintf(CCreply.Data, "1 Found %d errors -- see syslog", errors);
@@ -563,6 +627,10 @@ CCfilter(av)
 
 #if defined(DO_PERL)
 
+#if defined (DO_NEED_BOOL)
+typedef enum { false = 0, true = 1 } bool;
+#endif
+
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
@@ -676,8 +744,9 @@ CCgo(av)
     DISPOSE(ModeReason);
     ModeReason = NULL;
     Mode = OMrunning;
+    ThrottledbyIOError = FALSE;
 
-    if (NNRPReason && NNRPFollows) {
+    if (NNRPReason && innconf->allowreaders) {
 	av[0] = YES;
 	av[1] = p;
 	(void)CCreaders(av);
@@ -685,6 +754,21 @@ CCgo(av)
     if (ErrorCount < 0)
 	ErrorCount = IO_ERROR_COUNT;
     HISsetup();
+    if (innconf->storageapi) {
+	int fdcountold = Overfdcount;
+	if (!OVERinit()) {
+	    syslog(L_FATAL, "%s cant initialize the unified overview");
+	    exit(1);
+	}
+	if ((Overfdcount = OVERgetnum()) < 0) {
+            syslog(L_FATAL, "%s cant get config for the unified overview");
+            exit(1);
+	}
+	if (fdcountold != Overfdcount) {
+	    MaxOutgoing += Overfdcount - fdcountold;
+            syslog(L_NOTICE, "%s outgoing %d", LogName, MaxOutgoing);
+	}
+    }
     syslog(L_NOTICE, "%s running", LogName);
     if (ICDneedsetup)
 	ICDsetup(TRUE);
@@ -792,11 +876,11 @@ CCmode(av)
 	i++;
     *p++ = '\n';
     (void)sprintf(p, "Parameters c %ld i %d (%d) l %ld o %d t %ld H %d T %d X %d %s %s",
-	    (long)Cutoff / (24L * 60L * 60L),
-	    MaxIncoming, i,
-	    LargestArticle, MaxOutgoing, (long)TimeOut.tv_sec,
+	    (long)innconf->artcutoff / (24L * 60L * 60L),
+	    innconf->maxconnections, i,
+	    innconf->maxartsize, MaxOutgoing, (long)TimeOut.tv_sec,
 	    RemoteLimit, RemoteTotal, (int) RemoteTimer,
-	    AmSlave ? "slave" : "normal",
+	    innconf->xrefslave ? "slave" : "normal",
 	    AnyIncoming ? "any" : "specified");
     p += strlen(p);
 
@@ -812,7 +896,7 @@ CCmode(av)
     /* Newsreaders. */
     *p++ = '\n';
     p += strlen(strcpy(p, "Readers "));
-    if (NNRPFollows)
+    if (innconf->allowreaders)
 	p += strlen(strcpy(p, "follow "));
     else
 	p += strlen(strcpy(p, "separate "));
@@ -905,7 +989,7 @@ STATIC STRING
 CCnewgroup(av)
     char		*av[];
 {
-    static char		TIMES[] = _PATH_ACTIVETIMES;
+    static char		*TIMES = NULL;
     static char		WHEN[] = "updating active.times";
     register int	fd;
     register char	*p;
@@ -913,9 +997,12 @@ CCnewgroup(av)
     char		*Name;
     char		*Rest;
     STRING		who;
-    char		buff[SMBUF];
+    char		*buff;
     int			oerrno;
 
+    if (TIMES == NULL)
+	TIMES = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVETIMES));
+
     Name = av[0];
     if (Name[0] == '.' || strspn(Name, "0123456789") == strlen(Name))
 	return "1 Illegal newsgroup name";
@@ -939,23 +1026,33 @@ CCnewgroup(av)
     if ((ngp = NGfind(Name)) != NULL)
 	return CCdochange(ngp, Rest);
 
+    if (Mode == OMthrottled && ThrottledbyIOError)
+	return "1 server throttled";
+
     /* Update the log of groups created.  Don't use stdio because SunOS
      * 4.1 has broken libc which can't handle fd's greater than 127. */
     if ((fd = open(TIMES, O_WRONLY | O_APPEND | O_CREAT, 0664)) < 0) {
 	oerrno = errno;
-	syslog(L_ERROR, "%s cant fopen %s %m", LogName, TIMES);
+	syslog(L_ERROR, "%s cant open %s %m", LogName, TIMES);
 	IOError(WHEN, oerrno);
     }
     else {
 	who = av[2];
 	if (*who == '\0')
 	    who = NEWSMASTER;
+
+	/* %s + ' ' + %ld + ' ' + %s + '\n' + terminator */
+	buff = NEW(char, strlen(Name) + 1 + 20 + 1 + strlen(who) + 1 + 1);
+
 	(void)sprintf(buff, "%s %ld %s\n", Name, Now.time, who);
 	if (xwrite(fd, buff, strlen(buff)) < 0) {
 	    oerrno = errno;
 	    syslog(L_ERROR, "%s cant write %s %m", LogName, TIMES);
 	    IOError(WHEN, oerrno);
 	}
+
+	DISPOSE(buff);
+	
 	if (close(fd) < 0) {
 	    oerrno = errno;
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, TIMES);
@@ -975,7 +1072,7 @@ CCnewgroup(av)
 	for (p = Name; *p; p++)
 	    if (*p == '.')
 		*p = '/';
-	if (!MakeSpoolDirectory(Name))
+	if (!innconf->storageapi && !MakeDirectory(Name, TRUE))
 	    syslog(L_NOTICE, "%s cant mkdir %s %m", LogName, Name);
     }
     return NULL;
@@ -1015,6 +1112,7 @@ CCparam(av)
 {
     static char	BADVAL[] = "1 Bad value";
     char	*p;
+    int		temp;
 
     p = av[1];
     switch (av[0][0]) {
@@ -1025,19 +1123,19 @@ CCparam(av)
 	    return BADVAL;
 	break;
     case 'c':
-	Cutoff = atoi(p) * 24 * 60 * 60;
-	syslog(L_NOTICE, "%s changed -c %d", LogName, Cutoff);
+	innconf->artcutoff = atoi(p) * 24 * 60 * 60;
+	syslog(L_NOTICE, "%s changed -c %d", LogName, innconf->artcutoff);
 	break;
     case 'i':
-	MaxIncoming = atoi(p);
-	syslog(L_NOTICE, "%s changed -i %d", LogName, MaxIncoming);
+	innconf->maxconnections = atoi(p);
+	syslog(L_NOTICE, "%s changed -i %d", LogName, innconf->maxconnections);
 	break;
     case 'l':
-	LargestArticle = atol(p);
-	syslog(L_NOTICE, "%s changed -l %ld", LogName, LargestArticle);
+	innconf->maxartsize = atol(p);
+	syslog(L_NOTICE, "%s changed -l %ld", LogName, innconf->maxartsize);
 	break;
     case 'n':
-	if (!CCparsebool('n', &NNRPFollows, *p))
+	if (!CCparsebool('n', &innconf->allowreaders, *p))
 	    return BADVAL;
 	break;
     case 'o':
@@ -1053,10 +1151,15 @@ CCparam(av)
 	syslog(L_NOTICE, "%s changed -H %d", LogName, RemoteLimit);
 	break;
     case 'T':
-	RemoteTotal = atoi(p);
-	if (RemoteTotal > REMOTETABLESIZE)
-	    RemoteTotal = REMOTETABLESIZE;
-	syslog(L_NOTICE, "%s changed -T %d", LogName, RemoteTotal);
+        temp = atoi(p);
+	if (temp > REMOTETABLESIZE) {
+	    syslog(L_NOTICE, "%s -T must be lower than %d",
+		   LogName, REMOTETABLESIZE+1);
+	    temp = REMOTETABLESIZE;
+	}
+	syslog(L_NOTICE, "%s changed -T from %d to %d",
+	       LogName, RemoteTotal, temp);
+	RemoteTotal = temp;
 	break;
     case 'X':
 	RemoteTimer = (time_t) atoi(p);
@@ -1099,11 +1202,13 @@ CCblock(NewMode, reason)
 
     ICDwrite();
     HISclose();
+    if (innconf->storageapi)
+	OVERshutdown();
     Mode = NewMode;
     if (ModeReason)
 	DISPOSE(ModeReason);
     ModeReason = COPY(reason);
-    if (NNRPReason == NULL && NNRPFollows) {
+    if (NNRPReason == NULL && innconf->allowreaders) {
 	av[0] = NO;
 	av[1] = ModeReason;
 	(void)CCreaders(av);
@@ -1176,21 +1281,17 @@ STATIC STRING
 CCxexec(av)
     char	*av[];
 {
-    static char	INND[] = _PATH_INND;
-    static char	INNDSTART[] = _PATH_INNDSTART;
+    char	*inndstart;
     char	*p;
 
     if (CCargv == NULL)
 	return "1 no argv!";
     
+    inndstart = COPY(cpcatpath(innconf->pathbin, "inndstart"));
     /* Get the pathname. */
     p = av[0];
-    if (*p == '\0')
-	CCargv[0] = AmRoot ? INND : INNDSTART;
-    else if (EQ(p, "innd"))
-	CCargv[0] = INND;
-    else if (EQ(p, "inndstart"))
-	CCargv[0] = INNDSTART;
+    if (*p == '\0' || EQ(p, "innd") || EQ(p, "inndstart"))
+	CCargv[0] = inndstart;
     else
 	return "1 Bad value";
 
@@ -1200,30 +1301,9 @@ CCxexec(av)
     syslog(L_FATAL, "%s cant execv %s %m", LogName, CCargv[0]);
     _exit(1);
     /* NOTREACHED */
+    return "1 Exit failed";
 }
 
-
-/*
-**  Refile an article.
-*/
-STATIC STRING
-CCrefile(av)
-    char	*av[];
-{
-    char	*head;
-
-    /* xxx multiple groups? */
-    if (NGfind(av[1]) == NULL)
-	return CCnogroup;
-
-    head = ARTreadheader(av[0]);
-    if (head == NULL)
-	return "1 No such article";
-
-    return "1 Not yet implemented";
-}
-
-
 /*
 **  Reject remote readers.
 */
@@ -1257,8 +1337,25 @@ CCreload(av)
     if (*p == '\0' || EQ(p, "all")) {
 	SITEflushall(FALSE);
 	HISclose();
+	if (innconf->storageapi)
+	    OVERshutdown();
 	RCreadlist();
 	HISsetup();
+	if (innconf->storageapi) {  
+	    int fdcountold = Overfdcount;
+	    if (!OVERinit()) {
+		syslog(L_FATAL, "%s cant initialize the unified overview");
+		exit(1);
+	    } 
+	    if ((Overfdcount = OVERgetnum()) < 0) {
+		syslog(L_FATAL, "%s cant get config for the unified overview");
+		exit(1);
+	    }
+	    if (fdcountold != Overfdcount) {
+		MaxOutgoing += Overfdcount - fdcountold;
+		syslog(L_NOTICE, "%s outgoing %d", LogName, MaxOutgoing);
+	    }   
+	}
 	ICDwrite();
 	ICDsetup(TRUE);
 	if (!ARTreadschema())
@@ -1280,12 +1377,54 @@ CCreload(av)
 	HISclose();
 	HISsetup();
     }
-    else if (EQ(p, "hosts.nntp"))
+    else if (EQ(p, "incoming.conf"))
 	RCreadlist();
     else if (EQ(p, "overview.fmt")) {
 	if (!ARTreadschema())
 	    return BADSCHEMA;
     }
+    else if (EQ(p, "overview.ctl")) {
+	if (innconf->storageapi) {
+	    int fdcountold = Overfdcount;
+	    OVERshutdown();
+	    if (!OVERinit()) {
+		syslog(L_FATAL, "%s cant initialize the unified overview");
+		exit(1);
+	    } 
+	    if ((Overfdcount = OVERgetnum()) < 0) {
+		syslog(L_FATAL, "%s cant get config for the unified overview");
+		exit(1);
+	    }
+	    if (fdcountold != Overfdcount) {
+		MaxOutgoing += Overfdcount - fdcountold;
+		syslog(L_NOTICE, "%s outgoing %d", LogName, MaxOutgoing);
+	    }   
+	}
+    }
+#if 0 /* we should check almost all innconf parameter, but the code
+         is still incomplete for innd, so just commented out */
+    else if (EQ(p, "inn.conf")) {
+	ReadInnConf();
+	if (innconf->pathhost == NULL) {
+	    syslog(L_FATAL, "%s No pathhost set", LogName);
+	    exit(1);
+	}   
+	DISPOSE(Path.Data);
+	Path.Used = strlen(innconf->pathhost) + 1;
+	Path.Data = NEW(char, Path.Used + 1);
+	(void)sprintf(Path.Data, "%s!", innconf->pathhost);
+	if (Pathalias.Used > 0)
+	    DISPOSE(Pathalias.Data);
+	if (innconf->pathalias == NULL) {
+	    Pathalias.Used = 0;
+	    Pathalias.Data = NULL;
+	} else {
+	    Pathalias.Used = strlen(innconf->pathalias) + 1;
+	    Pathalias.Data = NEW(char, Pathalias.Used + 1);
+	    (void)sprintf(Pathalias.Data, "%s!", innconf->pathalias);
+	}
+    }
+#endif
 #if defined(DO_TCL)
     else if (EQ(p, "filter.tcl")) {
 	TCLreadfilter();
@@ -1376,6 +1515,9 @@ CCrmgroup(av)
     if ((ngp = NGfind(av[0])) == NULL)
 	return CCnogroup;
 
+    if (Mode == OMthrottled && ThrottledbyIOError)
+	return "1 server throttled";
+
     /* Update the in-core data. */
     if (!ICDrmgroup(ngp))
 	return "1 Failed";
@@ -1412,6 +1554,7 @@ CCshutdown(av)
     syslog(L_NOTICE, "%s shutdown %s", LogName, av[0]);
     CleanupAndExit(0, av[0]);
     /* NOTREACHED */
+    return "1 Exit failed";
 }
 
 
@@ -1486,6 +1629,45 @@ CCthrottle(av)
     return "1 unknown mode";
 }
 
+/*
+**  Turn on or off performance monitoring
+*/
+STATIC STRING CCtimer(char *av[]) {
+    int                 value;
+    char                *p;
+    
+    if (EQ(av[0], "off"))
+	value = 0;
+    else {
+	for (p = av[0]; *p; p++) {
+	    if (!isdigit(*p))
+		return "1 parameter should be a number or 'off'";
+	}
+	value = atoi(av[0]);
+    }
+    innconf->timer = value;
+    return NULL;
+}
+
+/*
+**  Turn innd status creation on or off
+*/
+STATIC STRING CCstatus(char *av[]) {
+    int                 value;
+    char                *p;
+    
+    if (EQ(av[0], "off"))
+	value = 0;
+    else {
+	for (p = av[0]; *p; p++) {
+	    if (!isdigit(*p))
+		return "1 parameter should be a number or 'off'";
+	}
+	value = atoi(av[0]);
+    }
+    innconf->status = value;
+    return NULL;
+}
 
 /*
 **  Add or remove tracing.
@@ -1570,14 +1752,14 @@ CCreader(cp)
     register char	*q;
     ICC_MSGLENTYPE	bufflen;
     ICC_PROTOCOLTYPE	protocol ;
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     struct sockaddr_un	client;
 #else
     int			written;
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     int			i;
-    char		buff[BUFSIZ + 2];
-    char		copy[BUFSIZ + 2];
+    char                buff[BIG_BUFFER + 2];
+    char                copy[BIG_BUFFER + 2];
     char		*argv[SC_MAXFIELDS + 2];
     int			argc;
     int			len;
@@ -1589,9 +1771,9 @@ CCreader(cp)
 	return;
     }
 
-#if defined (DO_HAVE_UNIX_DOMAIN)
+#if defined (HAVE_UNIX_DOMAIN_SOCKETS)
     
-    i = RECVorREAD(CCchan->fd, buff, BUFSIZ) ;
+    i = RECVorREAD(CCchan->fd, buff, BIG_BUFFER) ;
     if (i < 0) {
 	syslog(L_ERROR, "%s cant recv CCreader %m", LogName);
 	return;
@@ -1621,7 +1803,7 @@ CCreader(cp)
         return ;
     }
 
-#else  /* defined (DO_HAVE_UNIX_DOMAIN) */
+#else  /* defined (HAVE_UNIX_DOMAIN_SOCKETS) */
 
     i = RECVorREAD(CCchan->fd, buff, HEADER_SIZE) ;
     if (i < 0) {
@@ -1659,7 +1841,7 @@ CCreader(cp)
         return ;
     }
 
-#endif /* defined (DO_HAVE_UNIX_DOMAIN) */
+#endif /* defined (HAVE_UNIX_DOMAIN_SOCKETS) */
     
     /* Copy to a printable buffer, and log. */
     (void)strcpy(copy, buff);
@@ -1711,7 +1893,7 @@ CCreader(cp)
     strcpy (tbuff,p) ;
     tbuff -= HEADER_SIZE ;
 
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     (void)memset((POINTER)&client, 0, sizeof client);
     client.sun_family = AF_UNIX;
     (void)strcpy(client.sun_path, argv[0]);
@@ -1736,7 +1918,7 @@ CCreader(cp)
 	if (close(i) < 0)
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, argv[0]);
     }
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     DISPOSE (tbuff) ;
 }
 
@@ -1758,17 +1940,19 @@ void
 CCsetup()
 {
     int			i;
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     struct sockaddr_un	server;
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
+    if (CCpath == NULL)
+	CCpath = COPY(cpcatpath(innconf->pathrun, _PATH_NEWSCONTROL));
     /* Remove old detritus. */
     if (unlink(CCpath) < 0 && errno != ENOENT) {
 	syslog(L_FATAL, "%s cant unlink %s %m", LogName, CCpath);
 	exit(1);
     }
 
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     /* Create a socket and name it. */
     if ((i = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0) {
 	syslog(L_FATAL, "%s cant socket %s %m", LogName, CCpath);
@@ -1797,7 +1981,7 @@ CCsetup()
 	syslog(L_FATAL, "%s cant open %s %m", LogName, CCpath);
 	exit(1);
     }
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
     CCchan = CHANcreate(i, CTcontrol, CSwaiting, CCreader, CCwritedone);
     syslog(L_NOTICE, "%s ccsetup %s", LogName, CHANname(CCchan));
@@ -1828,10 +2012,10 @@ CCclose()
     CCchan = NULL;
     if (unlink(CCpath) < 0)
 	syslog(L_ERROR, "%s cant unlink %s %m", LogName, CCpath);
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     if (close(CCwriter) < 0)
 	syslog(L_ERROR, "%s cant close unbound %m", LogName);
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 }
 
 
@@ -1846,3 +2030,51 @@ CCresetup(s)
     CCclose();
     CCsetup();
 }
+
+# include "qio.h"
+/*
+ * Read a file containing lines of the form "newsgroup lowmark",
+ * and reset the low article number for the specified groups.
+ */
+STATIC STRING CClowmark(char *av[])
+{
+    long lo;
+    char *line, *cp, *ret = NULL;
+    QIOSTATE *qp;
+    NEWSGROUP *ngp;
+
+    if (Mode != OMrunning)
+	return CCnotrunning;
+    if (ICDneedsetup)
+	return "1 Must first reload newsfeeds";
+    if ((qp = QIOopen(av[0])) == NULL) {
+	syslog(L_ERROR, "%s cant open %s %m", LogName, av[0]);
+	return "1 Cannot read input file";
+    }
+    while ((line = QIOread(qp)) != NULL) {
+	if (QIOerror(qp))
+		break;
+	if (QIOtoolong(qp) || (cp = strchr(line, ' ')) == NULL) {
+	    ret = "1 Malformed input line";
+	    break;
+	}
+	*cp++ = '\0';
+	if ((lo = atol(cp)) == 0) {
+	    ret = "1 Malformed input line (missing low mark)";
+	    break;
+	}
+        if ((ngp = NGfind(line)) == NULL) {
+	    /* ret = CCnogroup; break; */
+	    continue;
+	}
+        if (!NGlowmark(ngp, lo)) {
+	    ret = "1 Cannot set low mark - see syslog";
+	    break;
+	}
+    }
+    if (ret == NULL && QIOerror(qp))
+	ret = "1 Error reading input file";
+    QIOclose(qp);
+    ICDwrite();
+    return ret;
+}
