@@ -1,7 +1,12 @@
-/*  $Revision: 1.1 $
+/*  $Id: icd.c,v 1.10 1998/05/20 10:08:09 scrappy Exp $
 **
 **  Routines to read and write the active file.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 #if	defined(ACT_MMAP)
 #include <sys/mman.h>
@@ -11,7 +16,7 @@
 typedef struct iovec	IOVEC;
 
 
-STATIC char		ICDactpath[] = _PATH_ACTIVE;
+STATIC char		*ICDactpath = NULL;
 STATIC char		*ICDactpointer;
 STATIC int		ICDactfd;
 STATIC int		ICDactsize;
@@ -22,21 +27,14 @@ STATIC int		ICDactsize;
 **  avoid problems with mmap.
 */
 #if	defined(ACT_MMAP)
-#define ICDiovset(iovp, base, len)	\
-	do { \
-	    (iovp)->iov_len = len; \
-	    (iovp)->iov_base = NEW(char, (iovp)->iov_len); \
-	    (void)memcpy((POINTER)(iovp)->iov_base, (POINTER)base, \
-		    (SIZE_T)(iovp)->iov_len); \
-	} while (JUSTONCE)
+void ICDiovset(IOVEC *iovp, char *base, int len) {
+    (iovp)->iov_len = len; 
+    (iovp)->iov_base = NEW(char, (iovp)->iov_len); 
+    (void)memcpy((POINTER)(iovp)->iov_base, (POINTER)base, 
+	    (SIZE_T)(iovp)->iov_len);
+}
 #define ICDiovrelease(iovp)		DISPOSE((iovp)->iov_base)
 
-#if	defined(MAP_FILE)
-#define MAP__ARG	(MAP_FILE | MAP_SHARED)
-#else
-#define MAP__ARG	(MAP_SHARED)
-#endif	/* defined(MAP_FILE) */
-
 #else
 
 #define ICDiovset(iovp, base, len)	\
@@ -54,7 +52,7 @@ ICDcloseactive()
     if (ICDactpointer) {
 #if	defined(ACT_MMAP)
 	if (munmap(ICDactpointer, ICDactsize) < 0)
-	    syslog(L_ERROR, "%s cant munmap %m", LogName, ICDactpath);
+	    syslog(L_ERROR, "%s cant munmap %s %m", LogName, ICDactpath);
 #else
 	DISPOSE(ICDactpointer);
 #endif	/* defined(ACT_MMAP) */
@@ -74,9 +72,13 @@ void
 ICDsetup(StartSites)
     BOOL	StartSites;
 {
-    ICDneedsetup = FALSE;
-    ICDcloseactive();
-    NGparsefile();
+    if (ICDneedsetup == TRUE) {
+	ICDneedsetup = FALSE;
+    }
+    else {
+	ICDcloseactive();
+	NGparsefile();
+    }
     if (NGfind("control") == NULL || NGfind("junk") == NULL) {
 	syslog(L_FATAL, "%s internal no control and/or junk group", LogName);
 	exit(1);
@@ -126,7 +128,7 @@ ICDclose()
 /*
 **  Scan the active file, and renumber the min/max counts.
 */
-void
+BOOL
 ICDrenumberactive()
 {
     register int	i;
@@ -134,56 +136,65 @@ ICDrenumberactive()
 
     for (i = nGroups, ngp = Groups; --i >= 0; ngp++)
 	if (!NGrenumber(ngp))
-	    break;
+	    return FALSE;
     if (i < 0)
 	ICDwrite();
+    return TRUE;
 }
 
 
 /*
 **  Use writev() to replace the active file.
 */
-STATIC BOOL
-ICDwritevactive(vp, vpcount)
-    IOVEC		*vp;
-    int			vpcount;
+STATIC BOOL ICDwritevactive(IOVEC *vp, int vpcount)
 {
-    static char		BACKUP[] = _PATH_OLDACTIVE;
+    static char		*BACKUP = NULL;
+    static char         *NEWACT = NULL;
     static char		WHEN[] = "backup active";
-    register int	fd;
-    char		*dummy;
+    int	                fd;
+    int			oerrno;
 
+    if (BACKUP == NULL)
+	BACKUP = COPY(cpcatpath(innconf->pathdb, _PATH_OLDACTIVE));
+    if (NEWACT == NULL)
+	NEWACT = COPY(cpcatpath(innconf->pathdb, _PATH_NEWACTIVE));
     /* Write the current file to a backup. */
     if (unlink(BACKUP) < 0 && errno != ENOENT) {
-	IOError(WHEN);
+	oerrno = errno;
 	syslog(L_ERROR, "%s cant unlink %s %m", LogName, BACKUP);
+	IOError(WHEN, oerrno);
     }
     if ((fd = open(BACKUP, O_WRONLY | O_TRUNC | O_CREAT, 0664)) < 0) {
-	IOError(WHEN);
+	oerrno = errno;
 	syslog(L_ERROR, "%s cant open %s %m", LogName, BACKUP);
+	IOError(WHEN, oerrno);
     }
     else if (xwrite(fd, ICDactpointer, ICDactsize) < 0) {
-	IOError(WHEN);
+	oerrno = errno;
 	syslog(L_ERROR, "%s cant write %s %m", LogName, BACKUP);
+	IOError(WHEN, oerrno);
 	(void)close(fd);
     }
     else if (close(fd) < 0) {
-	IOError(WHEN);
+	oerrno = errno;
 	syslog(L_ERROR, "%s cant close %s %m", LogName, BACKUP);
+	IOError(WHEN, oerrno);
     }
 
     /* Open the active file. */
-    fd = open(ICDactpath, O_WRONLY | O_TRUNC | O_CREAT, ARTFILE_MODE);
+    fd = open(NEWACT, O_WRONLY | O_TRUNC | O_CREAT, ARTFILE_MODE);
     if (fd < 0) {
-	IOError(WHEN);
-	syslog(L_ERROR, "%s cant open %s %m", LogName, ICDactpath);
+	oerrno = errno;
+	syslog(L_ERROR, "%s cant open %s %m", LogName, NEWACT);
+	IOError(WHEN, oerrno);
 	return FALSE;
     }
 
     /* Write it. */
     if (xwritev(fd, vp, vpcount) < 0) {
-	IOError(WHEN);
-	syslog(L_ERROR, "%s cant write %s %m", LogName, ICDactpath);
+	oerrno = errno;
+	syslog(L_ERROR, "%s cant write %s %m", LogName, NEWACT);
+	IOError(WHEN, oerrno);
 	(void)close(fd);
 	return FALSE;
     }
@@ -191,7 +202,16 @@ ICDwritevactive(vp, vpcount)
     /* Close it. */
     (void)close(fd);
     if (AmRoot)
-	xchown(ICDactpath);
+	xchown(NEWACT);
+
+    /* Rename it to be the canonical active file */
+    if (rename(NEWACT, ICDactpath) < 0) {
+	oerrno = errno;
+	syslog(L_ERROR, "%s cant rename %s to %s %m",
+	       LogName, NEWACT, ICDactpath);
+	IOError(WHEN, oerrno);
+	return FALSE;
+    }
 
     /* Invalidate in-core pointers. */
     ICDcloseactive();
@@ -200,7 +220,7 @@ ICDwritevactive(vp, vpcount)
     if (Mode != OMrunning) {
 	ICDneedsetup = TRUE;
 	/* Force the active file into memory. */
-	(void)ICDreadactive(&dummy);
+	NGparsefile();
     }
     else
 	ICDsetup(TRUE);
@@ -323,7 +343,8 @@ ICDreadactive(endp)
 	*endp = ICDactpointer + ICDactsize;
 	return ICDactpointer;
     }
-
+    if (ICDactpath == NULL) 
+	ICDactpath = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVE));
     if ((ICDactfd = open(ICDactpath, O_RDWR)) < 0) {
 	syslog(L_FATAL, "%s cant open %s %m", LogName, ICDactpath);
 	exit(1);
@@ -338,7 +359,7 @@ ICDreadactive(endp)
     }
     ICDactsize = Sb.st_size;
     ICDactpointer = mmap((caddr_t)0, ICDactsize, PROT_READ|PROT_WRITE,
-			MAP__ARG, ICDactfd, (off_t)0);
+			MAP__ARG, ICDactfd, (OFFSET_T)0);
     if (ICDactpointer == (char *)-1) {
 	syslog(L_FATAL, "%s cant mmap %d %s %m",
 	    LogName, ICDactfd, ICDactpath);
@@ -366,8 +387,19 @@ void
 ICDwriteactive()
 {
 #if	defined(ACT_MMAP)
-    msync(ICDactpointer, ICDactsize);
-
+#if defined (DO_MMAP_SYNC)
+#if defined (HAVE_MSYNC_3_ARG)
+    if (msync(ICDactpointer, ICDactsize, MS_ASYNC) < 0) {
+        syslog(L_FATAL, "%s msync failed %s %m", LogName, ICDactpath);
+        exit(1);
+    }
+#else
+    if (msync(ICDactpointer, 0) < 0) {
+        syslog(L_FATAL, "%s msync failed %s %m", LogName, ICDactpath);
+        exit(1);
+    }
+#endif
+#endif
 #else
 
     if (lseek(ICDactfd, 0L, SEEK_SET) == -1) {
