@@ -1,9 +1,10 @@
-/*  $Revision: 1.52 $
+/*  $Id: innd.c,v 1.38 1998/05/25 22:40:20 kondou Exp $
 **
 **  Variable definitions, miscellany, and main().
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #define DEFINE_DATA
@@ -19,11 +20,11 @@
 #endif	/* defined(DO_FAST_RESOLV) */
 
 
-#if	defined(HAVE_SETBUFFER)
-#define SETBUFFER(F, buff, size)	setbuffer((F), (buff), (size))
+#if defined(HAVE_SETBUFFER)
+# define SETBUFFER(F, buff, size)	setbuffer((F), (buff), (size))
 STATIC int	LogBufferSize = 4096;
 #else
-#define SETBUFFER(F, buff, size)	setbuf((F), (buff))
+# define SETBUFFER(F, buff, size)	setbuf((F), (buff))
 STATIC int	LogBufferSize = BUFSIZ;
 #endif	/* defined(HAVE_SETBUFFER) */
 
@@ -36,16 +37,13 @@ BOOL		Tracing = FALSE;
 BOOL		DoLinks = TRUE;
 BOOL		DoCancels = TRUE;
 char		LogName[] = "SERVER";
-char		SPOOL[] = _PATH_SPOOL;
 int		ErrorCount = IO_ERROR_COUNT;
-int		MaxIncoming = DEFAULT_CONNECTIONS;
-int		SPOOLlen = STRLEN(SPOOL);
-long		LargestArticle = MAX_ART_SIZE;
+int		SPOOLlen;
 OPERATINGMODE	Mode = OMrunning;
-time_t		Cutoff = DEFAULT_CUTOFF * 24 * 60 * 60;
 int		RemoteLimit = REMOTELIMIT;
 time_t		RemoteTimer = REMOTETIMER;
 int		RemoteTotal = REMOTETOTAL;
+BOOL		ThrottledbyIOError = FALSE;
 
 #if	defined(__CENTERLINE__)
 BOOL		Debug = TRUE;
@@ -60,10 +58,9 @@ int		KeepLintQuiet = 0;
 
 STATIC char	*ErrlogBuffer;
 STATIC char	*LogBuffer;
-STATIC char	ERRLOG[] = _PATH_ERRLOG;
-STATIC char	INNDDIR[] = _PATH_INNDDIR;
-STATIC char	LOG[] = _PATH_LOGFILE;
-STATIC char	PID[] = _PATH_SERVERPID;
+STATIC char	*ERRLOG = NULL;
+STATIC char	*LOG = NULL;
+STATIC char	*PID = NULL;
 STATIC UID_T	NewsUID;
 STATIC GID_T	NewsGID;
 
@@ -245,7 +242,8 @@ NeedShell(p, av, end)
 **  (and a syslog'd message) on error.
 */
 PID_T
-Spawn(fd0, fd1, fd2, av)
+Spawn(niceval, fd0, fd1, fd2, av)
+    int		niceval;
     int		fd0;
     int		fd1;
     int		fd2;
@@ -297,20 +295,14 @@ Spawn(fd0, fd1, fd2, av)
     CloseOnExec(2, FALSE);
 
     /* Try to set our permissions. */
-#if	defined(DO_INND_NICE_KIDS)
-    (void)nice(INND_NICE_VALUE);
-#endif	/* defined(DO_INND_NICE_KIDS) */
+    if (niceval != 0)
+	if (nice(niceval) == -1) 
+		syslog(L_ERROR, "Could not nice child to %d: %m", niceval);
     if (setgid(NewsGID) == -1)
 	syslog(L_ERROR, "%s cant setgid in %s %m", LogName, av[0]);
     if (setuid(NewsUID) == -1)
 	syslog(L_ERROR, "%s cant setuid in %s %m", LogName, av[0]);
 
-    /* Close the DBZ database without doing any writing. */
-    /* Not needed with the patched DBZ; can't be used with vfork.
-     * (void)dbzcancel();
-     * (void)dbmclose();
-     */
-
     /* Start the desired process (finally!). */
     (void)execv(av[0], av);
     syslog(L_FATAL, "%s cant exec in %s %m", LogName, av[0]);
@@ -328,7 +320,7 @@ GetNewsOwnerships()
     struct stat	Sb;
 
     /* Make sure item exists and is of the right type. */
-    if (stat(INNDDIR, &Sb) < 0)
+    if (stat(innconf->pathrun, &Sb) < 0)
 	return FALSE;
     if (!S_ISDIR(Sb.st_mode))
 	return FALSE;
@@ -350,55 +342,6 @@ xchown(p)
 }
 
 
-/*
-**  Try to make one directory.  Return FALSE on error.
-*/
-STATIC BOOL
-MakeDir(Name)
-    char		*Name;
-{
-    struct stat		Sb;
-
-    if (mkdir(Name, GROUPDIR_MODE) >= 0) {
-	if (AmRoot)
-	    xchown(Name);
-	return TRUE;
-    }
-
-    /* See if it failed because it already exists. */
-    return stat(Name, &Sb) >= 0 && S_ISDIR(Sb.st_mode);
-}
-
-
-/*
-**  Given a directory, comp/foo/bar, create that directory and all
-**  intermediate directories needed.  Return 0 if ok, else -1.
-*/
-BOOL
-MakeSpoolDirectory(Name)
-    register char	*Name;
-{
-    register char	*p;
-    BOOL		made;
-
-    /* Optimize common case -- parent almost always exists. */
-    if (MakeDir(Name))
-	return TRUE;
-
-    /* Try to make each of comp and comp/foo in turn. */
-    for (p = Name; *p; p++)
-	if (*p == '/') {
-	    *p = '\0';
-	    made = MakeDir(Name);
-	    *p = '/';
-	    if (!made)
-		return FALSE;
-	}
-
-    return MakeDir(Name);
-}
-
-
 /*
 **  Flush one log file, with pessimistic size of working filename buffer.
 */
@@ -406,7 +349,7 @@ void
 ReopenLog(F)
     FILE	*F;
 {
-    char	buff[sizeof LOG + sizeof ERRLOG + 4 + 1];
+    char	buff[SMBUF];
     char	*Name;
     char	*Buffer;
     int		mask;
@@ -476,6 +419,7 @@ ThrottleIOError(when)
 	    syslog(L_ERROR, "%s cant throttle %s", LogName, p);
 	syslog(L_FATAL, "%s throttle %s", LogName, buff);
 	errno = oerrno;
+	ThrottledbyIOError = TRUE;
     }
 }
 
@@ -495,6 +439,10 @@ JustCleanup()
     ICDclose();
     HISclose();
     ARTclose();
+    if (innconf->storageapi) {
+	SMshutdown();
+	OVERshutdown();
+    }
 #if defined(DO_TCL)
     TCLclose();
 #endif /* defined(DO_TCL) */
@@ -576,23 +524,19 @@ Usage()
 }
 
 
-int
-main(ac, av)
-    int			ac;
-    char		*av[];
+int main(int ac, char *av[])
 {
     static char		WHEN[] = "PID file";
     int			i;
-    int			port;
+    int			fd;
     int			logflags;
     char		buff[SMBUF];
-    char		*master;
-    STRING		path;
     char		*p;
     FILE		*F;
     BOOL		ShouldFork;
     BOOL		ShouldRenumber;
     BOOL		ShouldSyntaxCheck;
+    BOOL		val;
     PID_T		pid;
 #if	defined(_DEBUG_MALLOC_INC)
     union malloptarg	m;
@@ -625,20 +569,22 @@ main(ac, av)
     ShouldRenumber = FALSE;
     ShouldSyntaxCheck = FALSE;
     logflags = L_OPENLOG_FLAGS | LOG_NOWAIT;
-    port = -1;
-    master = NULL;
-#if	defined(DONT_ALLOW_READERS)
-    NNRPFollows = TRUE;
-#endif	/* defined(DONT_ALLOW_READERS) */
+    fd = -1;
 
 #if	defined(DO_FAST_RESOLV)
     /* We only use FQDN's in the hosts.nntp file. */
     _res.options &= ~(RES_DEFNAMES | RES_DNSRCH);
 #endif	/* defined(DO_FAST_RESOLV) */
 
+  /* Set some options from inn.conf(5) that can be overridden with
+     command-line options if they exist */
+    if (ReadInnConf() < 0) exit(1);
+    LOG = COPY(cpcatpath(innconf->pathlog, _PATH_LOGFILE));
+    ERRLOG = COPY(cpcatpath(innconf->pathlog, _PATH_ERRLOG));
+
     /* Parse JCL. */
     CCcopyargv(av);
-    while ((i = getopt(ac, av, "ac:Cdfi:l:Lm:o:n:p:rsS:t:uH:T:X:")) != EOF)
+    while ((i = getopt(ac, av, "ac:Cdfi:l:Lm:o:n:p:P:rst:uH:T:X:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -647,7 +593,7 @@ main(ac, av)
 	    AnyIncoming = TRUE;
 	    break;
 	case 'c':
-	    Cutoff = atoi(optarg) * 24 * 60 * 60;
+	    innconf->artcutoff = atoi(optarg) * 24 * 60 * 60;
 	    break;
 	case 'd':
 	    Debug = TRUE;
@@ -662,10 +608,14 @@ main(ac, av)
 	    RemoteLimit = atoi(optarg);
 	    break;
 	case 'i':
-	    MaxIncoming = atoi(optarg);
+	    innconf->maxconnections = atoi(optarg);
+	    break;
+	case 'I':
+	    if (innconf->bindaddress) DISPOSE(innconf->bindaddress);
+	    innconf->bindaddress = COPY(optarg);
 	    break;
 	case 'l':
-	    LargestArticle = atol(optarg);
+	    innconf->maxartsize = atol(optarg);
 	    break;
 	case 'm':
 	    if (ModeReason)
@@ -689,8 +639,8 @@ main(ac, av)
 	    default:
 		Usage();
 		/* NOTREACHED */
-	    case 'n':	NNRPFollows = TRUE;	break;
-	    case 'y':	NNRPFollows = FALSE;	break;
+	    case 'n':	innconf->allowreaders = TRUE;	break;
+	    case 'y':	innconf->allowreaders = FALSE;	break;
 	    }
 	    break;
 	case 'o':
@@ -699,22 +649,20 @@ main(ac, av)
 	case 'p':
 	    /* Silently ignore multiple -p flags, in case ctlinnd xexec
 	     * called inndstart. */
-	    if (port == -1) {
-		port = atoi(optarg);
+	    if (fd == -1) {
+		fd = atoi(optarg);
 		AmRoot = FALSE;
 	    }
 	    break;
+	case 'P':
+	    innconf->port = atoi(optarg);
+	    break;
 	case 'r':
 	    ShouldRenumber = TRUE;
 	    break;
 	case 's':
 	    ShouldSyntaxCheck = TRUE;
 	    break;
-	case 'S':
-	    if (strlen(optarg) >= SMBUF - 4)
-		Usage();
-	    master = optarg;
-	    break;
 	case 't':
 	    TimeOut.tv_sec = atol(optarg);
 	    break;
@@ -740,7 +688,7 @@ main(ac, av)
     ac -= optind;
     if (ac != 0)
 	Usage();
-    if (ModeReason && NNRPFollows)
+    if (ModeReason && innconf->allowreaders)
 	NNRPReason = COPY(ModeReason);
 
     openlog(path, logflags, LOG_INN_SERVER);
@@ -752,30 +700,45 @@ main(ac, av)
 	exit(1);
     }
 
+    SPOOLlen = strlen(innconf->patharticles);
     /* Go to where the data is. */
-    if (chdir(SPOOL) < 0) {
-	syslog(L_FATAL, "%s cant chdir %s %m", LogName, SPOOL);
+    if (chdir(innconf->patharticles) < 0) {
+	syslog(L_FATAL, "%s cant chdir %s %m", LogName, innconf->patharticles);
+	exit(1);
+    }
+
+    val = FALSE;
+    if (!OVERsetup(OVER_MMAP, &val)) {
+	syslog(L_FATAL, "%s cant setup for the unified overview no mmap");
+	exit(1);
+    }
+    val = TRUE;
+    if (!OVERsetup(OVER_PREOPEN, &val)) {
+	syslog(L_FATAL, "%s cant setup for the unified overview preopen");
+	exit(1);
+    }
+
+    if (innconf->storageapi && (Overfdcount = OVERgetnum()) < 0) {
+	syslog(L_FATAL, "%s cant get config for the unified overview");
 	exit(1);
     }
 
     /* Get the Path entry. */
-    if ((path = GetConfigValue(_CONF_PATHHOST)) == NULL) {
-	syslog(L_FATAL, "%s cant GetConfigValue %s %m",
-	    LogName, _CONF_PATHHOST);
+    if (innconf->pathhost == NULL) {
+	syslog(L_FATAL, "%s No pathhost set", LogName);
 	exit(1);
     }
-    Path.Used = strlen(path) + 1;
+    Path.Used = strlen(innconf->pathhost) + 1;
     Path.Data = NEW(char, Path.Used + 1);
-    (void)sprintf(Path.Data, "%s!", path);
-
-    /* Get the Xref prefix. */
-    Xref.Size = STRLEN("Xref: ") + strlen(path) + 1;
-    Xref.Data = NEW(char, Xref.Size);
-    FileGlue(Xref.Data, "Xref:", ' ', path);
-    if ((p = strchr(Xref.Data, '!')) != NULL)
-	*p = '\0';
-    Xref.Used = strlen(Xref.Data);
-    Xrefbase = Xref.Used;
+    (void)sprintf(Path.Data, "%s!", innconf->pathhost);
+    if (innconf->pathalias == NULL) {
+	Pathalias.Used = 0;
+	Pathalias.Data = NULL;
+    } else {
+	Pathalias.Used = strlen(innconf->pathalias) + 1;
+	Pathalias.Data = NEW(char, Pathalias.Used + 1);
+	(void)sprintf(Pathalias.Data, "%s!", innconf->pathalias);
+    }
 
 #if	!defined(__CENTERLINE__)
     /* Set standard input to /dev/null. */
@@ -787,6 +750,11 @@ main(ac, av)
 	syslog(L_NOTICE, "%s cant dup2 %d to 0 %m", LogName, i);
     (void)close(i);
 #endif	/* !defined(__CENTERLINE__) */
+    i = dbzneedfilecount();
+    if (!fdreserve(2 + i)) { /* TEMPORARYOPEN, INND_HISTORY and i */
+	syslog(L_FATAL, "%s cant reserve file descriptors %m", LogName);
+	exit(1);
+    }
 
     /* Set up our permissions. */
     (void)umask(NEWSUMASK);
@@ -794,7 +762,7 @@ main(ac, av)
 	syslog(L_FATAL, "%s internal cant stat control directory %m", LogName);
 	exit(1);
     }
-    if (port != -1 && setgid(NewsGID) < 0)
+    if (fd != -1 && setgid(NewsGID) < 0)
 	syslog(L_ERROR, "%s cant setgid running as %d not %d %m",
 	    LogName, (int)getgid(), (int)NewsGID);
 
@@ -823,9 +791,9 @@ main(ac, av)
 		    syslog(L_ERROR, "%s cant close /dev/tty %m", LogName);
 	    }
 #endif	/* defined(TIOCNOTTY) */
-#if	defined(DO_HAVE_SETSID)
+#if	defined(HAVE_SETSID)
 	    (void)setsid();
-#endif	/* defined(DO_HAVE_SETSID) */
+#endif	/* defined(HAVE_SETSID) */
 	}
 
 	/* Open the Log. */
@@ -863,12 +831,14 @@ main(ac, av)
     }
     syslog(L_NOTICE, "%s descriptors %d", LogName, i);
     if (MaxOutgoing == 0) {
-	/* getfdcount() - (stdio + dbz + cc + lc + rc + art + fudge) */
-	MaxOutgoing = i - (  3   +  3  +  2 +  1 +  1 +  1  +   2  );
+	/* getfdcount() - (stdio + dbz + cc + lc + rc + art + Overfdcount + fudge) */
+	MaxOutgoing = i - (  3   +  3  +  2 +  1 +  1 +  1  + Overfdcount +  2  );
 	syslog(L_NOTICE, "%s outgoing %d", LogName, MaxOutgoing);
     }
 
     /* See if another instance is alive. */
+    if (PID == NULL)
+	PID = COPY(cpcatpath(innconf->pathrun, _PATH_SERVERPID));
     if ((F = fopen(PID, "r")) != NULL) {
 	if (fgets(buff, sizeof buff, F) != NULL
 	 && ((pid = (PID_T) atol(buff)) > 0)
@@ -891,14 +861,31 @@ main(ac, av)
     (void)signal(SIGDANGER, CatchTerminate);
 #endif	/* defined(SIGDANGER) */
     CHANsetup(i);
-    PROCsetup(i * 2);
+    PROCsetup(10);
     HISsetup();
     CCsetup();
     LCsetup();
-    RCsetup(port, master);
+    RCsetup(fd);
+    WIPsetup();
     NCsetup(i);
     ARTsetup();
     ICDsetup(TRUE);
+    if (innconf->storageapi) {
+	innconf->wireformat = TRUE;
+	val = TRUE;
+	if (!SMsetup(SM_RDWR, (void *)&val) || !SMsetup(SM_PREOPEN, (void *)&val)) {
+	    syslog(L_FATAL, "%s cant setup the storage subsystem", LogName);
+	    exit(1);
+	}
+	if (!SMinit()) {
+	    syslog(L_FATAL, "%s cant initialize the storage subsystem %s", LogName, SMerrorstr);
+	    exit(1);
+	}
+    }
+    if (innconf->storageapi && !OVERinit()) {
+	syslog(L_FATAL, "%s cant initialize the unified overview", LogName);
+	exit(1);
+    }
 #if	defined(_DEBUG_MALLOC_INC)
     m.i = 1;
     dbmallopt(MALLOC_CKCHAIN, &m);
@@ -936,8 +923,12 @@ main(ac, av)
 
 #if defined(DO_PERL)
     /* Load the Perl code */
-    PERLsetup(_PATH_PERL_STARTUP_INND, _PATH_PERL_FILTER_INND, "filter_art");
+    /* Make a temp copy because the path is a static var */
+    p = COPY(cpcatpath(innconf->pathfilter, _PATH_PERL_STARTUP_INND));
+    PERLsetup(p, cpcatpath(innconf->pathfilter, _PATH_PERL_FILTER_INND),
+				"filter_art");
     PerlFilter (TRUE) ;
+    DISPOSE(p);
 #endif /* defined(DO_PERL) */
  
     /* And away we go... */
