@@ -1,4 +1,4 @@
-/*  $Revision: 1.37 $
+/*  $Revision: 1.38 $
 **
 **  Many of the data types used here have abbreviations, such as CT
 **  for CHANNELTYPE.  Here are a list of the conventions and meanings:
@@ -20,13 +20,11 @@
 **	RCHAN	A channel in "read" state
 **	SITE	Something that gets told when we get an article
 **	WCHAN	A channel in "write" state
+**      WIP     Work-In-Progress, keeps track of articles before committed.
 */
-#include "configdata.h"
-#include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
 #include <signal.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #if	defined(DO_NEED_TIME)
 #include <time.h>
@@ -37,13 +35,21 @@
 #include <fcntl.h>
 #include "nntp.h"
 #include "paths.h"
-#include "logging.h"
+#include <syslog.h> 
 #include "libinn.h"
-#include "clibrary.h"
 #include "macros.h"
+#include "dbz.h"
+
+#if     defined(DO_TCL)
+#include <tcl.h>
+#undef EXTERN /* TCL defines EXTERN; this undef prevents error messages 
+	         when we define it later */
+#endif  /* defined(DO_TCL) */
 
 
-#define NOSITE		-1
+typedef short  SITEIDX;
+
+#define NOSITE		((SITEIDX) -1)
 
 
 /*
@@ -51,6 +57,9 @@
 */
 typedef struct in_addr	INADDR;
 
+#ifndef u_long
+#define u_long		unsigned long
+#endif
 
 /*
 **  Server's operating mode.
@@ -67,9 +76,9 @@ typedef enum _OPERATINGMODE {
 **  much space is left if reading or how much still needs to be written.
 */
 typedef struct _BUFFER {
-    int		Size;
-    int		Used;
-    int		Left;
+    long	Size;
+    long	Used;
+    long	Left;
     char	*Data;
 } BUFFER;
 
@@ -79,7 +88,6 @@ typedef struct _BUFFER {
 */
 typedef enum _HANDOFF {
     HOnnrpd,
-    HOnnrqd,
     HOnntpd
 } HANDOFF;
 
@@ -91,6 +99,7 @@ typedef enum _CHANNELTYPE {
     CTany,
     CTfree,
     CTremconn,
+    CTreject,
     CTnntp,
     CTlocalconn,
     CTcontrol,
@@ -114,7 +123,8 @@ typedef enum _CHANNELSTATE {
     CSpaused,
     CSgetarticle,
     CSeatarticle,
-    CSgetrep
+    CSgetrep,
+    CSgetxbatch
 } CHANNELSTATE;
 
 
@@ -126,7 +136,28 @@ typedef enum _CHANNELSTATE {
 typedef struct _CHANNEL {
     CHANNELTYPE		Type;
     CHANNELSTATE	State;
+    BOOL		Skip;
+    BOOL		Streaming;
     int			fd;
+    u_long		Duplicate;
+    u_long		Unwanted_s;
+    u_long		Unwanted_f;
+    u_long		Unwanted_d;
+    u_long		Unwanted_g;
+    u_long		Unwanted_u;
+    u_long		Unwanted_o;
+    u_long		Size;
+    u_long		Check;
+    u_long		Check_send;
+    u_long		Check_deferred;
+    u_long		Check_got;
+    u_long		Takethis;
+    u_long		Takethis_Ok;
+    u_long		Takethis_Err;
+    u_long		Ihave;
+    u_long		Ihave_Duplicate;
+    u_long		Ihave_Deferred;
+    u_long		Ihave_SendIt;
     int			Reported;
     long		Received;
     long		Refused;
@@ -134,6 +165,7 @@ typedef struct _CHANNEL {
     int			BadWrites;
     int			BadReads;
     int			BlockedWrites;
+    int			BadCommands;
     time_t		LastActive;
     time_t		NextLog;
     INADDR		Address;
@@ -142,10 +174,23 @@ typedef struct _CHANNEL {
     time_t		Waketime;
     time_t		Started;
     FUNCPTR		Waker;
-    char		*Argument;
+    POINTER		Argument;
+    POINTER		Event;
     BUFFER		In;
     BUFFER		Out;
     BOOL		Tracing;
+    BUFFER		Sendid;
+    HASH                CurrentMessageIDHash;
+#define PRECOMMITCACHESIZE 128
+    struct _WIP         *PrecommitWIP[PRECOMMITCACHESIZE];
+    int                 PrecommitiCachenext;
+    int                 XBatchSize;
+    int                 LargeArtSize;
+    int			Lastch;
+    int			Rest;
+    int			SaveUsed;
+    int			ActiveCnx;
+    int			MaxCnx;
 } CHANNEL;
 
 
@@ -165,8 +210,10 @@ typedef struct _NEWSGROUP {
     int			PostCount;	/* Have we already put it here?	*/
     char		*LastString;
     char		*Rest;
-    int			nSites;
+    SITEIDX		nSites;
     int			*Sites;
+    SITEIDX		nPoison;
+    int			*Poison;
     struct _NEWSGROUP	*Alias;
 } NEWSGROUP;
 
@@ -180,6 +227,7 @@ typedef enum _FEEDTYPE {
     FTchannel,
     FTexploder,
     FTfunnel,
+    FTlogonly,
     FTprogram
 } FEEDTYPE;
 
@@ -195,10 +243,18 @@ typedef struct _SITE {
     char		**Exclusions;
     char		**Distributions;
     char		**Patterns;
+    BOOL		Poison;
+    BOOL		PoisonEntry;
     BOOL		Sendit;
     BOOL		Seenit;
+    BOOL		IgnoreControl;
     BOOL		DistRequired;
+    BOOL		IgnorePath;
+    BOOL		ControlOnly;
+    BOOL		DontWantNonExist;
+    BOOL		NeedOverviewCreation;
     int			Hops;
+    int			Groupcount;
     FEEDTYPE		Type;
     NEWSGROUP		*ng;
     BOOL		Spooling;
@@ -210,6 +266,8 @@ typedef struct _SITE {
     char		*Param;
     char		FileFlags[FEED_MAXFLAGS + 1];
     long		MaxSize;
+    long		MinSize;
+    int			Nice;
     CHANNEL		*Channel;
     BOOL		IsMaster;
     int			Master;
@@ -218,11 +276,12 @@ typedef struct _SITE {
     BUFFER		FNLnames;
     int			Process;
     PID_T		pid;
-    int			Flushpoint;
+    long		Flushpoint;
     BUFFER		Buffer;
     BOOL		Buffered;
-    struct _SITE	*Next;
-    struct _SITE	*Prev;
+    char                *Originator;
+    int			Next;
+    int			Prev;
 } SITE;
 
 
@@ -258,13 +317,18 @@ typedef struct _ARTDATA {
     STRING	Replyto;
     char	*Body;
     time_t	Posted;
+    time_t	Arrived;
     time_t	Expires;
-    int		Lines;
+    int		Groupcount;
+    int		LinesValue;
+    char	Lines[SMBUF];
     long	SizeValue;
     char	Size[SMBUF];
     int		SizeLength;
     char	Name[SPOOLNAMEBUFF];
     int		NameLength;
+    char	TimeReceived[33];
+    int		TimeReceivedLength;
     STRING	MessageID;
     int		MessageIDLength;
     STRING	Newsgroups;
@@ -273,72 +337,51 @@ typedef struct _ARTDATA {
     int		DistributionLength;
     STRING	Feedsite;
     int		FeedsiteLength;
+    STRING      Path;
+    int		PathLength;
     STRING	Replic;
     int		ReplicLength;
+    HASH	*Hash;
+    BUFFER	*Headers;
+    BUFFER	*Overview;
 } ARTDATA;
 
+typedef struct _WIP {
+    HASH        MessageID;       /* Hash of the messageid.  Doing it like this saves
+			            us from haveing to allocate and deallocate memory
+			            a lot, and also means lookups are faster. */
+    time_t      Timestamp;       /* Time we last looked at this MessageID */
+    CHANNEL     *Chan;           /* Channel that this message is associated with */
+    struct _WIP *Next;           /* Next item in this bucket */
+} WIP;
+
+typedef enum {TMR_IDLE, TMR_ARTWRITE, TMR_ARTLINK, TMR_HISWRITE,
+	      TMR_HISSYNC, TMR_SITESEND, TMR_ARTCTRL, TMR_ARTCNCL,
+	      TMR_HISHAVE, TMR_HISGREP, TMR_MAX} TMRTYPE;
 
 
 
 /*
 **  In-line macros for efficiency.
-*/
-
-#if	defined(lint) || defined(__CENTERLINE__)
-extern int	KeepLintQuiet;
-#define JUSTONCE	KeepLintQuiet
-#else
-#define JUSTONCE	0
-#endif	/* defined(lint) || defined(__CENTERLINE__) */
-
-/*
+**
 **  Set or append data to a channel's output buffer.
 */
 #define WCHANset(cp, p, l)	BUFFset(&(cp)->Out, (p), (l))
 #define WCHANappend(cp, p, l)	BUFFappend(&(cp)->Out, (p), (l))
 
-
-/*
-**  Append data to a buffer.
-*/
-#define BUFFappend(bp_parm, p_parm, l_parm) \
-    do { \
-	register int	l_; \
-	register BUFFER	*bp_; \
-	int		i_; \
-    \
-	if ((l_ = l_parm) != 0) { \
-	    bp_ = bp_parm; \
-	    /* Note end of buffer, grow it if we need more room. */ \
-	    i_ = bp_->Used + bp_->Left; \
-	    if (i_ + l_ > bp_->Size) { \
-		/* Round size up to next 1K. */ \
-		bp_->Size += (l_ + 0x3FF) & ~0x3FF; \
-		RENEW(bp_->Data, char, bp_->Size); \
-	    } \
-	    bp_->Left += l_; \
-	    if (l_ > MEMCPY_THRESHOLD) \
-		(void)memcpy((POINTER)&bp_->Data[i_], (POINTER)p_parm, (SIZE_T)l_); \
-	    else { \
-		register STRING	p_; \
-		register char	*dest_; \
-    \
-		for (p_ = p_parm, dest_ = &bp_->Data[i_], l_++; --l_ > 0; ) \
-		    *dest_++ = *p_++; \
-	    } \
-	} \
-    } while (JUSTONCE)
-
-
 /*
 **  Mark that an I/O error occurred, and block if we got too many.
 */
-#define IOError(WHEN)	\
-	if (--ErrorCount <= 0 || errno == ENOSPC) ThrottleIOError(WHEN); else
+#define IOError(WHEN, e)	\
+	if (--ErrorCount <= 0 || (e) == ENOSPC) ThrottleIOError(WHEN); else
 
 
 /*
 **  Global data.
+**
+** Do not change "extern" to "EXTERN" in the Global data.  The ones
+** marked with "extern" are initialized in innd.c.  The ones marked
+** with "EXTERN" are not explicitly initialized in innd.c.
 */
 #if	defined(DEFINE_DATA)
 #define EXTERN		/* NULL */
@@ -346,19 +389,25 @@ extern int	KeepLintQuiet;
 #define EXTERN		extern
 #endif	/* defined(DEFINE_DATA) */
 extern BOOL		AmRoot;
-EXTERN BOOL		AmSlave;
+extern BOOL		BufferedLogs;
 EXTERN BOOL		AnyIncoming;
 extern BOOL		Debug;
 EXTERN BOOL		ICDneedsetup;
-EXTERN BOOL		NNRPFollows;
+EXTERN BOOL		NeedHeaders;
+EXTERN BOOL		NeedOverview;
+EXTERN BOOL		NeedPath;
 extern BOOL		NNRPTracing;
+extern BOOL		StreamingOff;
 extern BOOL		Tracing;
+EXTERN int              Overfdcount;
+EXTERN int		SeqNum;
+EXTERN STRING		path;
 EXTERN BUFFER		Path;
-EXTERN BUFFER		Xref;
+EXTERN BUFFER		Pathalias;
 EXTERN char		*ModeReason;	/* NNTP reject message		*/
 EXTERN char		*NNRPReason;	/* NNRP reject message		*/
 EXTERN char		*Reservation;	/* Reserved lock message	*/
-extern char		SPOOL[];
+EXTERN char		*RejectReason;	/* NNTP reject message		*/
 EXTERN char		*Version;
 EXTERN FILE		*Errlog;
 EXTERN FILE		*Log;
@@ -366,10 +415,10 @@ extern char		LogName[];
 EXTERN INADDR		MyAddress;
 extern int		ErrorCount;
 EXTERN int		ICDactivedirty;
-extern int		MaxIncoming;
+EXTERN int		KillerSignal;
 EXTERN int		MaxOutgoing;
 EXTERN int		nGroups;
-EXTERN int		nSites;
+EXTERN SITEIDX		nSites;
 EXTERN int		PROCneedscan;
 extern int		SPOOLlen;
 EXTERN int		Xrefbase;
@@ -382,7 +431,27 @@ EXTERN SITE		*Sites;
 EXTERN SITE		ME;
 EXTERN struct timeval	TimeOut;
 EXTERN TIMEINFO		Now;		/* Reasonably accurate time	*/
-extern time_t		Cutoff;
+EXTERN BOOL		ThrottledbyIOError;
+EXTERN BOOL		AddAlias;
+EXTERN BOOL		Hassamepath;
+
+/*
+** Table size for limiting incoming connects.  Do not change the table
+** size unless you look at the code manipulating it in rc.c.
+*/
+#define REMOTETABLESIZE 128
+
+/*
+** Setup the default values.  The REMOTETIMER being zero turns off the
+** code to limit incoming connects.
+*/
+#define REMOTELIMIT	2
+#define REMOTETIMER	0
+#define REMOTETOTAL	60
+#define REJECT_TIMEOUT	10
+extern int		RemoteLimit; /* Per host limit. */
+extern time_t		RemoteTimer; /* How long to remember connects. */
+extern int		RemoteTotal; /* Total limit. */
 
 
 /*
@@ -393,7 +462,7 @@ extern BOOL		MakeSpoolDirectory();
 extern BOOL		NeedShell();
 extern char		**CommaSplit();
 extern char		*MaxLength();
-extern int		Spawn();
+extern PID_T		Spawn();
 extern NORETURN		CleanupAndExit();
 extern void		FileGlue();
 extern void		JustCleanup();
@@ -401,56 +470,60 @@ extern void		ThrottleIOError();
 extern void		ReopenLog();
 extern void		xchown();
 
-extern BOOL		ARTidok();
-extern char		*ARTreadarticle();
-extern char		*ARTreadheader();
-extern STRING		ARTpost();
-extern void		ARTcancel();
-extern void		ARTsetup();
+extern BOOL		ARTidok(const char *MessageID);
+extern BOOL		ARTreadschema(void);
+extern const char       *ARTreadarticle(char *files);
+extern char             *ARTreadheader(char *files);
+extern STRING		ARTpost(CHANNEL *cp);
+extern void		ARTcancel(const ARTDATA *Data, const char *MessageID, const BOOL Trusted);
+extern void		ARTclose(void);
+extern void		ARTsetup(void);
 
-extern void		BUFFset();
+extern void		BUFFset(BUFFER *bp, const char *p, const int length);
+extern void		BUFFswap(BUFFER *b1, BUFFER *b2);
+extern void             BUFFappend(BUFFER *bp, const char *p, const int len);
 
-extern BOOL		CHANsleeping();
-extern CHANNEL		*CHANcreate();
-extern CHANNEL		*CHANiter();
-extern CHANNEL		*CHANfromdescriptor();
-extern char		*CHANname();
-extern int		CHANreadtext();
-extern void		CHANclose();
-extern void		CHANreadloop();
-extern void		CHANsetup();
-extern void		CHANtracing();
+extern BOOL		CHANsleeping(CHANNEL *cp);
+extern CHANNEL		*CHANcreate(int fd, CHANNELTYPE Type, CHANNELSTATE STate, FUNCPTR Reader, FUNCPTR WriteDone);
+extern CHANNEL		*CHANiter(int *cp, CHANNELTYPE Type);
+extern CHANNEL		*CHANfromdescriptor(int fd);
+extern char		*CHANname(const CHANNEL *cp);
+extern int		CHANreadtext(CHANNEL *cp);
+extern void		CHANclose(CHANNEL *cp, char *name);
+extern void		CHANreadloop(void);
+extern void		CHANsetup(int i);
+extern void		CHANtracing(CHANNEL *cp, BOOL Flag);
 
-extern void		RCHANadd();
-extern void		RCHANremove();
+extern void		RCHANadd(CHANNEL *cp);
+extern void		RCHANremove(CHANNEL *cp);
 
-extern void		SCHANadd();
-extern void		SCHANremove();
+extern void		SCHANadd(CHANNEL *cp, time_t Waketime, POINTER Event, FUNCPTR Waker, POINTER Argument);
+extern void		SCHANremove(CHANNEL *cp);
+extern void		SCHANwakeup(POINTER *Event);
 
-extern BOOL		WCHANflush();
-extern void		WCHANadd();
-extern void		WCHANfappend();
-extern void		WCHANremove();
-extern void		WCHANsetfrombuffer();
+extern BOOL		WCHANflush(CHANNEL *cp);
+extern void		WCHANadd(CHANNEL *cp);
+extern void		WCHANremove(CHANNEL *cp);
+extern void		WCHANsetfrombuffer(CHANNEL *cp, BUFFER *bp);
 
 extern void		CCcopyargv();
 extern STRING		CCblock();
 extern STRING		CCcancel();
 extern STRING		CCcheckfile();
 
-extern BOOL		HIShavearticle();
-extern BOOL		HISwrite();
-extern char		*HISfilesfor();
-extern void		HISclose();
-extern void		HISsetup();
-extern void		HISsync();
+extern BOOL		HIShavearticle(const HASH MessageID);
+extern BOOL		HISwrite(const ARTDATA *Data, HASH hash, char *paths, TOKEN *token);
+extern BOOL             HISremember(const HASH MessageID);
+extern char		*HISfilesfor(const HASH MessageID);
+extern void		HISclose(void);
+extern void		HISsetup(void);
+extern void		HISsync(void);
 
 extern BOOL		ICDnewgroup();
 extern char		*ICDreadactive();
 extern BOOL		ICDchangegroup();
 extern void		ICDclose();
-extern void		ICDcloseactive();
-extern void		ICDrenumberactive();
+extern BOOL		ICDrenumberactive();
 extern BOOL		ICDrmgroup();
 extern void		ICDsetup();
 extern void		ICDwrite();
@@ -467,7 +540,9 @@ extern NEWSGROUP	*NGfind();
 extern CHANNEL		*NCcreate();
 extern void		NGparsefile();
 extern BOOL		NGrenumber();
+extern BOOL		NGlowmark(NEWSGROUP *ngp, long lomark);
 
+extern void		NCclearwip(CHANNEL *cp);
 extern void		NCclose();
 extern void		NCsetup();
 
@@ -477,7 +552,10 @@ extern void		PROCunwatch();
 extern void		PROCscan();
 extern void		PROCsetup();
 
+extern int		RClimit();
+extern BOOL		RCnolimit();
 extern BOOL		RCauthorized();
+extern int		RCcanpost();
 extern char		*RChostname();
 extern void		RCclose();
 extern void		RChandoff();
@@ -487,6 +565,7 @@ extern void		RCsetup();
 extern BOOL		SITEfunnelpatch();
 extern BOOL		SITEsetup();
 extern BOOL		SITEwantsgroup();
+extern BOOL		SITEpoisongroup();
 extern char		**SITEreadfile();
 extern SITE		*SITEfind();
 extern SITE		*SITEfindnext();
@@ -495,9 +574,50 @@ extern void		SITEchanclose();
 extern void		SITEdrop();
 extern void		SITEflush();
 extern void		SITEflushall();
+extern void		SITEforward();
 extern void		SITEfree();
+extern void		SITEinfo();
 extern void		SITElinkall();
 extern void		SITEparsefile();
 extern void		SITEprocdied();
 extern void		SITEsend();
 extern void		SITEwrite();
+
+extern void		STATUSinit(void);
+extern void             STATUSmainloophook(void);
+
+extern void             TMRinit(void);
+extern void             TMRmainloophook(void);
+extern void             TMRstart(TMRTYPE t);
+extern void             TMRstop(TMRTYPE t);
+
+extern void             WIPsetup(void);
+extern WIP              *WIPnew(const char *messageid, CHANNEL *cp);
+extern void             WIPprecomfree(CHANNEL *cp);
+extern void             WIPfree(WIP *wp);
+extern BOOL             WIPinprogress(const char *msgid, CHANNEL *cp, const BOOL Add);
+extern WIP              *WIPbyid(const char *mesageid);
+extern WIP              *WIPbyhash(const HASH hash);
+
+/*
+**  TCL Globals
+*/
+
+#if defined(DO_TCL)
+extern Tcl_Interp      *TCLInterpreter;
+extern BOOL            TCLFilterActive;
+extern BUFFER          *TCLCurrArticle;
+extern ARTDATA         *TCLCurrData;
+#endif /* defined(DO_TCL) */
+
+
+/*
+**  TCL Functions
+*/
+
+#if defined(DO_TCL)
+extern void            TCLfilter();
+extern void            TCLreadfilter();
+extern void            TCLsetup();
+extern void            TCLclose();
+#endif /* defined(DO_TCL) */
