@@ -1,17 +1,19 @@
-/*  $Revision: 1.14 $
+/*  $Id: inndstart.c,v 1.14 1998/06/03 00:24:51 kondou Exp $
 **
 **  Open the privileged port, then exec innd.
 */
 #include <stdio.h>
+#include <unistd.h>
 #include <sys/types.h>
+#include <grp.h>
 #include "configdata.h"
-#include "clibrary.h"
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include "clibrary.h"
 #include <fcntl.h>
 #include "paths.h"
-#include "logging.h"
+#include <syslog.h> 
 #include "libinn.h"
 #include "macros.h"
 #if	NOFILE_LIMIT > 0
@@ -19,12 +21,12 @@
 #include <time.h>
 #endif	/* defined(DO_NEED_TIME) */
 #include <sys/time.h>
-#include <sys/resource.h>
 #endif	/* NOFILE_LIMIT > 0 */
+#include <sys/resource.h>
+#include <errno.h>
 
 /* #define DEBUGGER "/usr/ucb/dbx" */
 
-
 #if	NOFILE_LIMIT > 0
 /*
 **  Set the limit on the number of open files we can have.  I don't
@@ -49,12 +51,8 @@ SetDescriptorLimit(i)
 #endif	/* NOFILE_LIMIT > 0 */
 
 
-int
-main(ac, av)
-    int			ac;
-    char		*av[];
+int main(int ac, char *av[])
 {
-    static char		INNDDIR[] = _PATH_INNDDIR;
     GID_T		NewsGID;
     UID_T		NewsUID;
     struct sockaddr_in	server;
@@ -69,12 +67,15 @@ main(ac, av)
     char		buff[BUFSIZ];
     STRING		env[8];
     struct stat		Sb;
+    char		*inndpath;
 
     (void)openlog("inndstart", L_OPENLOG_FLAGS, LOG_INN_PROG);
 
+    if (ReadInnConf() < 0) exit(1);
+
     /* Make sure INND directory exists. */
-    if (stat(INNDDIR, &Sb) < 0 || !S_ISDIR(Sb.st_mode)) {
-	syslog(L_FATAL, "inndstart cant stat %s %m", INNDDIR);
+    if (stat(innconf->pathrun, &Sb) < 0 || !S_ISDIR(Sb.st_mode)) {
+	syslog(L_FATAL, "inndstart cant stat %s %m", innconf->pathrun);
 	exit(1);
     }
     NewsUID = Sb.st_uid;
@@ -98,9 +99,34 @@ main(ac, av)
 	syslog(L_ERROR, "inndstart cant setsockopt %m");
 #endif	/* defined(SO_REUSEADDR) */
     (void)memset((POINTER)&server, 0, sizeof server);
-    server.sin_port = htons(NNTP_PORT);
+    server.sin_port = htons(innconf->port);
+    for (j = 1; av[j]; j++) {
+	if (!strncmp("-P", av[j], 2)) {
+	    server.sin_port = htons(atoi(&av[j][2]));
+	    break;
+	}
+    }
     server.sin_family = AF_INET;
+
     server.sin_addr.s_addr = htonl(INADDR_ANY);
+    p = innconf->bindaddress;
+    if ((p != NULL) && !EQ(p, "all") && !EQ(p, "any")) {
+	server.sin_addr.s_addr = inet_addr(p);
+	if (server.sin_addr.s_addr == INADDR_NONE) {
+	    syslog(L_FATAL, "inndstart unable to determine bind ip (%s) %m", p);
+	    exit(1);
+	}
+    }
+    for (j = 1; av[j]; j++) {
+	if (!strncmp("-I", av[j], 2)) {
+	    server.sin_addr.s_addr = inet_addr(&av[j][2]);
+	    if (server.sin_addr.s_addr == INADDR_NONE) {
+		syslog(L_FATAL, "inndstart unable to determine bind ip (%s) %m",
+					&av[j][2]);
+	    	exit(1);
+	    }
+	}
+    }
     if (bind(i, (struct sockaddr *)&server, sizeof server) < 0) {
 	syslog(L_FATAL, "inndstart cant bind %m");
 	exit(1);
@@ -110,16 +136,18 @@ main(ac, av)
     /* Build the new argument vector. */
     argv = NEW(STRING, 2 + ac + 1);
     j = 0;
+    inndpath = cpcatpath(innconf->pathbin, "innd");
 #if	defined(DEBUGGER)
     argv[j++] = DEBUGGER;
-    argv[j++] = _PATH_INND;
+    argv[j++] = inndpath;
     argv[j] = NULL;
     (void)printf("Use -dp%d\n", i);
 #else
-    argv[j++] = _PATH_INND;
+    argv[j++] = inndpath;
     argv[j++] = pflag;
     for (i = 1; av[i]; ) {
-	if (strncmp(av[i], "-p", 2) != 0)
+	if ((strncmp(av[i], "-p", 2) != 0) && (strncmp(av[i], "-P", 2) != 0) &&
+		(strncmp(av[i], "-I", 2) != 0))
 	argv[j++] = av[i++];
 	else
 	    i++;
@@ -137,9 +165,9 @@ main(ac, av)
 
     /* Set up the environment. */
     (void)sprintf(buff, "PATH=%s:%s:/bin:/usr/bin:/usr/ucb",
-	    _PATH_NEWSBIN, _PATH_NEWSLIB);
+	    innconf->pathbin, innconf->pathetc);
     env[0] = COPY(buff);
-    (void)sprintf(buff, "TMPDIR=%s", _PATH_TMP);
+    (void)sprintf(buff, "TMPDIR=%s", innconf->pathtmp);
     env[1] = COPY(buff);
     (void)sprintf(buff, "SHELL=%s", _PATH_SH);
     env[2] = COPY(buff);
@@ -147,7 +175,7 @@ main(ac, av)
     env[3] = COPY(buff);
     (void)sprintf(buff, "USER=%s", NEWSMASTER);
     env[4] = COPY(buff);
-    (void)sprintf(buff, "HOME=%s", _PATH_NEWSHOME);
+    (void)sprintf(buff, "HOME=%s", innconf->pathnews);
     env[5] = COPY(buff);
     i = 6;
     /* linux uglyness */
@@ -166,4 +194,5 @@ main(ac, av)
     syslog(L_FATAL, "inndstart cant exec %s %m", argv[0]);
     _exit(1);
     /* NOTREACHED */
+    return(1);
 }
