@@ -1,18 +1,18 @@
-/*  $Revision: 1.51 $
+/*  $Id: nc.c,v 1.30 1998/06/03 05:56:18 kondou Exp $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #include "innd.h"
 #include "dbz.h"
-
+#include "qio.h"
 
 #define BAD_COMMAND_COUNT	10
-#define WIP_CHECK		(1 * 60)
 #define SAVE_AMT		10
 #define ART_EOF(c, s)		\
     ((c) >= 5 && (s)[-5] == '\r' && (s)[-4] == '\n' && (s)[-3] == '.' \
@@ -29,22 +29,6 @@ typedef struct _NCDISPATCH {
     int		Size;
 } NCDISPATCH;
 
-
-/*
-**  Information about the work in progress on all our open channels.
-*/
-typedef struct _WIP {
-    char	*MessageID;
-    long	Size;
-    time_t	Timestamp;
-    BUFFER	Replic;
-    BOOL	Wanted;
-} WIP;
-
-
-#if	0
-static FUNCTYPE	NCarticle();
-#endif	/* 0 */
 static FUNCTYPE	NCauthinfo();
 static FUNCTYPE	NChead();
 static FUNCTYPE	NChelp();
@@ -54,7 +38,6 @@ static FUNCTYPE	NCmode();
 static FUNCTYPE	NCquit();
 static FUNCTYPE	NCstat();
 static FUNCTYPE	NCxpath();
-static FUNCTYPE	NCxreplic();
 static FUNCTYPE	NC_unimp();
 /* new modules for streaming */
 static FUNCTYPE	NCxbatch();
@@ -63,10 +46,6 @@ static FUNCTYPE	NCtakethis();
 static FUNCTYPE NCwritedone();
 
 STATIC int		NCcount;	/* Number of open connections	*/
-STATIC int		NCwipsize;	/* Size of NCwip array		*/
-STATIC char		*NCfreelist[5];	/* Free string list		*/
-STATIC WIP		*NCwip;		/* Work-in-progress		*/
-STATIC WIP		NCnullwip;
 STATIC NCDISPATCH	NCcommands[] = {
 #if	0
     {	"article",	NCarticle },
@@ -94,38 +73,23 @@ STATIC NCDISPATCH	NCcommands[] = {
     {	"xbatch",	NCxbatch },
     {	"xhdr",		NC_unimp },
     {	"xpath",	NCxpath	},
-    {	"xreplic",	NCxreplic }
 };
 STATIC char		*NCquietlist[] = {
     INND_QUIET_BADLIST
 };
 STATIC char		NCterm[] = "\r\n";
-STATIC char		NCdotterm[] = ".\r\n";
 STATIC char 		NCdot[] = "." ;
 STATIC char		NCbadcommand[] = NNTP_BAD_COMMAND;
 STATIC STRING		NCgreeting;
 
-
 /*
-**  Clear the work-in-progress entry and wake up anyone who might
-**  have been waiting for us.
+** Clear the WIP entry for the given channel
 */
-STATIC void
-NCclearwip(wp)
-    register WIP	*wp;
-{
-    char		*p;
-
-    if ((p = wp->MessageID) != NULL)
-	*p = '\0';
-    wp->Size = 0;
-    if (wp->Wanted) {
-        wp->Wanted = FALSE;
-	SCHANwakeup((POINTER)wp);
-    }
+void NCclearwip(CHANNEL *cp) {
+    WIPfree(WIPbyhash(cp->CurrentMessageIDHash));
+    HashClear(&cp->CurrentMessageIDHash);
 }
 
-
 /*
 **  Write an NNTP reply message.
 **
@@ -140,12 +104,10 @@ NCclearwip(wp)
 **  *before* NCwritereply is called.
 */
 STATIC void
-NCwritereply(cp, text)
-    CHANNEL	*cp;
-    char	*text;
+NCwritereply(CHANNEL *cp, char *text)
 {
-    register BUFFER	*bp;
-    register int	i;
+    BUFFER	*bp;
+    int		i;
 
     /* XXX could do RCHANremove(cp) here, as the old NCwritetext() used to
      * do, but that would be wrong if the channel is sreaming (because it
@@ -180,9 +142,7 @@ NCwritereply(cp, text)
 **  Tell the NNTP channel to go away.
 */
 STATIC void
-NCwriteshutdown(cp, text)
-    CHANNEL	*cp;
-    char	*text;
+NCwriteshutdown(CHANNEL *cp, char *text)
 {
     cp->State = CSwritegoodbye;
     RCHANremove(cp); /* we're not going to read anything more */
@@ -204,9 +164,7 @@ NCwriteshutdown(cp, text)
 **  If a Message-ID is bad, write a reject message and return TRUE.
 */
 STATIC BOOL
-NCbadid(cp, p)
-    register CHANNEL	*cp;
-    register char	*p;
+NCbadid(CHANNEL *cp, char *p)
 {
     if (ARTidok(p))
 	return FALSE;
@@ -222,11 +180,9 @@ NCbadid(cp, p)
 **  not running, drop the article or just pause and reschedule.
 */
 STATIC void
-NCpostit(cp)
-    register CHANNEL	*cp;
+NCpostit(CHANNEL *cp)
 {
     STRING		response;
-    WIP			*wp;
 
     /* Note that some use break, some use return here. */
     switch (Mode) {
@@ -234,16 +190,16 @@ NCpostit(cp)
 	syslog(L_ERROR, "%s internal NCpostit mode %d", CHANname(cp), Mode);
 	return;
     case OMpaused:
-	SCHANadd(cp, (time_t)(Now.time + PAUSE_RETRY_TIME), (POINTER)&Mode,
-	    NCpostit, (POINTER)NULL);
+	SCHANadd(cp, (time_t)(Now.time + innconf->pauseretrytime),
+			(POINTER)&Mode, NCpostit, (POINTER)NULL);
 	return;
     case OMrunning:
-	wp = &NCwip[cp->fd];
-	response = ARTpost(cp, AmSlave ? &wp->Replic : NULL, wp->MessageID);
+	response = ARTpost(cp);
 	if (atoi(response) == NNTP_TOOKIT_VAL) {
 	    cp->Received++;
 	    if (cp->Sendid.Size > 3) { /* We be streaming */
 		char buff[4];
+		cp->Takethis_Ok++;
 		(void)sprintf(buff, "%d", NNTP_OK_RECID_VAL);
 		cp->Sendid.Data[0] = buff[0];
 		cp->Sendid.Data[1] = buff[1];
@@ -255,15 +211,19 @@ NCpostit(cp)
 	    if (cp->Sendid.Size) response = cp->Sendid.Data;
         }
 	cp->Reported++;
-	if (cp->Reported >= NNTP_ACTIVITY_SYNC) {
+	if (cp->Reported >= innconf->nntpactsync) {
 	    syslog(L_NOTICE,
 	    "%s checkpoint seconds %ld accepted %ld refused %ld rejected %ld",
 		CHANname(cp), (long)(Now.time - cp->Started),
 		cp->Received, cp->Refused, cp->Rejected);
 	    cp->Reported = 0;
 	}
+	if (Mode == OMthrottled) {
+	    NCwriteshutdown(cp, ModeReason);
+	    break;
+	}
 	cp->State = CSgetcmd;
-	NCwritereply(cp, response);
+	NCwritereply(cp, (char *)response);
 	break;
 
     case OMthrottled:
@@ -273,7 +233,7 @@ NCpostit(cp)
     }
 
     /* Clear the work-in-progress entry. */
-    NCclearwip(&NCwip[cp->fd]);
+    NCclearwip(cp);
 }
 
 
@@ -282,8 +242,7 @@ NCpostit(cp)
 **  read next.
 */
 STATIC FUNCTYPE
-NCwritedone(cp)
-    register CHANNEL	*cp;
+NCwritedone(CHANNEL *cp)
 {
     switch (cp->State) {
     default:
@@ -309,55 +268,14 @@ NCwritedone(cp)
 
 
 
-#if	0
-/*
-**  The "article" command.
-*/
-STATIC FUNCTYPE
-NCarticle(cp)
-    register CHANNEL	*cp;
-{
-    register char	*p;
-    register char	*q;
-    char		*art;
-
-    /* Snip off the Message-ID. */
-    for (p = cp->In.Data + STRLEN("article"); ISWHITE(*p); p++)
-	continue;
-    if (NCbadid(cp, p))
-	return;
-
-    /* Get the article filenames, and the article header+body. */
-    if ((art = ARTreadarticle(HISfilesfor(p))) == NULL) {
-	NCwritereply(cp, NNTP_DONTHAVEIT);
-	return;
-    }
-
-    /* Write it. */
-    NCwritereply(cp, NNTP_ARTICLE_FOLLOWS);
-    for (p = art; ((q = strchr(p, '\n')) != NULL); p = q + 1) {
-	if (*p == '.')
-	    WCHANappend(cp, ".", 1);
-	WCHANappend(cp, p, q - p);
-	WCHANappend(cp, NCterm, STRLEN(NCterm));
-    }
-
-    /* Write the terminator. */
-    WCHANappend(cp, NCdotterm, STRLEN(NCdotterm));
-}
-#endif	/* 0 */
-
-
 /*
 **  The "head" command.
 */
-STATIC FUNCTYPE
-NChead(cp)
-    CHANNEL		*cp;
+STATIC FUNCTYPE NChead(CHANNEL *cp)
 {
-    register char	*p;
-    register char	*q;
-    char		*head;
+    char	        *p;
+    char	        *q;
+    QIOSTATE            *qp;
 
     /* Snip off the Message-ID. */
     for (p = cp->In.Data + STRLEN("head"); ISWHITE(*p); p++)
@@ -365,38 +283,44 @@ NChead(cp)
     if (NCbadid(cp, p))
 	return;
 
-    /* Get the article filenames, and the header. */
-    if ((head = ARTreadheader(HISfilesfor(p))) == NULL) {
+    /* Get the article filenames; open the first file */
+    if ((q = HISfilesfor(HashMessageID(p))) == NULL) {
+	NCwritereply(cp, NNTP_DONTHAVEIT);
+	return;
+    }
+    if ((p = strchr(q, ' ')))
+	*p = '\0';
+    
+    if ((qp = QIOopen(q)) == NULL) {
 	NCwritereply(cp, NNTP_DONTHAVEIT);
 	return;
     }
 
     /* Write it. */
-    NCwritereply(cp, NNTP_HEAD_FOLLOWS);
-    for (p = head; ((q = strchr(p, '\n')) != NULL); p = q + 1) {
+    WCHANappend(cp, NNTP_HEAD_FOLLOWS, STRLEN(NNTP_HEAD_FOLLOWS));
+    WCHANappend(cp, NCterm, STRLEN(NCterm));
+    for (p = QIOread(qp); (p != NULL) && (*p != '\0'); p = QIOread(qp)) {
 	if (*p == '.')
 	    WCHANappend(cp, ".", 1);
-	WCHANappend(cp, p, q - p);
+	WCHANappend(cp, p, QIOlength(qp));
 	WCHANappend(cp, NCterm, STRLEN(NCterm));
     }
 
     /* Write the terminator. */
-    WCHANappend(cp, NCdotterm, STRLEN(NCdotterm));
+    NCwritereply(cp, NCdot);
+    QIOclose(qp);
 }
 
 
 /*
 **  The "stat" command.
 */
-STATIC FUNCTYPE
-NCstat(cp)
-    CHANNEL		*cp;
+STATIC FUNCTYPE NCstat(CHANNEL *cp)
 {
-    register char	*p;
-    /* mibsoft 97-Aug-12 If DBZMAXKEY is changed to be much different than SMBUF,
-       then better bump up SMBUF!
-     */
-    char		buff[SMBUF+16];
+    char	        *p;
+    char                *q;
+    char		*buff;
+    QIOSTATE            *qp;
 
     /* Snip off the Message-ID. */
     for (p = cp->In.Data + STRLEN("stat"); ISWHITE(*p); p++)
@@ -404,17 +328,26 @@ NCstat(cp)
     if (NCbadid(cp, p))
 	return;
 
-    /* Get the article filenames; read the header (to make sure not
+    /* Get the article filenames; open the first file (to make sure
      * the article is still here). */
-    if (ARTreadheader(HISfilesfor(p)) == NULL) {
+    if ((buff = HISfilesfor(HashMessageID(p))) == NULL) {
 	NCwritereply(cp, NNTP_DONTHAVEIT);
 	return;
     }
+    if ((q = strchr(buff, ' ')))
+	*q = '\0';
+    
+    if ((qp = QIOopen(buff)) == NULL) {
+	NCwritereply(cp, NNTP_DONTHAVEIT);
+	return;
+    }
+    QIOclose(qp);
 
     /* Write the message. */
-	/* We know there is enough room as long as DBZMAXKEY < SMBUF+16-7 */
+    buff = NEW(char, strlen(p) + 16);
     (void)sprintf(buff, "%d 0 %s", NNTP_NOTHING_FOLLOWS_VAL, p);
     NCwritereply(cp, buff);
+    DISPOSE(buff);
 }
 
 
@@ -423,13 +356,12 @@ NCstat(cp)
 **  channel is in CSgetauth state and we just got a command.
 */
 STATIC FUNCTYPE
-NCauthinfo(cp)
-    register CHANNEL	*cp;
+NCauthinfo(CHANNEL *cp)
 {
     static char		AUTHINFO[] = "authinfo ";
     static char		PASS[] = "pass ";
     static char		USER[] = "user ";
-    register char	*p;
+    char		*p;
 
     p = cp->In.Data;
 
@@ -473,58 +405,21 @@ NCauthinfo(cp)
     }
 }
 
-
-/*
-**  Is someone already sending us this article?
-*/
-STATIC BOOL
-NCinprogress(cp, id, who)
-    CHANNEL		*cp;
-    register char	*id;
-    WIP			**who;
-{
-    register WIP	*wp;
-    register char	*p;
-    register int	i;
-
-    for (i = NCwipsize, wp = NCwip; --i >= 0; wp++)
-	if ((p = wp->MessageID) != NULL && *p == *id && EQ(p, id)
-	 && Now.time - wp->Timestamp < WIP_CHECK) {
-	    *who = wp;
-	    return TRUE;
-	}
-    wp = &NCwip[cp->fd];
-    if (wp->MessageID == NULL) {
-	for (i = SIZEOF(NCfreelist); --i >= 0; )
-	    if (NCfreelist[i] != NULL) {
-		wp->MessageID = NCfreelist[i];
-		NCfreelist[i] = NULL;
-		break;
-	    }
-	if (i < 0)
-	    wp->MessageID = NEW(char, DBZMAXKEY + 3);
-    }
-    (void)strcpy(wp->MessageID, id);
-    return FALSE;
-}
-
-
 /*
 **  The "help" command.
 */
 STATIC FUNCTYPE
-NChelp(cp)
-    register CHANNEL	*cp;
+NChelp(CHANNEL *cp)
 {
     static char		LINE1[] = "For more information, contact \"";
     static char		LINE2[] = "\" at this machine.";
-    register NCDISPATCH	*dp;
+    NCDISPATCH		*dp;
 
     WCHANappend(cp, NNTP_HELP_FOLLOWS,STRLEN(NNTP_HELP_FOLLOWS));
     WCHANappend(cp, NCterm,STRLEN(NCterm));
     for (dp = NCcommands; dp < ENDOF(NCcommands); dp++)
 	if (dp->Function != NC_unimp) {
-            if (!StreamingOff || cp->Streaming ||
+            if ((!StreamingOff && cp->Streaming) ||
                 (dp->Function != NCcheck && dp->Function != NCtakethis)) {
                 WCHANappend(cp, "\t", 1);
                 WCHANappend(cp, dp->Name, dp->Size);
@@ -538,122 +433,64 @@ NChelp(cp)
     NCwritereply(cp, NCdot) ;
 }
 
-
-#if	!defined(NNTP_RESENDIT_LATER)
-/*
-**  We woke up because we got offered an article that was already in
-**  progress somewhere else.  If the other channel finished, then we
-**  don't want the article, otherwise let's accept it.
-*/
-STATIC FUNCTYPE
-NCwaitfor(cp)
-    register CHANNEL	*cp;
-{
-    WIP			*who;
-
-    if (HIShavearticle(cp->Argument)) {
-	cp->Refused++;
-	NCwritereply(cp, NNTP_HAVEIT);
-	DISPOSE(cp->Argument);
-	cp->Argument = NULL;
-    }
-    else if (NCinprogress(cp, cp->Argument, &who)) {
-	who->Wanted = TRUE;
-	SCHANadd(cp, (time_t)(Now.time + WIP_CHECK / 2 + 1), (POINTER)who,
-	    NCwaitfor, (POINTER)cp->Argument);
-    }
-    else {
-	cp->State = CSgetarticle;
-	NCwritereply(cp, NNTP_SENDIT);
-	DISPOSE(cp->Argument);
-	cp->Argument = NULL;
-    }
-}
-#endif	/* !defined(NNTP_RESENDIT_LATER) */
-
-
 /*
 **  The "ihave" command.  Check the Message-ID, and see if we want the
 **  article or not.  Set the state appropriately.
 */
 STATIC FUNCTYPE
-NCihave(cp)
-    CHANNEL		*cp;
+NCihave(CHANNEL *cp)
 {
-    register char	*p;
-    WIP			*who;
-
-    if (AmSlave) {
-	NCwritereply(cp, NCbadcommand);
-	return;
-    }
+    char	*p;
 
+    cp->Ihave++;
     /* Snip off the Message-ID. */
     for (p = cp->In.Data + STRLEN("ihave"); ISWHITE(*p); p++)
 	continue;
     if (NCbadid(cp, p))
 	return;
 
-    if (HIShavearticle(p)) {
+    if (HIShavearticle(HashMessageID(p))) {
 	cp->Refused++;
+	cp->Ihave_Duplicate++;
 	NCwritereply(cp, NNTP_HAVEIT);
     }
-    else if (NCinprogress(cp, p, &who)) {
-#if	defined(NNTP_RESENDIT_LATER)
+    else if (WIPinprogress(p, cp, FALSE)) {
+	cp->Ihave_Deferred++;
 	NCwritereply(cp, NNTP_RESENDIT_LATER);
-#else
-	/* Somebody else is sending it to us; wait until they're done. */
-	who->Wanted = TRUE;
-	SCHANadd(cp, (time_t)(Now.time + WIP_CHECK + 1), (POINTER)who,
-	    NCwaitfor, (POINTER)COPY(p));
-	/* Clear input buffer. */
-	cp->In.Used = 0;
-#endif	/* defined(NNTP_RESENDIT_LATER) */
     }
     else {
-	cp->State = CSgetarticle;
+	cp->Ihave_SendIt++;
 	NCwritereply(cp, NNTP_SENDIT);
+	cp->State = CSgetarticle;
     }
 }
 
-/* The "xbatch" command. We (ab)use WIP->Size to memorize the batch size.
-** Set the state appropriately.
+/* 
+** The "xbatch" command. Set the state appropriately.
 */
 
 STATIC FUNCTYPE
-NCxbatch(cp)
-    CHANNEL		*cp;
+NCxbatch(CHANNEL *cp)
 {
-    register char	*p;
-    WIP			*wp;
-
-    if (AmSlave) {
-	NCwritereply(cp, NCbadcommand);
-	return;
-    }
+    char	*p;
 
     /* Snip off the batch size */
     for (p = cp->In.Data + STRLEN("xbatch"); ISWHITE(*p); p++)
 	continue;
 
-    wp = &NCwip[cp->fd];
-
-    /* safety checks, unnecessary once we are running stable */
-    if (!wp)
-        syslog(L_FATAL, "NCxbatch(): oops, wp == NULL");
-    if (wp->Size) {
-        syslog(L_FATAL, "NCxbatch(): oops, wp->Size already set to %ld",
-	       wp->Size);
+    if (cp->XBatchSize) {
+        syslog(L_FATAL, "NCxbatch(): oops, cp->XBatchSize already set to %ld",
+	       cp->XBatchSize);
     }
 
-    wp->Size = atoi(p);
+    cp->XBatchSize = atoi(p);
     if (Tracing || cp->Tracing)
         syslog(L_TRACE, "%s will read batch of size %ld",
-	       CHANname(cp), wp->Size);
+	       CHANname(cp), cp->XBatchSize);
 
-    if (wp->Size <= 0) {
+    if (cp->XBatchSize <= 0) {
         syslog(L_NOTICE, "%s got bad xbatch size %ld",
-	       CHANname(cp), wp->Size);
+	       CHANname(cp), cp->XBatchSize);
 	NCwritereply(cp, NNTP_XBATCH_BADSIZE);
 	return;
     }
@@ -668,9 +505,9 @@ NCxbatch(cp)
      * LOW_WATER + 1 more than needed, so that CHANreadtext() will never
      * reallocate it.
      */
-    if (cp->In.Size < wp->Size + LOW_WATER + 1) {
+    if (cp->In.Size < cp->XBatchSize + LOW_WATER + 1) {
         DISPOSE(cp->In.Data);
-	cp->In.Left = cp->In.Size = wp->Size + LOW_WATER + 1;
+	cp->In.Left = cp->In.Size = cp->XBatchSize + LOW_WATER + 1;
 	cp->In.Used = 0;
 	cp->In.Data = NEW(char, cp->In.Size);
     }
@@ -683,22 +520,23 @@ NCxbatch(cp)
 **  The "list" command.  Send the active file.
 */
 STATIC FUNCTYPE
-NClist(cp)
-    register CHANNEL	*cp;
+NClist(CHANNEL *cp)
 {
-    register char	*p;
-    register char	*q;
+    char		*p;
+    char		*q;
     char		*trash;
     char		*end;
 
     for (p = cp->In.Data + STRLEN("list"); ISWHITE(*p); p++)
 	continue;
     if (caseEQ(p, "newsgroups")) {
-	trash = p = ReadInFile(_PATH_NEWSGROUPS, (struct stat *)NULL);
+	trash = p = ReadInFile(cpcatpath(innconf->pathdb, _PATH_NEWSGROUPS),
+			(struct stat *)NULL);
 	end = p + strlen(p);
     }
     else if (caseEQ(p, "active.times")) {
-	trash = p = ReadInFile(_PATH_ACTIVETIMES, (struct stat *)NULL);
+	trash = p = ReadInFile(cpcatpath(innconf->pathdb, _PATH_ACTIVETIMES),
+			(struct stat *)NULL);
 	end = p + strlen(p);
     }
     else if (*p == '\0' || (caseEQ(p, "active"))) {
@@ -727,10 +565,9 @@ NClist(cp)
 **  The "mode" command.  Hand off the channel.
 */
 STATIC FUNCTYPE
-NCmode(cp)
-    register CHANNEL	*cp;
+NCmode(CHANNEL *cp)
 {
-    register char	*p;
+    char		*p;
     HANDOFF		h;
 
     /* Skip the first word, get the argument. */
@@ -739,10 +576,8 @@ NCmode(cp)
 
     if (caseEQ(p, "reader"))
 	h = HOnnrpd;
-    else if (caseEQ(p, "query"))
-	h = HOnnrqd;
     else if (caseEQ(p, "stream") &&
-             (!StreamingOff || (StreamingOff && cp->Streaming))) {
+             (!StreamingOff && cp->Streaming)) {
 	char buff[16];
 	(void)sprintf(buff, "%d StreamOK.", NNTP_OK_STREAM_VAL);
 	NCwritereply(cp, buff);
@@ -763,26 +598,8 @@ NCmode(cp)
 /*
 **  The "quit" command.  Acknowledge, and set the state to closing down.
 */
-STATIC FUNCTYPE
-NCquit(cp)
-    CHANNEL		*cp;
+STATIC FUNCTYPE NCquit(CHANNEL *cp)
 {
-    register WIP	*wp;
-    register int	i;
-
-    wp = &NCwip[cp->fd];
-    for (i = SIZEOF(NCfreelist); --i >= 0; )
-	if (NCfreelist[i] == NULL) {
-	    NCfreelist[i] = wp->MessageID;
-	    wp->MessageID = NULL;
-	    break;
-	}
-#if	0
-    if (i < 0) {
-	DISPOSE(wp->MessageID);
-	wp->MessageID = NULL;
-    }
-#endif	/* 0 */
     cp->State = CSwritegoodbye;
     NCwritereply(cp, NNTP_GOODBYE_ACK);
 }
@@ -792,12 +609,11 @@ NCquit(cp)
 **  The "xpath" command.  Return the paths for an article is.
 */
 STATIC FUNCTYPE
-NCxpath(cp)
-    CHANNEL		*cp;
+NCxpath(CHANNEL *cp)
 {
     static BUFFER	Reply;
-    register char	*p;
-    register int	i;
+    char		*p;
+    int			i;
 
     /* Nip off the Message-ID. */
     for (p = cp->In.Data + STRLEN("xpath"); ISWHITE(*p); p++)
@@ -805,7 +621,7 @@ NCxpath(cp)
     if (NCbadid(cp, p))
 	return;
 
-    if ((p = HISfilesfor(p)) == NULL) {
+    if ((p = HISfilesfor(HashMessageID(p))) == NULL) {
 	NCwritereply(cp, NNTP_DONTHAVEIT);
 	return;
     }
@@ -822,49 +638,13 @@ NCxpath(cp)
     NCwritereply(cp, Reply.Data);
 }
 
-
-/*
-**  The "xreplic" command.  Take an article and the places to file it.
-*/
-STATIC FUNCTYPE
-NCxreplic(cp)
-    CHANNEL		*cp;
-{
-    register char	*p;
-    register BUFFER	*bp;
-    register int	i;
-
-    if (!RCismaster(cp->Address)) {
-	NCwritereply(cp, NCbadcommand);
-	return;
-    }
-
-    /* Stash the filename arguments. */
-    for (p = cp->In.Data + STRLEN("xreplic"); ISWHITE(*p); p++)
-	continue;
-    i = cp->In.Used - (p - cp->In.Data) + 1;
-    bp = &NCwip[cp->fd].Replic;
-    if (bp->Data == NULL) {
-	bp->Size = i;
-	bp->Data = NEW(char, i);
-    }
-    BUFFset(bp, p, i);
-    bp->Used = bp->Left;
-
-    /* Tell master to send it to us. */
-    cp->State = CSgetrep;
-    NCwritereply(cp, NNTP_SENDIT);
-}
-
-
 /*
 **  The catch-all for inimplemented commands.
 */
 STATIC FUNCTYPE
-NC_unimp(cp)
-    CHANNEL		*cp;
+NC_unimp(CHANNEL *cp)
 {
-    register char	*p;
+    char		*p;
     char		buff[SMBUF];
 
     /* Nip off the first word. */
@@ -882,12 +662,11 @@ NC_unimp(cp)
 **  Remove the \r\n and leading dot escape that the NNTP protocol adds.
 */
 STATIC void
-NCclean(bp)
-    register BUFFER	*bp;
+NCclean(BUFFER *bp)
 {
-    register char	*end;
-    register char	*p;
-    register char	*dest;
+    char	*end;
+    char	*p;
+    char	*dest;
 
     for (p = bp->Data, dest = p, end = p + bp->Used; p < end; ) {
 	if (p[0] == '\r' && p[1] == '\n') {
@@ -910,17 +689,12 @@ NCclean(bp)
 **  Check whatever data is available on the channel.  If we got the
 **  full amount (i.e., the command or the whole article) process it.
 */
-STATIC FUNCTYPE
-NCproc(cp)
-    register CHANNEL	*cp;
+STATIC FUNCTYPE NCproc(CHANNEL *cp)
 {
-    register char	*p;
-    register NCDISPATCH	*dp;
-    register BUFFER	*bp;
-    register WIP	*wp;
-    STRING		q;
+    char	        *p;
+    NCDISPATCH   	*dp;
+    BUFFER	        *bp;
     char		buff[SMBUF];
-    char		*av[2];
     int			i;
 
     if (Tracing || cp->Tracing)
@@ -928,10 +702,10 @@ NCproc(cp)
 	    CHANname(cp), cp->In.Used);
 
     bp = &cp->In;
-    if (bp->Used == 0) return;
+    if (bp->Used == 0)
+	return;
 
     p = &bp->Data[bp->Used];
-    wp = &NCwip[cp->fd];
     for ( ; ; ) {
 	cp->Rest = 0;
 	cp->SaveUsed = bp->Used;
@@ -958,9 +732,16 @@ NCproc(cp)
 	    p = &bp->Data[cp->Rest];
 	    if (p[-2] != '\r' || p[-1] != '\n') { /* probably in an article */
 		int j;
+		char *tmpstr;
 
+		tmpstr = NEW(char, bp->Used);
+		memcpy(tmpstr, bp->Data, bp->Used);
+		tmpstr[bp->Used] = '\0';
+		
 		syslog(L_NOTICE, "%s bad_command %s",
-		    CHANname(cp), MaxLength(bp->Data, bp->Data));
+		    CHANname(cp), MaxLength(tmpstr, tmpstr));
+		DISPOSE(tmpstr);
+
 		if (++(cp->BadCommands) >= BAD_COMMAND_COUNT) {
 		    cp->State = CSwritegoodbye;
 		    NCwritereply(cp, NCbadcommand);
@@ -1041,12 +822,10 @@ NCproc(cp)
 	     && (bp->Data[1] == '\r') && (bp->Data[2] == '\n')) {
 		cp->Rest = 3;	/* null article (canceled?) */
 		cp->Rejected++;
-#if 0
-                syslog(L_NOTICE, "%s empty article", CHANname(cp)) ;
-#endif
 		cp->State = CSgetcmd;
 		if (cp->Sendid.Size > 3) { /* We be streaming */
 		    char buff[4];
+		    cp->Takethis_Err++;
 		    (void)sprintf(buff, "%d", NNTP_ERR_FAILID_VAL);
 		    cp->Sendid.Data[0] = buff[0];
 		    cp->Sendid.Data[1] = buff[1];
@@ -1057,7 +836,7 @@ NCproc(cp)
 		bp->Used = 0;
 
 		/* Clear the work-in-progress entry. */
-		NCclearwip(wp);
+		NCclearwip(cp);
 		break;
 	    }
 	    /* Reading an article; look for "\r\n.\r\n" terminator. */
@@ -1077,11 +856,11 @@ NCproc(cp)
 	    cp->Lastch = i;
 	    if (i > bp->Used) {	/* did not find terminator */
 		/* Check for big articles. */
-		if (LargestArticle > SAVE_AMT && bp->Used > LargestArticle) {
+		if (innconf->maxartsize > SAVE_AMT && bp->Used > innconf->maxartsize) {
 		    /* Make some room, saving only the last few bytes. */
 		    for (p = bp->Data, i = 0; i < SAVE_AMT; p++, i++)
 			p[0] = p[bp->Used - SAVE_AMT];
-		    wp->Size += bp->Used - SAVE_AMT;
+		    cp->LargeArtSize += bp->Used - SAVE_AMT;
 		    bp->Used = cp->Lastch = SAVE_AMT;
 		    cp->State = CSeatarticle;
 		}
@@ -1091,7 +870,7 @@ NCproc(cp)
 	    if (Mode == OMpaused) { /* defer processing while paused */
 		cp->Rest = 0;
 		bp->Used = cp->SaveUsed;
-		SCHANadd(cp, (time_t)(Now.time + PAUSE_RETRY_TIME),
+		SCHANadd(cp, (time_t)(Now.time + innconf->pauseretrytime),
 		    (POINTER)&Mode, NCproc, (POINTER)NULL);
 		return;
 	    }
@@ -1104,8 +883,11 @@ NCproc(cp)
 		DISPOSE(cp->Argument);
 		cp->Argument = NULL;
 	    }
-	    NCclean(bp);
+	    if (!innconf->wireformat)
+  	        NCclean(bp);
 	    NCpostit(cp);
+	    if (cp->State == CSwritegoodbye)
+		break;
 	    cp->State = CSgetcmd;
 	    break;
 
@@ -1132,27 +914,27 @@ NCproc(cp)
 		    DISPOSE(cp->Argument);
 		    cp->Argument = NULL;
 		}
-		p = wp->MessageID;
-		i = wp->Size + bp->Used;
-		syslog(L_ERROR, "%s internal rejecting huge article %s (%d > %d)",
-		    CHANname(cp), p ? p : "(null)", i, LargestArticle);
+		i = cp->LargeArtSize + bp->Used;
+		syslog(L_ERROR, "%s internal rejecting huge article (%d > %d)",
+		    CHANname(cp), i, innconf->maxartsize);
+		cp->LargeArtSize = 0;
 		(void)sprintf(buff, "%d Article exceeds local limit of %ld bytes",
-			NNTP_REJECTIT_VAL, LargestArticle);
+			NNTP_REJECTIT_VAL, innconf->maxartsize);
 		cp->State = CSgetcmd;
-		if (cp->Sendid.Size) NCwritereply(cp, cp->Sendid.Data);
-		else NCwritereply(cp, buff);
+		if (cp->Sendid.Size)
+		    NCwritereply(cp, cp->Sendid.Data);
+		else
+		    NCwritereply(cp, buff);
 		cp->Rejected++;
 
 		/* Write a local cancel entry so nobody else gives it to us. */
-		if (p) {
-		    av[0] = p;
-		    av[1] = NULL;
-		    if ((q = CCcancel(av)) != NULL)
-			syslog(L_ERROR, "%s cant cancel %s %s", LogName, av[0], q);
-		}
+		    if (!HISremember(cp->CurrentMessageIDHash)) 
+		    	if (!HIShavearticle(cp->CurrentMessageIDHash))
+			    syslog(L_ERROR, "%s cant cancel %s", LogName, 
+			        HashToText(cp->CurrentMessageIDHash)); 
 
 		/* Clear the work-in-progress entry. */
-		NCclearwip(wp);
+		NCclearwip(cp);
 
 		/*
 		 * only free and allocate the buffer back to
@@ -1176,7 +958,7 @@ NCproc(cp)
 		/* Make some room; save the last few bytes of the article */
 		for (p = bp->Data, i = 0; i < SAVE_AMT; p++, i++)
 		    p[0] = p[bp->Used - SAVE_AMT + 0];
-		wp->Size += bp->Used - SAVE_AMT;
+		cp->LargeArtSize += bp->Used - SAVE_AMT;
 		bp->Used = cp->Lastch = SAVE_AMT;
 		cp->Rest = 0;
 	    }
@@ -1187,13 +969,13 @@ NCproc(cp)
 	    */
 	    if (Tracing || cp->Tracing)
 		syslog(L_TRACE, "%s CSgetxbatch: now %ld of %ld bytes",
-			CHANname(cp), bp->Used, wp->Size);
+			CHANname(cp), bp->Used, cp->XBatchSize);
 
-	    if (bp->Used < wp->Size) {
+	    if (bp->Used < cp->XBatchSize) {
 		cp->Rest = 0;
 		break;	/* give us more data */
 	    }
-	    cp->Rest = wp->Size;
+	    cp->Rest = cp->XBatchSize;
 
 	    /* now do something with the batch */
 	    {
@@ -1204,7 +986,8 @@ NCproc(cp)
 		now = time(NULL);
 		failed = 0;
 		/* time+channel file descriptor should make an unique file name */
-		sprintf(buff, "%s/%ld%d.tmp", _PATH_XBATCHES, now, cp->fd);
+		sprintf(buff, "%s/%ld%d.tmp", innconf->pathincoming,
+						now, cp->fd);
 		fd = open(buff, O_WRONLY|O_CREAT|O_EXCL, ARTFILE_MODE);
 		if (fd < 0) {
 		    oerrno = errno;
@@ -1215,7 +998,7 @@ NCproc(cp)
 			    NNTP_RESENDIT_XBATCHERR, strerror(oerrno));
 		    NCwritereply(cp, buff);
 		} else {
-		    if (write(fd, cp->In.Data, wp->Size) != wp->Size) {
+		    if (write(fd, cp->In.Data, cp->XBatchSize) != cp->XBatchSize) {
 			oerrno = errno;
 			syslog(L_ERROR, "%s cant write batch to file %s: %m",
 				CHANname(cp), buff);
@@ -1234,7 +1017,8 @@ NCproc(cp)
 		    NCwritereply(cp, buff);
 		    failed = 1;
 		}
-		sprintf(buff2, "%s/%ld%d.x", _PATH_XBATCHES, now, cp->fd);
+		sprintf(buff2, "%s/%ld%d.x", innconf->pathincoming,
+						now, cp->fd);
 		if (rename(buff, buff2)) {
 		    oerrno = errno;
 		    syslog(L_ERROR, "%s cant rename %s to %s: %m",
@@ -1251,10 +1035,12 @@ NCproc(cp)
 		} else
 		    cp->Rejected++;
 	    }
-	    syslog(L_NOTICE, "%s accepted batch size %ld", CHANname(cp), wp->Size);
+	    syslog(L_NOTICE, "%s accepted batch size %ld",
+		   CHANname(cp), cp->XBatchSize);
+	    cp->State = CSgetcmd;
 	    
 	    /* Clear the work-in-progress entry. */
-	    NCclearwip(wp);
+	    NCclearwip(cp);
 
 #if 1
 	    /* leave the input buffer as it is, there is a fair chance
@@ -1269,8 +1055,11 @@ NCproc(cp)
 	    bp->Used = 0;
 	    bp->Data = NEW(char, bp->Size);
 #endif
+	    cp->State = CSgetcmd;
 	    break;
 	}
+	if (cp->State == CSwritegoodbye)
+	    break;
 	if (Tracing || cp->Tracing)
 		syslog(L_TRACE, "%s NCproc Rest=%d Used=%d SaveUsed=%d",
 		    CHANname(cp), cp->Rest, bp->Used, cp->SaveUsed);
@@ -1282,7 +1071,7 @@ NCproc(cp)
 		** would require changes to the bp structure and
 		** the way it is used.
 		*/
-		(void)memcpy((POINTER)bp->Data, (POINTER)&bp->Data[cp->Rest], (SIZE_T)bp->Used);
+		(void)memmove((POINTER)bp->Data, (POINTER)&bp->Data[cp->Rest], (SIZE_T)bp->Used);
 		cp->Rest = cp->Lastch = 0;
 	    } else {
 		bp->Used = cp->Lastch = 0;
@@ -1298,10 +1087,8 @@ NCproc(cp)
 **  full amount (i.e., the command or the whole article) process it.
 */
 STATIC FUNCTYPE
-NCreader(cp)
-    register CHANNEL	*cp;
+NCreader(CHANNEL *cp)
 {
-    register WIP	*wp;
     int			i;
 
     if (Tracing || cp->Tracing)
@@ -1317,7 +1104,7 @@ NCreader(cp)
             return;
         }
 #endif
-	if (i == 0 || cp->BadReads++ >= BAD_IO_COUNT) {
+	if (i == 0 || cp->BadReads++ >= innconf->badiocount) {
 	    if (NCcount > 0)
 		NCcount--;
 	    CHANclose(cp, CHANname(cp));
@@ -1325,10 +1112,6 @@ NCreader(cp)
 	return;
     }
 
-    /* Update timestamp. */
-    wp = &NCwip[cp->fd];
-    wp->Timestamp = Now.time;
-
     NCproc(cp);	    /* check and process data */
 }
 
@@ -1338,26 +1121,21 @@ NCreader(cp)
 **  Set up the NNTP channel state.
 */
 void
-NCsetup(i)
-    register int	i;
+NCsetup(int i)
 {
-    register WIP	*wp;
-    register NCDISPATCH	*dp;
+    NCDISPATCH		*dp;
     char		*p;
     char		buff[SMBUF];
 
     /* Set the greeting message. */
-    if ((p = GetConfigValue(_CONF_PATHHOST)) == NULL)
+    p = innconf->pathhost;
+    if (p == NULL)
 	/* Worked in main, now it fails?  Curious. */
 	p = Path.Data;
     (void)sprintf(buff, "%d %s InterNetNews server %s ready",
 	    NNTP_POSTOK_VAL, p, Version);
     NCgreeting = COPY(buff);
 
-    /* Set up the work-in-progress structure. */
-    for (wp = NCwip = NEW(WIP, i), NCwipsize = i; --i >= 0; wp++)
-	*wp = NCnullwip;
-
     /* Get the length of every command. */
     for (dp = NCcommands; dp < ENDOF(NCcommands); dp++)
 	dp->Size = strlen(dp->Name);
@@ -1368,11 +1146,9 @@ NCsetup(i)
 **  Tear down our state.
 */
 void
-NCclose()
+NCclose(void)
 {
-    register WIP	*wp;
-    register int	i;
-    register CHANNEL	*cp;
+    CHANNEL		*cp;
     int			j;
 
     /* Close all incoming channels. */
@@ -1381,18 +1157,6 @@ NCclose()
 	    NCcount--;
 	CHANclose(cp, CHANname(cp));
     }
-
-    /* Free the WIP list. */
-    for (wp = NCwip, i = NCwipsize; --i >= 0; wp++) {
-	if (wp->MessageID)
-	    DISPOSE(wp->MessageID);
-	if (wp->Replic.Data)
-	    DISPOSE(wp->Replic.Data);
-    }
-    DISPOSE(NCwip);
-    for (i = SIZEOF(NCfreelist); --i >= 0; )
-	if (NCfreelist[i] != NULL)
-	    DISPOSE(NCfreelist[i]);
 }
 
 
@@ -1400,42 +1164,33 @@ NCclose()
 **  Create an NNTP channel and print the greeting message.
 */
 CHANNEL *
-NCcreate(fd, MustAuthorize, IsLocal)
-    int			fd;
-    BOOL		MustAuthorize;
-    BOOL		IsLocal;
+NCcreate(int fd, BOOL MustAuthorize, BOOL IsLocal)
 {
-    register CHANNEL	*cp;
+    CHANNEL		*cp;
     int			i;
 
     /* Create the channel. */
     cp = CHANcreate(fd, CTnntp, MustAuthorize ? CSgetauth : CSgetcmd,
 	    NCreader, NCwritedone);
 
-    /* check for wip overflow */
-    if(fd > NCwipsize) {
-      NCwriteshutdown(cp, "too many file descriptors");
-      return cp;
-    }
-
-    NCclearwip(&NCwip[cp->fd]);
+    NCclearwip(cp);
 #if defined(SOL_SOCKET) && defined(SO_SNDBUF) && defined(SO_RCVBUF) 
-#if defined (DO_SET_SOCKOPT)
-    i = 24 * 1024;
-    if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (char *)&i, sizeof i) < 0)
-	syslog(L_ERROR, "%s cant setsockopt(SNDBUF) %m", CHANname(cp));
-    if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (char *)&i, sizeof i) < 0)
-	syslog(L_ERROR, "%s cant setsockopt(RCVBUF) %m", CHANname(cp));
-#endif
+    if (!IsLocal) {
+	i = 24 * 1024;
+	if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (char *)&i, sizeof i) < 0)
+	    syslog(L_ERROR, "%s cant setsockopt(SNDBUF) %m", CHANname(cp));
+	if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (char *)&i, sizeof i) < 0)
+	    syslog(L_ERROR, "%s cant setsockopt(RCVBUF) %m", CHANname(cp));
+    }
 #endif	/* defined(SOL_SOCKET) && defined(SO_SNDBUF) && defined(SO_RCVBUF) */
 
 #if	defined(SOL_SOCKET) && defined(SO_KEEPALIVE)
-#if defined (DO_SET_SOCKOPT)
-    /* Set KEEPALIVE to catch broken socket connections. */
-    i = 1;
-    if (setsockopt(fd, SOL_SOCKET,  SO_KEEPALIVE, (char *)&i, sizeof i) < 0)
-        syslog(L_ERROR, "%s cant setsockopt(KEEPALIVE) %m", CHANname(cp));
-#endif
+    if (!IsLocal) {
+	/* Set KEEPALIVE to catch broken socket connections. */
+	i = 1;
+	if (setsockopt(fd, SOL_SOCKET,  SO_KEEPALIVE, (char *)&i, sizeof i) < 0)
+	    syslog(L_ERROR, "%s cant setsockopt(KEEPALIVE) %m", CHANname(cp));
+    }
 #endif /* defined(SOL_SOCKET) && defined(SO_KEEPALIVE) */
 
     /* Now check our operating mode. */
@@ -1450,18 +1205,19 @@ NCcreate(fd, MustAuthorize, IsLocal)
     }
 
     /* See if we have too many channels. */
-    if (!IsLocal && MaxIncoming && NCcount >= MaxIncoming && !RCnolimit(cp)) {
+    if (!IsLocal && innconf->maxconnections && 
+			NCcount >= innconf->maxconnections && !RCnolimit(cp)) {
 	/* Recount, just in case we got out of sync. */
 	for (NCcount = 0, i = 0; CHANiter(&i, CTnntp) != NULL; )
 	    NCcount++;
-	if (NCcount >= MaxIncoming) {
+	if (NCcount >= innconf->maxconnections) {
 	    NCwriteshutdown(cp, "Too many connections");
 	    return cp;
 	}
     }
     cp->BadReads = 0;
     cp->BadCommands = 0;
-    NCwritereply(cp, NCgreeting);
+    NCwritereply(cp, (char *)NCgreeting);
     return cp;
 }
 
@@ -1476,18 +1232,12 @@ NCcreate(fd, MustAuthorize, IsLocal)
 **  article or not.  Stay in command state.
 */
 STATIC FUNCTYPE
-NCcheck(cp)
-    CHANNEL		*cp;
+NCcheck(CHANNEL *cp)
 {
-    register char	*p;
+    char		*p;
     int			msglen;
-    WIP			*who;
-
-    if (AmSlave) {
-	NCwritereply(cp, NCbadcommand);
-	return;
-    }
 
+    cp->Check++;
     /* Snip off the Message-ID. */
     for (p = cp->In.Data; !ISWHITE(*p); p++)
 	continue;
@@ -1507,14 +1257,17 @@ NCcheck(cp)
 	return;
     }
 
-    if (HIShavearticle(p)) {
+    if (HIShavearticle(HashMessageID(p))) {
 	cp->Refused++;
+	cp->Check_got++;
 	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_GOTID_VAL, p);
 	NCwritereply(cp, cp->Sendid.Data);
-    } else if (NCinprogress(cp, p, &who)) {
+    } else if (WIPinprogress(p, cp, TRUE)) {
+	cp->Check_deferred++;
 	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_RESENDID_VAL, p);
 	NCwritereply(cp, cp->Sendid.Data);
     } else {
+	cp->Check_send++;
 	(void)sprintf(cp->Sendid.Data, "%d %s", NNTP_OK_SENDID_VAL, p);
 	NCwritereply(cp, cp->Sendid.Data);
     }
@@ -1525,15 +1278,13 @@ NCcheck(cp)
 **  The "takethis" command.  Article follows.
 **  Remember <id> for later ack.
 */
-STATIC FUNCTYPE
-NCtakethis(cp)
-    CHANNEL		*cp;
+STATIC FUNCTYPE NCtakethis(CHANNEL *cp)
 {
-    register char	*p;
+    char	        *p;
     int			msglen;
-    register WIP	*wp;
-    register int	i;
+    WIP                 *wp;
 
+    cp->Takethis++;
     /* Snip off the Message-ID. */
     for (p = cp->In.Data + STRLEN("takethis"); ISWHITE(*p); p++)
 	continue;
@@ -1553,19 +1304,9 @@ NCtakethis(cp)
     (void)sprintf(cp->Sendid.Data, "%d %s", NNTP_ERR_FAILID_VAL, p);
 
     cp->State = CSgetarticle;
-    /* set wp->MessageID for benefit of later code in NCreader
-     * (especially while in CSeatarticle state)
-     */
-    wp = &NCwip[cp->fd];
-    if (wp->MessageID == NULL) {
-	for (i = SIZEOF(NCfreelist); --i >= 0; )
-	    if (NCfreelist[i] != NULL) {
-		wp->MessageID = NCfreelist[i];
-		NCfreelist[i] = NULL;
-		break;
-	    }
-	if (i < 0)
-	    wp->MessageID = NEW(char, DBZMAXKEY + 3);
-    }
-    (void)strcpy(wp->MessageID, p);
+    /* set WIP for benefit of later code in NCreader */
+    if ((wp = WIPbyid(p)) == (WIP *)NULL)
+	wp = WIPnew(p, cp);
+    WIPfree(WIPbyhash(cp->CurrentMessageIDHash));
+    cp->CurrentMessageIDHash = wp->MessageID;
 }
