@@ -1,16 +1,17 @@
-/*  $Revision: 1.38 $
+/*  $Id: newsfeeds.c,v 1.18 1998/04/21 04:29:57 scrappy Exp $
 **
 **  Routines for the in-core data structures for the newsfeeds file.
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #include "innd.h"
 
 
 STATIC SITE	SITEnull;
-STATIC char	SITEfeedspath[] = _PATH_NEWSFEEDS;
+STATIC char	*SITEfeedspath = NULL;
 
 
 /*
@@ -51,6 +52,8 @@ SITEreadfile(ReadOnly)
     struct stat		Sb;
     char		*data;
 
+    if (SITEfeedspath == NULL)
+	SITEfeedspath = COPY(cpcatpath(innconf->pathetc, _PATH_NEWSFEEDS));
     if (old_strings != NULL) {
 	/* If the file hasn't changed, return a copy of the old data. */
 	if (stat(SITEfeedspath, &Sb) >= 0
@@ -94,6 +97,8 @@ SITEreadfile(ReadOnly)
 	}
 	*to++ = '\0';
 	if (*site == COMMENT_CHAR || *site == '\0')
+            continue ;
+        if (strspn(site," \t") == strlen (site))
 	    continue;
 	old_strings[i++] = COPY(site);
     }
@@ -161,19 +166,13 @@ SITEsetlist(patlist, subbed, poison, poisonEntry)
 **  If subbed is NULL, we don't update the SITE array, since we're just
 **  doing syntax checking.
 */
-STRING
-SITEparseone(Entry, sp, subbed, poison)
-    char		*Entry;
-    SITE		*sp;
-    char		*subbed;
-    char		*poison;
+STRING SITEparseone(char *Entry, SITE *sp, char *subbed, char *poison)
 {
-    static char		BATCH[] = _PATH_BATCHDIR;
-    register int	i;
-    register int	j;
-    register NEWSGROUP	*ngp;
-    register char	*p;
-    register char	*u;
+    int	                i;
+    int	                j;
+    NEWSGROUP	        *ngp;
+    char	        *p;
+    char	        *u;
     char		*f2;
     char		*f3;
     char		*f4;
@@ -194,8 +193,13 @@ SITEparseone(Entry, sp, subbed, poison)
     sp->Process = -1;
     sp->Next = sp->Prev = NOSITE;
     sp->Entry = Entry;
+    sp->Originator = NULL;
     sp->FileFlags[0] = FEED_NAME;
     sp->FileFlags[1] = '\0';
+    sp->Nice = innconf->nicekids;
+    sp->ControlOnly = FALSE;
+    sp->DontWantNonExist = FALSE;
+    sp->NeedOverviewCreation = FALSE;
 
     /* Nip off the first field, the site name. */
     if ((f2 = strchr(Entry, NF_FIELD_SEP)) == NULL)
@@ -258,7 +262,15 @@ SITEparseone(Entry, sp, subbed, poison)
 		switch (*p) {
 		default:
 		    return "unknown A param in field 3";
+		case 'c': sp->IgnoreControl = TRUE;
+			  sp->ControlOnly = FALSE;
+			  break;
+		case 'C': sp->ControlOnly = TRUE;
+			  sp->IgnoreControl = FALSE;
+			  break;
 		case 'd': sp->DistRequired = TRUE;	break;
+		case 'e': sp->DontWantNonExist = TRUE;	break;
+		case 'o': sp->NeedOverviewCreation = TRUE;	break;
 		case 'p': sp->IgnorePath = TRUE;	break;
 		}
 	    break;
@@ -277,8 +289,9 @@ SITEparseone(Entry, sp, subbed, poison)
 	    else if (*p == '/')
 		sp->SpoolName = COPY(p);
 	    else {
-		sp->SpoolName = NEW(char, STRLEN(BATCH) + 1 + strlen(p) + 1);
-		FileGlue(sp->SpoolName, BATCH, '/', p);
+		sp->SpoolName = NEW(char, strlen(innconf->pathoutgoing) + 1 +
+						strlen(p) + 1);
+		FileGlue(sp->SpoolName, innconf->pathoutgoing, '/', p);
 	    }
 	    break;
 	case 'G':
@@ -306,6 +319,13 @@ SITEparseone(Entry, sp, subbed, poison)
 		case 'u': JustUnmoderated = TRUE;	break;
 		}
 	    break;
+	case 'O':
+	    sp->Originator = COPY(++p);
+	    break;
+        case 'P':
+            if (*++p && CTYPE(isdigit, *p))
+                sp->Nice = atoi(p);
+            break;
 	case 'S':
 	    if (*++p && CTYPE(isdigit, *p))
 		sp->StartSpooling = atol(p);
@@ -336,8 +356,12 @@ SITEparseone(Entry, sp, subbed, poison)
 		case FEED_OVERVIEW:
 		    NeedOverview = TRUE;	/* Overview data	*/
 		    break;
+		case FEED_PATH:			/* Path			*/
+		    NeedPath = TRUE;
+		    break;
 		case FEED_BYTESIZE:		/* Size in bytes	*/
 		case FEED_FULLNAME:		/* Full filename	*/
+		case FEED_HASH:			/* Hash			*/
 		case FEED_HDR_DISTRIB:		/* Distribution header	*/
 		case FEED_HDR_NEWSGROUP:	/* Newsgroup header	*/
 		case FEED_MESSAGEID:		/* Message-ID		*/
@@ -378,19 +402,22 @@ SITEparseone(Entry, sp, subbed, poison)
     if (*f4 == '\0' && sp != &ME) {
 	if (sp->Type != FTfile && sp->Type != FTlogonly)
 	    return "empty field 4";
-	sp->Param = NEW(char, STRLEN(BATCH) + 1 + sp->NameLength + 1);
-	FileGlue(sp->Param, BATCH, '/', sp->Name);
+	sp->Param = NEW(char, strlen(innconf->pathoutgoing) + 1 +
+						sp->NameLength + 1);
+	FileGlue(sp->Param, innconf->pathoutgoing, '/', sp->Name);
     }
     else if (sp->Type == FTfile && *f4 != '/') {
-	sp->Param = NEW(char, STRLEN(BATCH) + 1 + strlen(f4) + 1);
-	FileGlue(sp->Param, BATCH, '/', f4);
+	sp->Param = NEW(char, strlen(innconf->pathoutgoing) + 1 +
+						strlen(f4) + 1);
+	FileGlue(sp->Param, innconf->pathoutgoing, '/', f4);
     }
     else
 	sp->Param = COPY(f4);
 
     if (sp->SpoolName == NULL) {
-	sp->SpoolName = NEW(char, STRLEN(BATCH) + 1 + strlen(sp->Name) + 1);
-	FileGlue(sp->SpoolName, BATCH, '/', sp->Name);
+	sp->SpoolName = NEW(char, strlen(innconf->pathoutgoing) + 1 +
+						strlen(sp->Name) + 1);
+	FileGlue(sp->SpoolName, innconf->pathoutgoing, '/', sp->Name);
     }
 
     /* Make sure there is only one %s, and only one *. */
@@ -547,20 +574,21 @@ SITEparsefile(StartSite)
     for (strings = SITEreadfile(FALSE), nSites = 0; strings[nSites]; nSites++)
 	continue;
     Sites = NEW(SITE, nSites);
-    for (sp = Sites, i = 0; i < nSites; i++, sp++) {
-	sp->Name = NULL;
-	sp->Buffer.Data = NULL;
-        sp->Buffer.Size = sp->Buffer.Used = sp->Buffer.Left = 0 ;
-    }
+    (void)memset((POINTER)Sites, '\0', (SIZE_T)(nSites * sizeof(SITE)));
 
     /* Set up scratch subscription list. */
     subbed = NEW(char, nGroups);
     poison = NEW(char, nGroups);
 
+    ME.Prev = 0; /* Used as a flag to ensure exactly one ME entry */
     for (sp = Sites, errors = 0, i = 0; i < nSites; i++) {
 	p = strings[i];
 	if (p[0] == 'M' && p[1] == 'E' &&
             ((p[2] == NF_FIELD_SEP) || (p[2] == NF_SUBFIELD_SEP))) {
+	    if (ME.Prev == NOSITE) {
+		syslog(L_FATAL, "bad_newsfeeds. Must have exactly one ME entry");
+		errors++;
+	    } else
 	    if ((error = SITEparseone(p, &ME, subbed, poison)) != NULL) {
 		syslog(L_FATAL, "%s bad_newsfeeds %s", MaxLength(p, p), error);
 		errors++;
@@ -580,6 +608,10 @@ SITEparsefile(StartSite)
 	sp->Working = TRUE;
 	sp++;
     }
+    if (ME.Prev != NOSITE) {
+	syslog(L_FATAL, "bad_newsfeeds. Must have exactly one ME entry");
+	errors++;
+    }
 
     if (errors) {
 	syslog(L_FATAL, "%s syntax_error %s", LogName, SITEfeedspath);
