@@ -1,9 +1,15 @@
-/*  $Revision: 1.16 $
+/*  $Id: ng.c,v 1.12 1998/05/18 23:39:15 kondou Exp $
 **
 **  Routine for the in-core data structures for the active and newsfeeds
 **  files.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
+#include <sys/mman.h>
 #include "mydir.h"
 
 
@@ -18,7 +24,7 @@
 /*
 **  Size of hash table.   Change NGH_BUCKET if not a power of two.
 */
-#define NGH_SIZE	512
+#define NGH_SIZE	2048
 #define NGH_BUCKET(j)	&NGHtable[j & (NGH_SIZE - 1)]
 
 
@@ -48,8 +54,8 @@ STATIC int	NGHcount;
 */
 STATIC int
 NGcompare(p1, p2)
-    POINTER	p1;
-    POINTER	p2;
+    CPOINTER p1;
+    CPOINTER p2;
 {
     NEWSGROUP	**ng1;
     NEWSGROUP	**ng2;
@@ -119,6 +125,8 @@ NGparseentry(ngp, p, end)
     ngp->Last = atol(ngp->LastString);
     ngp->nSites = 0;
     ngp->Sites = NEW(int, NGHcount);
+    ngp->nPoison = 0;
+    ngp->Poison = NEW(int, NGHcount);
     ngp->Alias = NULL;
 
     /* Find the right bucket for the group, make sure there is room. */
@@ -154,11 +162,14 @@ NGparsefile()
     register NGHASH	*htp;
     char		**strings;
     char		*active;
+    char		*end;
 
     /* If re-reading, remove anything we might have had. */
     if (Groups) {
-	for (i = nGroups, ngp = Groups; --i >= 0; ngp++)
+	for (i = nGroups, ngp = Groups; --i >= 0; ngp++) {
 	    DISPOSE(ngp->Sites);
+	    DISPOSE(ngp->Poison);
+	}
 	DISPOSE(Groups);
 	DISPOSE(GroupPointers);
 	DISPOSE(NGdirs.Data);
@@ -167,17 +178,20 @@ NGparsefile()
 
 
     /* Get active file and space for group entries. */
-    active = ICDreadactive();
-    for (p = active, i = 0; (p = strchr(p, '\n')) != NULL; p++, i++)
+    active = ICDreadactive(&end);
+    for (p = active, i = 0; p < end && (p = strchr(p, '\n')) != NULL; p++, i++)
 	continue;
-    nGroups = i;
+    if ((nGroups = i) == 0) {
+	syslog(L_FATAL, "%s empty active file", LogName);
+	exit(1);
+    }
     Groups = NEW(NEWSGROUP, nGroups);
     GroupPointers = NEW(NEWSGROUP*, nGroups);
 
     /* Get space to hold copies of the names and the directory names.
      * This might take more space than individually allocating each
      * element, but it is definitely easier on the system. */
-    i = strlen(active);
+    i = end - active;
     NGdirs.Size = i;
     NGdirs.Data = NEW(char, NGdirs.Size + 1);
     NGdirs.Used = 0;
@@ -216,7 +230,7 @@ NGparsefile()
     for (p = active, ngp = Groups, i = nGroups; --i >= 0; ngp++, p = q + 1) {
 	ngp->Start = p - active;
 	if ((q = strchr(p, '\n')) == NULL || !NGparseentry(ngp, p, q)) {
-	    syslog(L_FATAL, "%s bad_active %.20s...", LogName, p);
+	    syslog(L_FATAL, "%s bad_active %s...", LogName, MaxLength(p, q));
 	    exit(1);
 	}
     }
@@ -313,58 +327,113 @@ NGsplit(p)
 /*
 **  Renumber a group.
 */
-BOOL
-NGrenumber(ngp)
-    NEWSGROUP		*ngp;
+static char		NORENUMBER[] = "%s cant renumber %s %s too wide";
+static char		RENUMBER[] = "%s renumber %s %s from %ld to %ld";
+
+BOOL NGrenumber(NEWSGROUP *ngp)
 {
-    static char		NORENUMBER[] = "%s cant renumber %s %s too wide";
-    static char		RENUMBER[] = "%s renumber %s %s from %ld to %ld";
-    register DIR	*dp;
-    register DIRENTRY	*ep;
-    register char	*f2;
+    DIR	                *dp;
+    DIRENTRY	        *ep;
+    char	        *f2;
+    char	        *p;
     char		*f3;
     char		*f4;
     char		*start;
     long		l;
     long		himark;
     long		lomark;
+    char		*dummy;
+    FILE                *fi;
+    OVERINDEX           index;
+    char                packed[OVERINDEXPACKSIZE];
+    struct stat		sb;
+    char		(*mapped)[][OVERINDEXPACKSIZE];
+    int			count;
 
     /* Get a valid offset into the active file. */
     if (ICDneedsetup) {
 	syslog(L_ERROR, "%s unsynched must reload before renumber", LogName);
 	return FALSE;
     }
-    start = ICDreadactive() + ngp->Start;
+    start = ICDreadactive(&dummy) + ngp->Start;
 
     /* Check the file format. */
     if ((f2 = strchr(start, ' ')) == NULL
      || (f3 = strchr(++f2, ' ')) == NULL
      || (f4 = strchr(++f3, ' ')) == NULL) {
-	syslog(L_ERROR, "%s bad_format active %.50s", LogName, start);
+	syslog(L_ERROR, "%s bad_format active %s",
+	    LogName, MaxLength(start, start));
 	return FALSE;
     }
     himark = atol(f2);
     lomark = himark + 1;
 
-    /* Scan the directory. */
-    if ((dp = opendir(ngp->Dir)) != NULL) {
-	while ((ep = readdir(dp)) != NULL) {
-	    if (!CTYPE(isdigit, ep->d_name[0])
-	     || strspn(ep->d_name, "0123456789") != strlen(ep->d_name)
-	     || (l = atol(ep->d_name)) == 0)
-		continue;
-	    if (l < lomark)
-		lomark = l;
-	    if (l > himark)
-		himark = l;
+    if (innconf->storageapi) {
+	p = NEW(char, strlen(innconf->pathoverview) + strlen(ngp->Dir) +
+					strlen(innconf->overviewname) + 32);
+	sprintf(p, "%s/%s/%s.index", innconf->pathoverview, ngp->Dir,
+					innconf->overviewname);
+	if ((fi = Fopen(p, "r", TEMPORARYOPEN)) == NULL) {
+	    DISPOSE(p);
+	    return TRUE;
+	}
+	DISPOSE(p);
+	if (innconf->overviewmmap) {
+	    if (fstat(fileno(fi), &sb) < 0) {
+		Fclose(fi);
+		return TRUE;
+	    }
+	    count = sb.st_size / OVERINDEXPACKSIZE;
+	    if (count == 0) {
+		Fclose(fi);
+		return TRUE;
+	    }
+	    if ((mapped = (char (*)[][OVERINDEXPACKSIZE])mmap((MMAP_PTR)0, count * OVERINDEXPACKSIZE,
+		PROT_READ, MAP__ARG, fileno(fi), 0)) == (char (*)[][OVERINDEXPACKSIZE])-1) {
+		Fclose(fi);
+		return TRUE;
+	    }
+	    Fclose(fi);
+	    /* assumes .overview.index is sorted */
+	    UnpackOverIndex((*mapped)[0], &index);
+	    if (index.artnum < lomark)
+		lomark = index.artnum;
+	    UnpackOverIndex((*mapped)[count-1], &index);
+	    if (index.artnum > himark)
+		himark = index.artnum;
+	    if ((munmap((MMAP_PTR)mapped, count * OVERINDEXPACKSIZE)) < 0)
+		return TRUE;
+	} else {
+	    while (fread(&packed, OVERINDEXPACKSIZE, 1, fi) == 1) {
+		UnpackOverIndex(packed, &index);
+		if (index.artnum < lomark)
+		    lomark = index.artnum;
+		if (index.artnum > himark)
+		    himark = index.artnum;
+	    }
+	    Fclose(fi);
+	}
+    } else {
+        /* Scan the directory. */
+	if ((dp = opendir(ngp->Dir)) != NULL) {
+	    while ((ep = readdir(dp)) != NULL) {
+		p = ep->d_name;
+		if (!CTYPE(isdigit, p[0]) || strspn(p, "0123456789") != strlen(p)
+		    || (l = atol(p)) == 0)
+		    continue;
+		if (l < lomark)
+		    lomark = l;
+		if (l > himark)
+		    himark = l;
+	    }
+	    (void)closedir(dp);
 	}
-	(void)closedir(dp);
     }
     l = atol(f2);
     if (himark != l) {
 	syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, himark);
 	if (!FormatLong(f2, himark, f3 - f2 - 1)) {
-	    syslog(L_NOTICE, NORENUMBER, LogName, ngp->Name, "hi");
+	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
 	    return FALSE;
 	}
 	ngp->Last = himark;
@@ -375,10 +444,43 @@ NGrenumber(ngp)
 	if (lomark < l)
 	    syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
 	if (!FormatLong(f3, lomark, f4 - f3)) {
-	    syslog(L_NOTICE, NORENUMBER, LogName, ngp->Name, "lo");
+	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
 	    return FALSE;
 	}
 	ICDactivedirty++;
     }
     return TRUE;
 }
+
+/*
+ * Set the low article count for the given group.
+ * Like NGrenumber(), but we don't scan the spool,
+ * and the himark is ignored.
+ */
+BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
+{
+    long l;
+    char *f2, *f3, *f4;
+    char *start;
+
+    start = ICDreadactive(&f2) + ngp->Start;
+    /* Check the file format. */
+    if ((f2 = strchr(start, ' ')) == NULL
+     || (f3 = strchr(++f2, ' ')) == NULL
+     || (f4 = strchr(++f3, ' ')) == NULL) {
+        syslog(L_ERROR, "%s bad_format active %s",
+            LogName, MaxLength(start, start));
+        return FALSE;
+    }
+    l = atol(f3);
+    if (lomark != l) {
+        if (lomark < l)
+            syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
+        if (!FormatLong(f3, lomark, f4 - f3)) {
+            syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
+            return FALSE;
+        }
+        ICDactivedirty++;
+    }
+    return TRUE;
+}
