@@ -1,10 +1,11 @@
-/*  $Revision: 1.40 $
+/*  $Id: site.c,v 1.20 1998/06/01 00:30:00 kondou Exp $
 **
 **  Routines to implement site-feeding.  Mainly working with channels to
 **  do buffering and determine what to send.
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #include "innd.h"
@@ -20,9 +21,7 @@ STATIC char	SITEshell[] = _PATH_SH;
 **  Called when input is ready to read.  Shouldn't happen.
 */
 /* ARGSUSED0 */
-STATIC FUNCTYPE
-SITEreader(cp)
-    CHANNEL	*cp;
+STATIC FUNCTYPE SITEreader(CHANNEL *cp)
 {
     syslog(L_ERROR, "%s internal SITEreader", LogName);
 }
@@ -32,9 +31,7 @@ SITEreader(cp)
 **  Called when write is done.  No-op.
 */
 /* ARGSUSED0 */
-STATIC FUNCTYPE
-SITEwritedone(cp)
-    CHANNEL	*cp;
+STATIC FUNCTYPE SITEwritedone(CHANNEL *cp)
 {
 }
 
@@ -42,10 +39,7 @@ SITEwritedone(cp)
 /*
 **  Make a site start spooling.
 */
-BOOL
-SITEspool(sp, cp)
-    register SITE	*sp;
-    CHANNEL		*cp;
+BOOL SITEspool(SITE *sp, CHANNEL *cp)
 {
     int			i;
     char		buff[SPOOLNAMEBUFF];
@@ -94,9 +88,7 @@ SITEspool(sp, cp)
 **  Delete a site from the file writing list.  Can be called even if
 **  site is not on the list.
 */
-static void
-SITEunlink(sp)
-    SITE		*sp;
+static void SITEunlink(SITE *sp)
 {
     if (sp->Next != NOSITE || sp->Prev != NOSITE
      || (SITEhead != NOSITE && sp == &Sites[SITEhead]))
@@ -119,12 +111,11 @@ SITEunlink(sp)
 /*
 **  Find the oldest "file feed" site and buffer it.
 */
-STATIC void
-SITEbufferoldest()
+STATIC void SITEbufferoldest(void)
 {
-    register SITE	*sp;
-    register BUFFER	*bp;
-    register BUFFER	*out;
+    SITE	        *sp;
+    BUFFER	        *bp;
+    BUFFER	        *out;
 
     /* Get the oldest user of a file. */
     if (SITEtail == NOSITE) {
@@ -180,11 +171,10 @@ SITEbufferoldest()
 /*
  * *  Bilge Site's Channel out buffer.
  */
-STATIC          BOOL
-SITECHANbilge(sp)
-    register SITE  *sp;
+STATIC BOOL SITECHANbilge(SITE *sp)
 {
-    int             fd, i;
+    int             fd;
+    int             i;
     char            buff[SPOOLNAMEBUFF];
     char           *name;
 
@@ -227,13 +217,10 @@ SITECHANbilge(sp)
 **  Check if we need to write out the site's buffer.  If we're buffered
 **  or the feed is backed up, this gets a bit complicated.
 */
-STATIC void
-SITEflushcheck(sp, bp)
-    register SITE	*sp;
-    register BUFFER	*bp;
+STATIC void SITEflushcheck(SITE *sp, BUFFER *bp)
 {
-    register int	i;
-    register CHANNEL	*cp;
+    int	                i;
+    CHANNEL	        *cp;
 
     /* If we're buffered, and we hit the flushpoint, do an LRU. */
     if (sp->Buffered) {
@@ -260,8 +247,32 @@ SITEflushcheck(sp, bp)
     if (i < sp->StopWriting)
 	WCHANremove(cp);
     if ((sp->StartWriting == 0 || i > sp->StartWriting)
-     && !CHANsleeping(cp))
-	WCHANadd(cp);
+     && !CHANsleeping(cp)) {
+	if (sp->Type == FTchannel) {	/* channel feed, try the write */
+	    int j;
+	    if (bp->Left == 0)
+		return;
+	    j = write(cp->fd, (POINTER)&bp->Data[bp->Used], (SIZE_T)bp->Left);
+	    if (j > 0) {
+		bp->Left -= j;
+		bp->Used += j;
+		i = cp->Out.Left;
+		/* Since we just had a successful write, we need to clear the 
+		 * channel's error counts. - dave@jetcafe.org */
+		cp->BadWrites = 0;
+		cp->BlockedWrites = 0;
+	    }
+	    if (bp->Left <= 0) {
+                /* reset Used, Left on bp, keep channel buffer size from
+                   exploding. */
+                bp->Used = bp->Left = 0;
+		WCHANremove(cp);
+            } else
+		WCHANadd(cp);
+	}
+	else
+	    WCHANadd(cp);
+    }
 
     cp->LastActive = Now.time;
 
@@ -280,13 +291,10 @@ SITEflushcheck(sp, bp)
 /*
 **  Send a control line to an exploder.
 */
-void
-SITEwrite(sp, text)
-    register SITE	*sp;
-    STRING		text;
+void SITEwrite(SITE *sp, STRING text)
 {
     static char		PREFIX[] = { EXP_CONTROL, '\0' };
-    register BUFFER	*bp;
+    BUFFER	        *bp;
 
     if (sp->Buffered)
 	bp = &sp->Buffer;
@@ -307,19 +315,16 @@ SITEwrite(sp, text)
 /*
 **  Send the desired data about an article down a channel.
 */
-STATIC void
-SITEwritefromflags(sp, Data)
-    register SITE	*sp;
-    ARTDATA		*Data;
+STATIC void SITEwritefromflags(SITE *sp, ARTDATA *Data)
 {
     static char		ITEMSEP[] = " ";
     static char		NL[] = "\n";
     char		pbuff[12];
-    register char	*p;
-    register BOOL	Dirty;
-    register BUFFER	*bp;
-    register SITE	*spx;
-    register int	i;
+    char	        *p;
+    BOOL	        Dirty;
+    BUFFER	        *bp;
+    SITE	        *spx;
+    int	                i;
 
     if (sp->Buffered)
 	bp = &sp->Buffer;
@@ -344,10 +349,19 @@ SITEwritefromflags(sp, Data)
 	case FEED_FULLNAME:
 	    if (Dirty)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
-	    BUFFappend(bp, SPOOL, SPOOLlen);
-	    BUFFappend(bp, "/", 1);
+	    if (!innconf->storageapi) {
+		BUFFappend(bp, innconf->patharticles, SPOOLlen);
+		BUFFappend(bp, "/", 1);
+	    }
 	    BUFFappend(bp, Data->Name, Data->NameLength);
 	    break;
+	case FEED_HASH:
+	    if (Dirty)
+		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
+	    BUFFappend(bp, "[", 1);
+	    BUFFappend(bp, HashToText(*(Data->Hash)), sizeof(HASH)*2);
+	    BUFFappend(bp, "]", 1);
+	    break;
 	case FEED_HDR_DISTRIB:
 	    if (Dirty)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
@@ -368,6 +382,15 @@ SITEwritefromflags(sp, Data)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
 	    BUFFappend(bp, Data->Overview->Data, Data->Overview->Left);
 	    break;
+	case FEED_PATH:
+	    if (Dirty)
+		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
+	    if (!Hassamepath)
+		BUFFappend(bp, Path.Data, Path.Used);
+	    if (AddAlias)
+		BUFFappend(bp, Pathalias.Data, Pathalias.Used);
+	    BUFFappend(bp, Data->Path, Data->PathLength);
+	    break;
 	case FEED_REPLIC:
 	    if (Dirty)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
@@ -438,13 +461,10 @@ SITEwritefromflags(sp, Data)
 /*
 **  Send one article to a site.
 */
-void
-SITEsend(sp, Data)
-    register SITE	*sp;
-    ARTDATA		*Data;
+void SITEsend(SITE *sp, ARTDATA *Data)
 {
-    register int	i;
-    register char	*p;
+    int	                i;
+    char	        *p;
     char		*temp;
     char		buff[BUFSIZ];
     STRING		argv[MAX_BUILTIN_ARGV];
@@ -503,7 +523,7 @@ SITEsend(sp, Data)
 	}
 
 	/* Start the process. */
-	i = Spawn(fd, (int)fileno(Errlog), (int)fileno(Errlog), argv);
+	i = Spawn(sp->Nice, fd, (int)fileno(Errlog), (int)fileno(Errlog), argv);
 	if (i >= 0)
 	    (void)PROCwatch(i, -1);
 	if (fd != 0)
@@ -517,9 +537,7 @@ SITEsend(sp, Data)
 **  The channel was sleeping because we had to spool our output to
 **  a file.  Flush and restart.
 */
-STATIC FUNCTYPE
-SITEspoolwake(cp)
-    CHANNEL	*cp;
+STATIC FUNCTYPE SITEspoolwake(CHANNEL *cp)
 {
     SITE	*sp;
     int		*ip;
@@ -542,11 +560,9 @@ SITEspoolwake(cp)
 **  Start up a process for a channel, or a spool to a file if we can't.
 **  Create a channel for the site to talk to.
 */
-STATIC BOOL
-SITEstartprocess(sp)
-    SITE		*sp;
+STATIC BOOL SITEstartprocess(SITE *sp)
 {
-    register PID_T	i;
+    PID_T	        i;
     STRING		argv[MAX_BUILTIN_ARGV];
     char		*process;
     int			*ip;
@@ -569,7 +585,7 @@ SITEstartprocess(sp)
     }
 
     /* Fork a child. */
-    i = Spawn(pan[PIPE_READ], (int)fileno(Errlog), (int)fileno(Errlog), argv);
+    i = Spawn(sp->Nice, pan[PIPE_READ], (int)fileno(Errlog), (int)fileno(Errlog), argv);
     if (i > 0) {
 	sp->pid = i;
 	sp->Spooling = FALSE;
@@ -594,7 +610,7 @@ SITEstartprocess(sp)
     syslog(L_ERROR, "%s cant spawn spooling %m", sp->Name);
     ip = NEW(int, 1);
     *ip = sp - Sites;
-    SCHANadd(sp->Channel, (time_t)(Now.time + CHANNEL_RETRY_TIME),
+    SCHANadd(sp->Channel, (time_t)(Now.time + innconf->chanretrytime),
 	(POINTER)NULL, SITEspoolwake, (POINTER)ip);
     return TRUE;
 }
@@ -603,11 +619,9 @@ SITEstartprocess(sp)
 /*
 **  Set up a site for internal buffering.
 */
-STATIC void
-SITEbuffer(sp)
-    register SITE	*sp;
+STATIC void SITEbuffer(SITE *sp)
 {
-    register BUFFER	*bp;
+    BUFFER	        *bp;
 
     SITEunlink(sp);
     sp->Buffered = TRUE;
@@ -630,13 +644,9 @@ SITEbuffer(sp)
 /*
 **  Link a site at the head of the "currently writing to a file" list
 */
-static void
-SITEmovetohead(sp)
-    SITE		*sp;
+static void SITEmovetohead(SITE *sp)
 {
-    if (SITEhead == NOSITE
-     && (sp->Next != NOSITE || sp->Prev != NOSITE
-      || sp == &Sites[SITEhead] || sp == &Sites[SITEtail]))
+    if ((SITEhead == NOSITE) && ((sp->Next != NOSITE) || (sp->Prev != NOSITE)))
 	SITEunlink(sp);
 
     if ((sp->Next = SITEhead) != NOSITE)
@@ -654,9 +664,7 @@ SITEmovetohead(sp)
 /*
 **  Set up a site's feed.  This means opening a file or channel if needed.
 */
-BOOL
-SITEsetup(sp)
-    register SITE	*sp;
+BOOL SITEsetup(SITE *sp)
 {
     int			fd;
     int			oerrno;
@@ -713,11 +721,7 @@ SITEsetup(sp)
 /*
 **  A site's channel process died; restart it.
 */
-void
-SITEprocdied(sp, process, pp)
-    SITE	*sp;
-    int		process;
-    PROCESS	*pp;
+void SITEprocdied(SITE *sp, int process, PROCESS *pp)
 {
     syslog(pp->Status ? L_ERROR : L_NOTICE, "%s exit %d elapsed %ld pid %ld",
 	sp->Name ? sp->Name : "?", pp->Status,
@@ -739,12 +743,10 @@ SITEprocdied(sp, process, pp)
 /*
 **  A channel is about to be closed; see if any site cares.
 */
-void
-SITEchanclose(cp)
-    register CHANNEL	*cp;
+void SITEchanclose(CHANNEL *cp)
 {
-    register int	i;
-    register SITE	*sp;
+    int	                i;
+    SITE	        *sp;
     int			*ip;
 
     for (i = nSites, sp = Sites; --i >= 0; sp++)
@@ -760,7 +762,7 @@ SITEchanclose(cp)
 	    WCHANadd(sp->Channel);
 	    ip = NEW(int, 1);
 	    *ip = sp - Sites;
-	    SCHANadd(sp->Channel, (time_t)(Now.time + CHANNEL_RETRY_TIME),
+	    SCHANadd(sp->Channel, (time_t)(Now.time + innconf->chanretrytime),
 		(POINTER)NULL, SITEspoolwake, (POINTER)ip);
 	    break;
 	}
@@ -770,13 +772,10 @@ SITEchanclose(cp)
 /*
 **  Flush any pending data waiting to be sent.
 */
-void
-SITEflush(sp, Restart)
-    register SITE	*sp;
-    BOOL		Restart;
+void SITEflush(SITE *sp, BOOL Restart)
 {
-    register CHANNEL	*cp;
-    register BUFFER	*out;
+    CHANNEL	        *cp;
+    BUFFER	        *out;
     int			count;
 
     if (sp->Name == NULL)
@@ -869,12 +868,10 @@ SITEflush(sp, Restart)
 /*
 **  Flush all sites.
 */
-void
-SITEflushall(Restart)
-    BOOL	Restart;
+void SITEflushall(BOOL Restart)
 {
-    register int	i;
-    register SITE	*sp;
+    int	                i;
+    SITE	        *sp;
 
     for (i = nSites, sp = Sites; --i >= 0; sp++)
 	if (sp->Name)
@@ -886,15 +883,12 @@ SITEflushall(Restart)
 **  Run down the site's pattern list and see if it wants the specified
 **  newsgroup.
 */
-BOOL
-SITEwantsgroup(sp, name)
-    register SITE	*sp;
-    register char	*name;
+BOOL SITEwantsgroup(SITE *sp, char *name)
 {
-    register BOOL	match;
-    register BOOL	subvalue;
-    register char	*pat;
-    register char	**argv;
+    BOOL	        match;
+    BOOL	        subvalue;
+    char	        *pat;
+    char	        **argv;
 
     match = SUB_DEFAULT;
     if (ME.Patterns) {
@@ -921,15 +915,12 @@ SITEwantsgroup(sp, name)
 **  Run down the site's pattern list and see if specified newsgroup is
 **  considered poison.
 */
-BOOL
-SITEpoisongroup(sp, name)
-    register SITE	*sp;
-    register char	*name;
+BOOL SITEpoisongroup(SITE *sp, char *name)
 {
-    register BOOL	match;
-    register BOOL	poisonvalue;
-    register char	*pat;
-    register char	**argv;
+    BOOL	        match;
+    BOOL	        poisonvalue;
+    char	        *pat;
+    char	        **argv;
 
     match = SUB_DEFAULT;
     if (ME.Patterns) {
@@ -955,12 +946,10 @@ SITEpoisongroup(sp, name)
 /*
 **  Find a site.
 */
-SITE *
-SITEfind(p)
-    char	*p;
+SITE *SITEfind(char *p)
 {
-    register int	i;
-    register SITE	*sp;
+    int	                i;
+    SITE	        *sp;
 
     for (i = nSites, sp = Sites; --i >= 0; sp++)
 	if (sp->Name && caseEQ(p, sp->Name))
@@ -972,12 +961,9 @@ SITEfind(p)
 /*
 **  Find the next site that matches this site.
 */
-SITE *
-SITEfindnext(p, sp)
-    char		*p;
-    register SITE	*sp;
+SITE *SITEfindnext(char *p, SITE *sp)
 {
-    register SITE	*end;
+    SITE	        *end;
 
     for (sp++, end = &Sites[nSites]; sp < end; sp++)
 	if (sp->Name && caseEQ(p, sp->Name))
@@ -989,14 +975,12 @@ SITEfindnext(p, sp)
 /*
 **  Close a site down.
 */
-void
-SITEfree(sp)
-    register SITE	*sp;
+void SITEfree(SITE *sp)
 {
-    register SITE	*s;
-    register int	new;
-    register int	i;
-
+    SITE                *s;
+    int                 new;
+    int                 i;
+    
     if (sp->Channel) {
 	CHANclose(sp->Channel, CHANname(sp->Channel));
 	sp->Channel = NULL;
@@ -1014,6 +998,10 @@ SITEfree(sp)
 	DISPOSE(sp->Entry);
 	sp->Entry = NULL;
     }
+    if (sp->Originator) {
+    	DISPOSE(sp->Originator);
+    	sp->Originator = NULL;
+    }
     if (sp->Param) {
 	DISPOSE(sp->Param);
 	sp->Param = NULL;
@@ -1065,13 +1053,10 @@ SITEfree(sp)
 **  If a site is an exploder or funnels into one, forward a command
 **  to it.
 */
-void
-SITEforward(sp, text)
-    register SITE	*sp;
-    char		*text;
+void SITEforward(SITE *sp, char *text)
 {
-    register SITE	*fsp;
-    register char	*p;
+    SITE	        *fsp;
+    char	        *p;
     char		buff[SMBUF];
 
     fsp = sp;
@@ -1094,9 +1079,7 @@ SITEforward(sp, text)
 /*
 **  Drop a site.
 */
-void
-SITEdrop(sp)
-    SITE		*sp;
+void SITEdrop(SITE *sp)
 {
     SITEforward(sp, "drop");
     SITEflush(sp, FALSE);
@@ -1107,15 +1090,11 @@ SITEdrop(sp)
 /*
 **  Append info about the current state of the site to the buffer
 */
-void
-SITEinfo(bp, sp, Verbose)
-    register BUFFER	*bp;
-    register SITE	*sp;
-    BOOL		Verbose;
+void SITEinfo(BUFFER *bp, SITE *sp, BOOL Verbose)
 {
     static char		FREESITE[] = "<<No name>>\n\n";
-    register char	*p;
-    register CHANNEL	*cp;
+    char	        *p;
+    CHANNEL	        *cp;
     char		*sep;
     char		buff[BUFSIZ];
 
@@ -1217,7 +1196,7 @@ Common:
 		ctime(&cp->LastActive) + 4);
 	    p += strlen(p);
 	    if (cp->Waketime > Now.time) {
-		(void)sprintf("\tSleeping until %.12s\n",
+		(void)sprintf(p, "\tSleeping until %.12s\n",
 		    ctime(&cp->Waketime) + 4);
 		p += strlen(p);
 	    }
