@@ -1,9 +1,10 @@
-/*  $Revision: 1.24 $
+/*  $Revision: 1.4 $
 **
 **  Library routines to let other programs control innd.
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #include <errno.h>
@@ -16,9 +17,9 @@
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
 #include <sys/un.h>
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 #include "nntp.h"
 #include "paths.h"
 #include "libinn.h"
@@ -28,11 +29,11 @@
 
 #define MIN_BUFFER_SIZE		4096
 
-STATIC char			ICCsockname[sizeof _PATH_TEMPSOCK];
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+STATIC char			*ICCsockname = NULL;
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
 STATIC struct sockaddr_un	ICCserv;
 STATIC struct sockaddr_un	ICCclient;
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 STATIC int			ICCfd;
 STATIC int			ICCtimeout;
 char				*ICCfailure;
@@ -59,14 +60,15 @@ ICCopen()
     int		oerrno;
 
     /* Create a temporary name. */
-    (void)strcpy(ICCsockname, _PATH_TEMPSOCK);
+    if (ICCsockname == NULL)
+	ICCsockname = COPY(cpcatpath(innconf->pathrun, _PATH_TEMPSOCK));
     (void)mktemp(ICCsockname);
     if (unlink(ICCsockname) < 0 && errno != ENOENT) {
 	ICCfailure = "unlink";
 	return -1;
     }
 
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     /* Make a socket and give it the name. */
     if ((ICCfd = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0) {
 	ICCfailure = "socket";
@@ -89,7 +91,9 @@ ICCopen()
     /* Name the server's socket. */
     (void)memset((POINTER)&ICCserv, 0, sizeof ICCserv);
     ICCserv.sun_family = AF_UNIX;
-    (void)strcpy(ICCserv.sun_path, _PATH_NEWSCONTROL);
+    (void)strcpy(ICCserv.sun_path, innconf->pathrun);
+    (void)strcat(ICCserv.sun_path, "/");
+    (void)strcat(ICCserv.sun_path, _PATH_NEWSCONTROL);
 #else
     /* Make a named pipe and open it. */
     mask = umask(0);
@@ -105,7 +109,7 @@ ICCopen()
 	ICCfailure = "open";
 	return -1;
     }
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
     ICCfailure = NULL;
     return 0;
@@ -145,7 +149,7 @@ ICCserverpid()
     char		buff[SMBUF];
 
     pid = 1;
-    if ((F = fopen(_PATH_SERVERPID, "r")) != NULL) {
+    if ((F = fopen(cpcatpath(innconf->pathrun, _PATH_SERVERPID), "r")) != NULL) {
 	if (fgets(buff, sizeof buff, F) != NULL)
 	    pid = (PID_T) atol(buff);
 	(void)fclose(F);
@@ -190,9 +194,9 @@ ICCcommand(cmd, argv, replyp)
     char		save;
     int			bufsiz;
     int			i ;
-#if	!defined(DO_HAVE_UNIX_DOMAIN)
+#if	!defined(HAVE_UNIX_DOMAIN_SOCKETS)
     int			fd;
-#endif	/* !defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* !defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     int			len;
     FDSET		Rmask;
     struct timeval	T;
@@ -249,7 +253,7 @@ ICCcommand(cmd, argv, replyp)
 
     buff -= HEADER_SIZE ;
 
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     if (sendto(ICCfd, buff, len, 0,
 	    (struct sockaddr *)&ICCserv, AF_UNIX_SOCKSIZE(ICCserv)) < 0) {
 	DISPOSE(buff);
@@ -257,7 +261,7 @@ ICCcommand(cmd, argv, replyp)
 	return -1;
     }
 #else
-    if ((fd = open(_PATH_NEWSCONTROL, O_WRONLY)) < 0) {
+    if ((fd = open(cpcatpath(inconf->pathrun, _PATH_NEWSCONTROL), O_WRONLY)) < 0) {
 	DISPOSE(buff);
 	ICCfailure = "open";
 	return -1;
@@ -271,7 +275,7 @@ ICCcommand(cmd, argv, replyp)
 	return -1;
     }
     (void)close(fd);
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
     /* Possibly get a reply. */
     switch (cmd) {
@@ -318,7 +322,7 @@ ICCcommand(cmd, argv, replyp)
     }
 
 
-#if defined (DO_HAVE_UNIX_DOMAIN)
+#if defined (HAVE_UNIX_DOMAIN_SOCKETS)
     
     /* Read the reply. */
     i = RECVorREAD(ICCfd, buff, bufsiz) ;
@@ -348,7 +352,7 @@ ICCcommand(cmd, argv, replyp)
 
     buff[i] = '\0';
 
-#else  /* defined (DO_HAVE_UNIX_DOMAIN) */
+#else  /* defined (HAVE_UNIX_DOMAIN_SOCKETS) */
 
     i = RECVorREAD (ICCfd, buff, HEADER_SIZE) ;
     if (i != HEADER_SIZE) {
@@ -370,7 +374,7 @@ ICCcommand(cmd, argv, replyp)
         return -1 ;
     }
 
-#endif /* defined (DO_HAVE_UNIX_DOMAIN) */
+#endif /* defined (HAVE_UNIX_DOMAIN_SOCKETS) */
     
     /* Parse the rest of the reply; expected to be like
        <exitcode><space><text>" */
