@@ -1,22 +1,19 @@
-/*  $Revision: 1.6 $
+/*  $Revision: 1.2 $
 **
 */
 #include <stdio.h>
-#include <errno.h>
 #include <sys/types.h>
 #include "configdata.h"
-#include "libinn.h"
 #include "clibrary.h"
+#include <errno.h>
+#include "libinn.h"
 #include "macros.h"
 
 
 /*
 **  Reallocate some memory or call the memory failure handler.
 */
-ALIGNPTR
-xrealloc(p, i)
-    char		*p;
-    unsigned int	i;
+ALIGNPTR xrealloc(char *p, unsigned int i)
 {
     POINTER		new;
 
