@@ -1,8 +1,33 @@
-/*  $Revision: 1.1 $
+/*  $Revision: 1.2 $
 **
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "nnrpd.h"
 #if	NNRP_LOADLIMIT > 0
+#ifdef linux
+
+/*
+**  Get the current load average as an integer.
+*/
+int
+GetLoadAverage()
+{
+    FILE *ProcLoadAve;
+    float load;
+
+    if ((ProcLoadAve=fopen("/proc/loadavg", "r"))==NULL)
+        return -1;
+    if (fscanf(ProcLoadAve,"%f", &load)!=1)
+    	return -1;
+    fclose(ProcLoadAve);
+    return (int)(load+0.5);
+}
+
+#else 
 #include <nlist.h>
 STATIC struct nlist NameList[] = {
     { "_avenrun" },
@@ -29,13 +54,17 @@ GetLoadAverage()
     if (fd < 0)
 	return -1;
 
-#if	!defined(hpux)
-    (void)nlist("/vmunix", NameList);
-#else
+#if	defined(HPUX)
     (void)nlist("/hp-ux", NameList);
-#endif	/* !defined(hpux) */
+#else
+#if	defined(SUNOS5)
+    (void)nlist("/dev/ksyms", NameList);
+#else
+    (void)nlist("/vmunix", NameList);
+#endif	/* defined(SUNOS5) */
+#endif	/* !defined(HPUX) */
     if (NameList[0].n_type == 0
-     || lseek(fd, (off_t) NameList[X_AVENRUN].n_value, SEEK_SET) == -1
+     || lseek(fd, (OFFSET_T) NameList[X_AVENRUN].n_value, SEEK_SET) == -1
      || read(fd, (char *)avenrun, sizeof avenrun) != sizeof avenrun) {
 	oerrno = errno;
 	(void)close(fd);
@@ -51,4 +80,5 @@ GetLoadAverage()
     return (int)(avenrun[0] + 0.5);
 #endif	/* defined(FSCALE) */
 }
+#endif /* linux */
 #endif	/* NNRP_LOADLIMIT > 0 */
