@@ -1,4 +1,4 @@
-/*  $Revision: 1.32 $
+/*  $Revision: 1.38 $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **  This server doesn't do any real load-limiting, except for what has
@@ -6,6 +6,12 @@
 */
 #include <stdio.h>
 #include <sys/types.h>
+#ifdef HAVE_WAIT_H
+# include <wait.h>
+#else
+# include <sys/wait.h>
+#endif
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #define MAINLINE
@@ -43,12 +49,15 @@ typedef struct _CMDENT {
 
 
 char	NOACCESS[] = NNTP_ACCESS;
-char	ACTIVE[] = _PATH_ACTIVE;
-char	ACTIVETIMES[] = _PATH_ACTIVETIMES;
-char	HISTORY[] = _PATH_HISTORY;
-char	NEWSGROUPS[] = _PATH_NEWSGROUPS;
+char	*ACTIVE = NULL;
+char	*ACTIVETIMES = NULL;
+char	*HISTORY = NULL;
+char	*NEWSGROUPS = NULL;
+char	*NNRPACCESS = NULL;
     /* Default permission -- change with adb. */
 BOOL	PERMdefault = FALSE;
+BOOL	ForceReadOnly = FALSE;
+char 	LocalLogFileName[256];
 
 STATIC double	STATstart;
 STATIC double	STATfinish;
@@ -58,6 +67,8 @@ STATIC char	*TITLEstart;
 STATIC char	*TITLEend;
 #endif	/* !defined(HPUX) */
 STATIC SIGVAR	ChangeTrace;
+BOOL	DaemonMode = FALSE;
+
 
 extern FUNCTYPE	CMDauthinfo();
 extern FUNCTYPE	CMDdate();
@@ -77,6 +88,10 @@ extern FUNCTYPE	CMDxpat();
 extern FUNCTYPE	CMDxpath();
 extern FUNCTYPE	CMD_unimp();
 
+extern int RARTenable;
+extern int LLOGenable;
+extern int TrackClient();
+
 STATIC char	CMDfetchhelp[] = "[MessageID|Number]";
 
 STATIC CMDENT	CMDtable[] = {
@@ -99,7 +114,7 @@ STATIC CMDENT	CMDtable[] = {
     {	"last",		CMDnextlast,	TRUE,	1,	1,
 	NULL },
     {	"list",		CMDlist,	TRUE,	1,	3,
-	"[active|active.times|newsgroups|distributions|distrib.pats|overview.fmt|subscriptions]" },
+	"[active|active.times|newsgroups|distributions|distrib.pats|overview.fmt|subscriptions|motd]" },
     {	"listgroup",	CMDgroup,	TRUE,	1,	2,
 	"newsgroup" },
     {	"mode",		CMDmode,	FALSE,	2,	2,
@@ -150,13 +165,40 @@ ExitWithStats(x)
     }
 
     GRPreport();
-    syslog(L_NOTICE, "%s exit articles %ld groups %ld",
-	   ClientHost, ARTcount, GRPcount);
+    if (ARTcount)
+        syslog(L_NOTICE, "%s exit articles %ld groups %ld", 
+    	    ClientHost, ARTcount, GRPcount);
     if (POSTreceived ||  POSTrejected)
 	syslog(L_NOTICE, "%s posts received %ld rejected %ld",
 	   ClientHost, POSTreceived, POSTrejected);
     syslog(L_NOTICE, "%s times user %.3f system %.3f elapsed %.3f",
 	ClientHost, usertime, systime, STATfinish - STATstart);
+    /* Tracking code - Make entries in the logfile(s) to show that we have
+	finished with this session */
+    if (RARTenable) {
+	syslog(L_NOTICE, "%s Tracking Disabled (%s)", ClientHost, Username);
+	if (LLOGenable) {
+		fprintf(locallog, "%s Tracking Disabled (%s)\n", ClientHost, Username);
+		fclose(locallog);
+		syslog(L_NOTICE,"%s Local Logging ends (%s) %s",ClientHost, Username, LocalLogFileName);
+	}
+    }
+    if (ARTget)
+        syslog(L_NOTICE, "%s artstats get %d time %d size %d", ClientHost,
+            ARTget, ARTgettime, ARTgetsize);
+    if (innconf->nnrpdoverstats && OVERcount)
+        syslog(L_NOTICE, "%s overstats count %d hit %d miss %d time %d size %d read %d dbz %d seek %d get %d", ClientHost,
+            OVERcount, OVERhit, OVERmiss, OVERtime, OVERsize, OVERread, OVERdbz, OVERseek, OVERget);
+
+     if (DaemonMode) {
+     	shutdown(STDIN, 2);
+     	shutdown(STDOUT, 2);
+     	shutdown(STDERR, 2);
+ 	close(STDIN);    	
+ 	close(STDOUT);
+ 	close(STDERR);    	
+     }
+            
     exit(x);
 }
 
@@ -178,8 +220,12 @@ CMDhelp(ac, av)
 	    Printf("  %s\r\n", cp->Name);
 	else
 	    Printf("  %s %s\r\n", cp->Name, cp->Help);
-    Printf("Report problems to <%s@%s>\r\n",
-	NEWSMASTER, GetConfigValue(_CONF_FROMHOST));
+    if (strchr(NEWSMASTER, '@') == NULL)
+	Printf("Report problems to <%s@%s>\r\n",
+	    NEWSMASTER, innconf->fromhost);
+    else
+	Printf("Report problems to <%s>\r\n",
+	    NEWSMASTER);
     Reply(".\r\n");
 }
 
@@ -211,7 +257,7 @@ STATIC void
 TITLEset(what)
     char		*what;
 {
-#if	defined(DO_HAVE_SETPROCTITLE)
+#if defined(HAVE_SETPROCTITLE)
     setproctitle("%s %s", ClientHost, what);
 #else
 #if	!defined(HPUX)
@@ -239,7 +285,7 @@ TITLEset(what)
     (void)sprintf(buff, "(nnrpd) %s %s", ClientHost, what);
     (void)pstat(PSTAT_SETCMD, un, strlen(buff), 0, 0);
 #endif	/* defined(HPUX) */
-#endif	/* defined(DO_HAVE_SETPROCTITLE) */
+#endif	/* defined(HAVE_SETPROCTITLE) */
 }
 
 
@@ -288,7 +334,7 @@ Address2Name(ap, hostname, i)
 
     /* Only needed for misconfigured YP/NIS systems. */
     if (strchr(hostname, '.') == NULL
-     && (p = GetConfigValue(_CONF_DOMAIN)) != NULL) {
+     && (p = innconf->domain) != NULL) {
 	(void)strcat(hostname, ".");
 	(void)strcat(hostname, p);
     }
@@ -303,31 +349,34 @@ Address2Name(ap, hostname, i)
 
 
 BOOL
-PERMinfile(hp, ip, user, pass, accesslist)
+PERMinfile(hp, ip, user, pass, accesslist, accessfile)
     char		*hp;
     char		*ip;
     char		*user;
     char		*pass;
     char		*accesslist;
+    char		*accessfile;
 {
-    static char		ACCESS[] = _PATH_NNRPACCESS;
     register FILE	*F;
     register char	*p;
     register BOOL	found;
     register int	i;
     struct passwd	*pwd;
     char		buff[BIG_BUFFER];
+    char		filename[SMBUF];
     char		*fields[5];
 
-    if ((F = fopen(ACCESS, "r")) == NULL) {
-	syslog(L_ERROR, "%s cant fopen %s %m", ClientHost, ACCESS);
+    if ((F = fopen(accessfile, "r")) == NULL) {
+	syslog(L_ERROR, "%s cant fopen %s %m", ClientHost, accessfile);
 	return FALSE;
     }
 
     PERMcanread = FALSE;
     PERMcanpost = FALSE;
+    PERMlocpost = FALSE;
     found = FALSE;
     accesslist[0] = '\0';
+    filename[0] = '\0';
     while (fgets(buff, sizeof buff, F) != NULL) {
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
@@ -342,35 +391,87 @@ PERMinfile(hp, ip, user, pass, accesslist)
 		*p = '\0';
 		fields[++i] = p + 1;
 	    }
-	if (i != 4)
+	if ((i != 4) && !((i == 1) && (*fields[1] == '/'))) {
 	    /* Malformed line. */
+	    syslog(L_ERROR, "Malformed line in access file");
 	    continue;
+	}
 
-	if (hp)
+	if (hp) {
 	    /* Got an address; try to match either the IP address or as
 	     * a text hostname. */
-	    if (!(ip && wildmat(ip, fields[0])) && !wildmat(hp, fields[0]))
-		continue;
+	    int ipmatch = FALSE;
+	    int hostmatch = FALSE;
+	    int netmatch = FALSE;
 
+	    /* compare IP number and first field */
+	    if (ip && wildmat(ip, fields[0]))
+		ipmatch = TRUE;
+
+	    /* compare hostname and first field */
+	    if (wildmat(hp, fields[0]))
+		hostmatch = TRUE;
+
+	    /* try a netmask match */
+	    if (ip && (p = strchr(fields[0], '/')) != (char *)NULL) {
+		int bits, c;
+		struct in_addr ia, net;
+		unsigned int mask;
+
+		*p = '\0';
+		ia.s_addr = inet_addr(ip);
+		net.s_addr = inet_addr(fields[0]);
+		if (ia.s_addr != (unsigned int)INADDR_NONE &&
+		    net.s_addr != (unsigned int)INADDR_NONE) {
+		    if (strchr(p+1, '.') == (char *)NULL) {
+			mask = atoi(p+1);
+			for (bits = c = 0; c < mask && c < 32; c++)
+			    bits |= (1 << (31 - c));
+			mask = htonl(bits);
+		    } else {
+			mask = inet_addr(p+1);
+		    }
+		    if ((ia.s_addr & mask) == (net.s_addr & mask))
+			netmatch = TRUE;
+		}
+	    }
+	    if (!ipmatch && !hostmatch && !netmatch)
+		continue;
+	}
+
+	/* If the PERM field starts with '/', then we go to another file */
+	if (*fields[1] == '/') {
+	    strncpy(filename, fields[1], sizeof(filename));
+	    continue;
+	} else
+	    filename[0] = '\0';
+
+#ifdef DONT_HAVE_SHADOW
 	/* See if we should lookup a specific user in the passwd file */
 	if (user && pass && EQ(fields[2], "+")) {
 	    if ((pwd = getpwnam(user)) == NULL)
 		continue;
 	    if (!EQ(pwd->pw_passwd, crypt(pass, pwd->pw_passwd)))
 		continue;
-	}
+	} else
+#endif
 	/* Matching for a specific user or just the host? */
-	else if (user && (!EQ(user, fields[2]) || !EQ(pass, fields[3])))
+	if (user && (!EQ(user, fields[2]) || !EQ(pass, fields[3])))
 	    continue;
 
 	PERMcanread = strchr(fields[1], 'R') != NULL;
 	PERMcanpost = strchr(fields[1], 'P') != NULL;
+	PERMlocpost = strchr(fields[1], 'L') != NULL;
+	if (strchr(fields[1], 'N') != NULL) PERMnewnews = TRUE;
+	if (ForceReadOnly) PERMcanpost=FALSE;
 	(void)strcpy(PERMuser, user ? user : fields[2]);
 	(void)strcpy(PERMpass, pass ? pass : fields[3]);
 	(void)strcpy(accesslist, fields[4]);
 	found = TRUE;
     }
     (void)fclose(F);
+    if (found && (strlen(filename) > 0))
+	return(PERMinfile(hp, ip, user, pass, accesslist, filename));
     return found;
 }
 
@@ -383,7 +484,7 @@ StartConnection(accesslist)
     char		*accesslist;
 {
     struct sockaddr_in	sin;
-    int			length;
+    ARGTYPE		length;
     char		buff[SMBUF];
     char		*ClientAddr;
 
@@ -392,24 +493,24 @@ StartConnection(accesslist)
     ClientAddr = NULL;
     if (getpeername(STDIN, (struct sockaddr *)&sin, &length) < 0) {
 	if (!isatty(STDIN)) {
-	    syslog(L_ERROR, "%s cant getpeername %m", "?");
+	    syslog(L_TRACE, "%s cant getpeername %m", "?");
             (void)strcpy(ClientHost, "?"); /* so stats generation looks correct. */
 	    Printf("%d I can't get your name.  Goodbye.\r\n", NNTP_ACCESS_VAL);
 	    ExitWithStats(1);
 	}
 	(void)strcpy(ClientHost, "stdin");
+        ClientIP = 0L;
     }
 #if defined(AF_DECnet)
     else if (sin.sin_family == AF_DECnet) {
-	char *p, *MyDECnetDomain;
+	char *p;
 	(void) strcpy(ClientHost, getenv("REMNODE"));
 	for (p = ClientHost; *p; p++)
 	    if (CTYPE(isupper, *p))
 		*p = tolower(*p);
-#define _CONF_DECNETDOMAIN "decnetdomain"
-	if ((MyDECnetDomain = GetConfigValue(_CONF_DECNETDOMAIN)) != NULL) {
+	if (innconf->decnetdomain != NULL) {
 	    (void)strcat(ClientHost, ".");
-	    (void)strcat(ClientHost, MyDECnetDomain);
+	    (void)strcat(ClientHost, innconf->decnetdomain);
 	}
     }
 #endif
@@ -423,26 +524,32 @@ StartConnection(accesslist)
 
 	/* Get client's name. */
 #if	defined(DO_NNRP_GETHOSTBYADDR)
-	if (!Address2Name(&sin.sin_addr, ClientHost, sizeof ClientHost)) {
+	if (!Address2Name(&sin.sin_addr, ClientHost, (int)sizeof ClientHost)) {
 	    (void)strcpy(ClientHost, inet_ntoa(sin.sin_addr));
 	    syslog(L_NOTICE,
 		"? cant gethostbyaddr %s %m -- using IP address for access",
 		ClientHost);
+	    ClientAddr = ClientHost;
+            ClientIP = inet_addr(ClientHost);
 	}
 	else {
 	    ClientAddr = buff;
 	    (void)strcpy(buff, inet_ntoa(sin.sin_addr));
+            ClientIP = inet_addr(buff);
 	}
 #else
 	(void)strcpy(ClientHost, inet_ntoa(sin.sin_addr));
+        ClientIP = inet_addr(ClientHost);
 #endif /* defined(DO_NNRP_GETHOSTBYADDR) */
+	(void)strncpy(ClientIp, inet_ntoa(sin.sin_addr), sizeof(ClientIp));
     }
 
-    strncpy (LogName,ClientHost,sizeof(LogName)-1) ;
+    strncpy (LogName,ClientHost,sizeof(LogName) - 1) ;
+    LogName[sizeof(LogName) - 1] = '\0';
 
     syslog(L_NOTICE, "%s connect", ClientHost);
     if (!PERMinfile(ClientHost, ClientAddr, (char *)NULL, (char *)NULL,
-	    accesslist)) {
+	    accesslist, NNRPACCESS)) {
 	syslog(L_NOTICE, "%s no_access", ClientHost);
 	Printf("%d You are not in my access file.  Goodbye.\r\n",
 	    NNTP_ACCESS_VAL);
@@ -518,6 +625,34 @@ ToggleTrace(s)
     (void)signal(s, ToggleTrace);
 }
 
+/*
+** Got a SIGPIPE; exit cleanly
+*/
+STATIC SIGHANDLER
+CatchPipe(s)
+    int		s;
+{
+    ExitWithStats(0);
+}
+
+/*
+**  Got a signal; wait for children.
+*/
+STATIC SIGHANDLER
+WaitChild(s)
+    int		s;
+{
+    int status;
+    int pid;
+
+    for (;;) {
+       pid = waitnb(&status);
+       if (pid <= 0)
+       	    break;
+    }
+    (void)signal(s, WaitChild);
+}
+
 
 /*
 **  Print a usage message and exit.
@@ -550,6 +685,18 @@ main(argc, argv, env)
     char		*Reject;
     char		accesslist[BIG_BUFFER];
     int			timeout;
+    BOOL		val;
+    char		*p;
+    int			vid=0; 
+    int 		count=123456789;
+    struct		timeval tv;
+    unsigned short	ListenPort = NNTP_PORT;
+    unsigned long	ListenAddr = htonl(INADDR_ANY);
+    int			lfd, fd;
+    ARGTYPE		clen;
+    struct sockaddr_in	ssa, csa;
+    PID_T		pid = -1;
+   
 
 #if	!defined(HPUX)
     /* Save start and extent of argv for TITLEset. */
@@ -560,43 +707,140 @@ main(argc, argv, env)
     /* Parse arguments.   Must COPY() optarg if used because the
      * TITLEset() routine would clobber it! */
     Reject = NULL;
-    while ((i = getopt(argc, argv, "S:r:s:t")) != EOF)
+    RARTenable=FALSE;
+    LLOGenable=FALSE;
+
+    openlog("nnrpd", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
+
+    if (ReadInnConf() < 0) exit(1);
+
+    while ((i = getopt(argc, argv, "b:Di:lop:Rr:s:t")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
 	    /* NOTREACHED */
+ 	case 'b':			/* bind to a certain address in
+ 	        			   daemon mode */
+ 	    ListenAddr = inet_addr(optarg);
+ 	    if (ListenAddr == -1)
+ 	    	ListenAddr = htonl(INADDR_ANY);
+ 	    break;
+ 	case 'D':			/* standalone daemon mode */
+ 	    DaemonMode = TRUE;
+ 	    break;
 	case 'i':			/* Initial command */
 	    PushedBack = COPY(optarg);
 	    break;
-	case 'S':			/* We're a slave to NNTP master */
-	    RemoteMaster = COPY(optarg);
+	case 'l':			/* Tracking */
+	    RARTenable=TRUE;
+	    break;
+	case 'o':
+	    Offlinepost = TRUE;		/* Offline posting only */
+	    break;
+ 	case 'p':			/* tcp port for daemon mode */
+ 	    ListenPort = atoi(optarg);
+ 	    break;
+	case 'R':			/* Ignore 'P' option in access file */
+	    ForceReadOnly = TRUE;
+	    break;
+	case 'r':			/* Reject connection message */
+	    Reject = COPY(optarg);
 	    break;
 	case 's':			/* Unused title string */
 	    break;
 	case 't':			/* Tracing */
 	    Tracing = TRUE;
 	    break;
-	case 'r':			/* Reject connection message */
-	    Reject = COPY(optarg);
-	    break;
 	}
     argc -= optind;
     if (argc)
 	Usage();
 
+    HISTORY = COPY(cpcatpath(innconf->pathdb, _PATH_HISTORY));
+    ACTIVE = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVE));
+    ACTIVETIMES = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVETIMES));
+    NEWSGROUPS = COPY(cpcatpath(innconf->pathdb, _PATH_NEWSGROUPS));
+    NNRPACCESS = COPY(cpcatpath(innconf->pathetc, _PATH_NNRPACCESS));
+    SPOOLlen = strlen(innconf->patharticles);
+
+    if (DaemonMode) {
+	if ((lfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	    syslog(L_FATAL, "can't open socket (%m)");
+	    exit(1);
+	}
+	    
+	ssa.sin_family = AF_INET;
+	ssa.sin_addr.s_addr = ListenAddr;
+	ssa.sin_port = htons(ListenPort);
+	
+	if (bind(lfd, (struct sockaddr *) &ssa, sizeof(ssa)) < 0) {
+	    fprintf(stderr, "%s: can't bind (%s)\n", argv[0], strerror(errno));
+	    syslog(L_FATAL, "can't bind local address (%m)");
+	    exit(1);
+	}
+
+	/* Detach */
+	if ((pid = FORK()) < 0) {
+	    fprintf(stderr, "%s: can't ford (%s)\n", argv[0], strerror(errno));
+	    syslog(L_FATAL, "can't fork (%m)");
+	    exit(1);
+	} else if (pid != 0) 
+	    exit(0);
+
+	setsid();
+ 
+	/* Set signal handle to care for dead children */
+	(void)signal(SIGCHLD, WaitChild);
+ 
+	TITLEset("nnrpd: accepting connections");
+ 	
+	listen(lfd, 5);	
+
+listen_loop:
+	clen = sizeof(csa);
+	fd = accept(lfd, (struct sockaddr *) &csa, &clen);
+	if (fd < 0)
+		goto listen_loop;
+    
+	for (i = 0; (pid = FORK()) < 0; i++) {
+	    if (i == MAX_FORKS) {
+		syslog(L_FATAL, "cant fork %m -- giving up");
+		exit(1);
+	    }
+	    syslog(L_NOTICE, "cant fork %m -- waiting");
+	    (void)sleep(1);
+	}
+
+	if (pid != 0)
+		goto listen_loop;
+
+	/* child process starts here */
+	TITLEset("nnrpd: connected");
+	close(lfd);
+	dup2(fd, 0);
+	close(fd);
+	dup2(0, 1);
+	dup2(0, 2);
+ 
+    }  /* DaemonMode */
+
     /* Setup. */
-    openlog("nnrpd", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
     if (GetTimeInfo(&Now) < 0) {
 	syslog(L_FATAL, "cant gettimeinfo %m");
 	exit(1);
     }
     STATstart = TIMEINFOasDOUBLE(Now);
 
-    if ((MyHostName = GetConfigValue(_CONF_PATHHOST)) == NULL) {
-	syslog(L_FATAL, "cant getconfigvalue %m");
+    PERMnewnews = innconf->allownewnews;
+
+    if (innconf->overviewmmap)
+	val = TRUE;
+    else
+	val = FALSE;
+    if (!OVERsetup(OVER_MMAP, &val) || !OVERsetup(OVER_MODE, "r")) {
+	syslog(L_FATAL, "cant setup unified overview");
 	ExitWithStats(1);
     }
-    MyHostName = COPY(MyHostName);
 
 #if	NNRP_LOADLIMIT > 0
     if ((load = GetLoadAverage()) > NNRP_LOADLIMIT) {
@@ -606,10 +850,13 @@ main(argc, argv, env)
     }
 #endif	/* NNRP_LOADLIMIT > 0 */
 
-    strcpy (LogName,"?") ;
-    
-    /* Ignore SIGPIPE, since we'll see closed connections with read. */
-    (void)signal(SIGPIPE, SIG_IGN);
+    strcpy (LogName, "?");
+
+    OVERindex = NULL;
+    OVERicount = 0;
+
+    /* Catch SIGPIPE so that we can exit out of long write loops */
+    (void)signal(SIGPIPE, CatchPipe);
 
     /* Arrange to toggle tracing. */
     (void)signal(SIGHUP, ToggleTrace);
@@ -635,6 +882,26 @@ main(argc, argv, env)
 	ExitWithStats(0);
     }
 
+    if (RARTenable)
+	RARTenable=TrackClient(ClientHost,Username);
+
+    if (RARTenable) {
+	syslog(L_NOTICE, "%s Tracking Enabled (%s)", ClientHost, Username);
+	pid=getpid();
+	gettimeofday(&tv,NULL);
+	count += pid;
+	vid = tv.tv_sec ^ tv.tv_usec ^ pid ^ count;
+	sprintf(LocalLogFileName, "%s/tracklogs/log-%ld", innconf->pathlog,vid);
+	if ((locallog=fopen(LocalLogFileName, "w")) != NULL) {
+		syslog(L_NOTICE, "%s Local Logging begins (%s) %s",ClientHost, Username, LocalLogFileName);
+		fprintf(locallog, "%s Tracking Enabled (%s)\n", ClientHost, Username);
+		fflush(locallog);
+		LLOGenable=TRUE;
+	} else {
+		syslog(L_NOTICE, "%s Local Logging failed (%s) %s", ClientHost, Username, LocalLogFileName);
+	}
+    }
+
     ARTreadschema();
     if (!GetGroupList()) {
 	/* This shouldn't really happen. */
@@ -643,19 +910,29 @@ main(argc, argv, env)
 	ExitWithStats(1);
     }
 
+    if (innconf->storageapi && !SMinit()) {
+	syslog(L_NOTICE, "%s cant initialize storage method, %s", ClientHost, SMerrorstr);
+	Reply("%d NNTP server unavailable. Try later.\r\n", NNTP_TEMPERR_VAL);
+	ExitWithStats(1);
+    }
+
 #if defined(DO_PERL)
     /* Load the Perl code */
-    PERLsetup(NULL, _PATH_PERL_FILTER_NNRPD, "filter_post");
+    PERLsetup(NULL, cpcatpath(innconf->pathfilter, _PATH_PERL_FILTER_NNRPD), "filter_post");
     PerlFilter (TRUE) ;
 #endif /* defined(DO_PERL) */
 
     Reply("%d %s InterNetNews NNRP server %s ready (%s).\r\n",
 	   PERMcanpost ? NNTP_POSTOK_VAL : NNTP_NOPOSTOK_VAL,
-	   MyHostName, INNVersion(),
+	   innconf->pathhost, INNVersion(),
 	   PERMcanpost ? "posting ok" : "no posting");
 
+    /* Exponential posting backoff */
+    (void)InitBackoffConstants();
+
     /* Main dispatch loop. */
-    for (timeout = INITIAL_TIMEOUT, av = NULL; ; timeout = CLIENT_TIMEOUT) {
+    for (timeout = INITIAL_TIMEOUT, av = NULL; ;
+			timeout = innconf->clienttimeout) {
 	(void)fflush(stdout);
 	if (ChangeTrace) {
 	    Tracing = Tracing ? FALSE : TRUE;
@@ -675,7 +952,7 @@ main(argc, argv, env)
 		syslog(L_ERROR, "%s internal %d in main", ClientHost, r);
 		/* FALLTHROUGH */
 	    case RTtimeout:
-		if (timeout < CLIENT_TIMEOUT)
+		if (timeout < innconf->clienttimeout)
 		    syslog(L_NOTICE, "%s timeout short", ClientHost);
 		else
 		    syslog(L_NOTICE, "%s timeout", ClientHost);
