@@ -1,10 +1,7 @@
-/*  $Revision: 1.1 $
+/*  $Revision: 1.22 $
 **
 **  Net News Reading Protocol server.
 */
-#include "configdata.h"
-#include <stdio.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <ctype.h>
@@ -19,11 +16,17 @@
 #include <fcntl.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#if	defined(DO_NEED_TIME)
+#include <time.h>
+#endif	/* defined(DO_NEED_TIME) */
+#include <sys/time.h>
+#if defined(OVER_MMAP) || defined(ART_MMAP)
+#include <sys/mman.h>
+#endif
 #include "paths.h"
 #include "nntp.h"
-#include "logging.h"
+#include <syslog.h> 
 #include "libinn.h"
-#include "clibrary.h"
 #include "qio.h"
 #include "macros.h"
 
@@ -33,39 +36,32 @@
 */
 #define ART_LINE_LENGTH		1000
 #define ART_LINE_MALLOC		1024
+#define ART_MAX			1024
 
 
 /*
 **  Some convenient shorthands.
 */
 typedef struct in_addr	INADDR;
-#define Printf		(void)printf
+#define Printf		printf
 #if	defined(VAR_NONE)
-#define Reply		(void)printf
+#define Reply		printf
 #endif	/* defined(VAR_NONE) */
 
 
-/*
-**  The XTHREAD command is too ugly to talk about, but if you want it
-**  change DONT_DO_XTRHEAD to DO_DO_XTRHEAD, below.
-*/
-#define DONT_DO_XTHREAD
-#define THREAD_NAMES_FLAT
-#define THREAD_SUFFIX		""
-#define THREAD_DB		"/usr/spool/news/trn.threads/db.init"
-#define THREAD_DIR		"/usr/spool/news/trn.threads"
-#define THREAD_NNTP_CODE	288
-
-
 /*
 **  A group entry.
 */
 typedef struct _GROUPENTRY {
+#ifdef NOTDEF
     char	*Name;
     ARTNUM	High;
     ARTNUM	Low;
     char	Flag;
     char	*Alias;
+#endif
+    char        *Ptr;   
+    struct _GROUPENTRY *Next;
 } GROUPENTRY;
 
 
@@ -88,6 +84,14 @@ typedef enum _READTYPE {
     RTtimeout
 } READTYPE;
 
+typedef struct _ARTLIST {
+    ARTNUM              ArtNum;
+    char                (*Index)[OVERINDEXPACKSIZE];
+    TOKEN		Token; /* for overview */
+    OFFSET_T		Offset;
+    BOOL		Tokenretrieved;
+} ARTLIST;
+
 
 #if	defined(MAINLINE)
 #define EXTERN	/* NULL */
@@ -100,30 +104,55 @@ EXTERN BOOL	PERMcanpost;
 EXTERN BOOL	PERMcanread;
 EXTERN BOOL	PERMneedauth;
 EXTERN BOOL	PERMspecified;
+EXTERN BOOL	PERMnewnews;
+EXTERN BOOL	PERMlocpost;
 EXTERN BOOL	Tracing;
+EXTERN BOOL 	Offlinepost;
 EXTERN char	**PERMlist;
-EXTERN STRING	MyHostName;
-extern char	ACTIVE[];
 EXTERN char	ClientHost[SMBUF];
-extern char	ACTIVETIMES[];
-extern char	HISTORY[];
-extern char	NEWSGROUPS[];
+EXTERN char	Username[SMBUF];
+EXTERN char     ClientIp[20];
+EXTERN char	LogName[256] ;
+extern char	*ACTIVETIMES;
+extern char	*HISTORY;
+extern char	*ACTIVE;
+extern char	*NEWSGROUPS;
+extern char	*NNRPACCESS;
 extern char	NOACCESS[];
+EXTERN int	SPOOLlen;
 EXTERN char	PERMpass[20];
 EXTERN char	PERMuser[20];
-EXTERN char	*RemoteMaster;
-EXTERN ARTNUM	*ARTcache;
-EXTERN ARTNUM	*ARTnumbers;
+EXTERN ARTLIST	*ARTcache;
+EXTERN ARTLIST	*ARTnumbers;
+EXTERN FILE	*locallog;
 EXTERN int	ARTindex;
 EXTERN int	ARTsize;
 extern int	PERMdefault;
 EXTERN long	ARTcount;
+EXTERN long	ARTget;
+EXTERN long	ARTgettime;
+EXTERN long	ARTgetsize;
+EXTERN long	OVERcount;	/* number of XOVER commands			*/
+EXTERN long	OVERhit;	/* number of XOVER records found in .overview	*/
+EXTERN long	OVERmiss;	/* number of XOVER records found in articles	*/
+EXTERN long	OVERtime;	/* number of ms spent sending XOVER data	*/
+EXTERN long	OVERread;	/* number of bytes of XOVER data read		*/
+EXTERN long	OVERsize;	/* number of bytes of XOVER data sent		*/
+EXTERN long	OVERdbz;	/* number of ms spent reading dbz data	*/
+EXTERN long	OVERseek;	/* number of ms spent seeking history	*/
+EXTERN long	OVERget;	/* number of ms spent reading history	*/
+/* Pointer to memory containing overview index, may be read only */
+EXTERN char     (*OVERindex)[][OVERINDEXPACKSIZE];
+EXTERN int      OVERicount;      /* Number of OVERINDEX entries at OVERindex */
 EXTERN long	GRParticles;
 EXTERN long	GRPcount;
 EXTERN char	GRPlast[SPOOLNAMEBUFF];
 EXTERN long	POSTreceived;
 EXTERN long	POSTrejected;
 
+EXTERN BOOL     BACKOFFenabled;
+EXTERN long     ClientIP;                                 
+
 
 #if	NNRP_LOADLIMIT > 0
 extern int		GetLoadAverage();
@@ -135,8 +164,11 @@ extern char		*Glom();
 extern int		Argify();
 extern NORETURN		ExitWithStats();
 extern BOOL		GetGroupList();
+extern char		*GetHeader();
 extern void		GRPreport();
 extern GROUPENTRY	*GRPfind();
+extern void		HIScheck();
+extern BOOL		OVERgetent();
 extern char		*HISgetent();
 extern long		LOCALtoGMT();
 extern BOOL		NGgetlist();
@@ -153,3 +185,10 @@ extern void		Reply(char *, ...);
 #if	defined(VAR_VARARGS)
 extern void		Reply();
 #endif	/* defined(VAR_VARARGS) */
+
+char *GPNAME(GROUPENTRY *gp);   
+ARTNUM GPLOW(GROUPENTRY *gp);
+ARTNUM GPHIGH(GROUPENTRY *gp);
+char GPFLAG(GROUPENTRY *gp);
+char *GPALIAS(GROUPENTRY *gp); 
+
