@@ -1,9 +1,10 @@
-/*  $Revision: 1.32 $
+/*  $Revision: 1.22 $
 **
 **  Check article, send it to the local server.
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include <netinet/in.h>
 #include "configdata.h"
 #include "clibrary.h"
 #include "nnrpd.h"
@@ -17,11 +18,15 @@
 #define FLUSH_ERROR(F)		(fflush((F)) == EOF || ferror((F)))
 #define HEADER_DELTA		20
 
+extern int RARTenable;
+extern int LLOGenable;
+
 STATIC char     *tmpPtr ;
 STATIC char	Error[SMBUF];
 STATIC char	NGSEPS[] = NG_SEPARATOR;
-STATIC char	**OtherHeaders;
-STATIC int	OtherCount;
+char	**OtherHeaders;
+int	OtherCount;
+BOOL   HeadersModified;
 STATIC int	OtherSize;
 STATIC BOOL	WasMailed;
 STATIC STRING	BadDistribs[] = {
@@ -72,6 +77,12 @@ HEADER	Table[] = {
 #define _contenttype	19
     {	"Content-Transfer-Encoding", TRUE, HTstd },
 #define _contenttransferencoding 20
+    {   "X-Trace",              FALSE, HTstd },
+#define _xtrace         21
+    {   "X-Complaints-To",	FALSE, HTstd },
+#define _xcomplaintsto	22
+    {   "NNTP-Posting-Date",	FALSE, HTstd },
+#define _nntppostdate	23
     {	"Xref",			FALSE,	HTstd },
     {	"Summary",		TRUE,	HTstd },
     {	"Keywords",		TRUE,	HTstd },
@@ -80,6 +91,12 @@ HEADER	Table[] = {
     {	"Posted",		FALSE,	HTobs },
     {	"Posting-Version",	FALSE,	HTobs },
     {	"Relay-Version",	FALSE,	HTobs },
+    {   "Cc",			TRUE, HTstd },
+#define _cc		32
+    {   "Bcc",			TRUE, HTstd },
+#define _bcc		33
+    {   "To",			TRUE, HTstd },
+#define _to		34
 };
 
 HEADER *EndOfTable = ENDOF(Table);
@@ -313,6 +330,8 @@ ProcessHeaders(linecount)
     static char		datebuff[40];
     static char		orgbuff[SMBUF];
     static char		linebuff[40];
+    static char		tracebuff[SMBUF];
+    static char 	complaintsbuff[SMBUF];
     static char		sendbuff[SMBUF];
     static char		mimeversion[SMBUF];
     static char		mimetype[SMBUF];
@@ -323,6 +342,7 @@ ProcessHeaders(linecount)
     struct tm		*gmt;
     TIMEINFO		Now;
     STRING		error;
+    pid_t               pid;
 
     /* Do some preliminary fix-ups. */
     for (hp = Table; hp < ENDOF(Table); hp++) {
@@ -356,14 +376,14 @@ ProcessHeaders(linecount)
 	return Error;
     }
 
-    if (HDR(_date) == NULL) {
-	if ((gmt = gmtime(&Now.time)) == NULL)
+    if ((gmt = gmtime(&Now.time)) == NULL)
 	    return "Can't get the time";
-	(void)sprintf(datebuff, "%d %3.3s %d %02d:%02d:%02d GMT",
+    (void)sprintf(datebuff, "%d %3.3s %d %02d:%02d:%02d GMT",
 	    gmt->tm_mday, &MONTHS[3 * gmt->tm_mon], 1900 + gmt->tm_year,
 	    gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
+
+    if (HDR(_date) == NULL)
 	HDR(_date) = datebuff;
-    }
     else {
 	if ((t = parsedate(HDR(_date), &Now)) == -1)
 	    return "Can't parse \"Date\" header";
@@ -443,7 +463,7 @@ ProcessHeaders(linecount)
 
     /* Set Organization */
     if (HDR(_organization) == NULL
-     && (p = GetConfigValue(_CONF_ORGANIZATION)) != NULL) {
+     && (p = innconf->organization) != NULL) {
 	(void)strcpy(orgbuff, p);
 	HDR(_organization) = orgbuff;
     }
@@ -454,13 +474,13 @@ ProcessHeaders(linecount)
 
     /* MIME headers. */
     if (HDR(_mimeversion) == NULL
-     && (p = GetConfigValue(_CONF_MIMEVERSION)) != NULL) {
+     && (p = innconf->mimeversion) != NULL) {
 	(void)strcpy(mimeversion, p);
 	HDR(_mimeversion) = mimeversion;
 
 	/* Set Content-Type */
 	if (HDR(_contenttype) == NULL) {
-	    if ((p = GetConfigValue(_CONF_CONTENTTYPE)) == NULL)
+	    if ((p = innconf->mimecontenttype) == NULL)
 		return "Can't get \"Content-Type\" header";
 	    (void)strcpy(mimetype, p);
 	    HDR(_contenttype) = mimetype;
@@ -468,7 +488,7 @@ ProcessHeaders(linecount)
 
 	/* Set Content-Transfer-Encoding */
 	if (HDR(_contenttransferencoding) == NULL) {
-	    if ((p = GetConfigValue(_CONF_ENCODING)) == NULL)
+	    if ((p = innconf->mimeencoding) == NULL)
 		return "Can't get \"Content-Transfer-Encoding\" header";
 	    (void)strcpy(mimeencoding, p);
 	    HDR(_contenttransferencoding) = mimeencoding;
@@ -483,12 +503,43 @@ ProcessHeaders(linecount)
 
     /* NNTP-Posting host; set. */
     HDR(_nntpposthost) = ClientHost;
+    /* NNTP-Posting-Date - not in RFC (yet) */
+    HDR(_nntppostdate) = datebuff;
 
+    /* X-Trace; set */
+    t = time((time_t *)NULL) ;
+    pid = (long) getpid() ;
+    if ((gmt = gmtime(&Now.time)) == NULL)
+	return "Can't get the time";
+    if ((p = GetFQDN()) == NULL)
+	p = "unknown";
+    sprintf(tracebuff, "%s %ld %ld %s (%d %3.3s %d %02d:%02d:%02d GMT)",
+             GetFQDN(), (long) t, (long) pid, ClientIp,
+	    gmt->tm_mday, &MONTHS[3 * gmt->tm_mon], 1900 + gmt->tm_year,
+	    gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
+    HDR (_xtrace) = tracebuff ;
+
+    /* X-Complaints-To; set */
+    if ((p = innconf->complaints) != NULL)
+      sprintf (complaintsbuff, "%s",p) ;
+    else {
+      if ((p = innconf->fromhost) != NULL) 
+	sprintf (complaintsbuff, "%s@%s",
+                NEWSMASTER, p);
+    }
+    if (p != NULL)  /* Only show it if we can create it */
+	HDR(_xcomplaintsto) = complaintsbuff ;
+
+    /* Clear out some headers that should not be here */
+    if (innconf->strippostcc) {
+	HDR(_cc) = NULL;
+	HDR(_bcc) = NULL;
+	HDR(_to) = NULL;
+    }
     /* Now make sure everything is there. */
     for (hp = Table; hp < ENDOF(Table); hp++)
 	if (hp->Type == HTreq && hp->Value == NULL) {
-	    (void)sprintf(Error, "Required \"%s\" header is missing",
-		    hp->Name);
+	    (void)sprintf(Error, "Required \"%s\" header is missing", hp->Name);
 	    return Error;
 	}
 
@@ -496,7 +547,6 @@ ProcessHeaders(linecount)
 }
 
 
-#if	defined(DO_CHECK_INCLUDED_TEXT)
 /*
 **  See if the user has more included text than new text.  Simple-minded, but
 **  reasonably effective for catching neophyte's mistakes.  A line starting
@@ -532,7 +582,6 @@ CheckIncludedText(p, lines)
 	return "Article not posted -- more included text than new text";
     return NULL;
 }
-#endif	/* defined(DO_CHECK_INCLUDED_TEXT) */
 
 
 
@@ -550,6 +599,7 @@ MailArticle(group, article)
     register int	i;
     char		*address;
     char		buff[SMBUF];
+    char		*mta;
 
     /* Try to get the address first. */
     if ((address = GetModeratorAddress(NULL, NULL, group)) == NULL) {
@@ -559,8 +609,10 @@ MailArticle(group, article)
     }
 
     /* Now build up the command (ignore format/argument mismatch errors,
-     * in case %s isn't in _PATH_SENDMAIL) and send the headers. */
-    (void)sprintf(buff, _PATH_SENDMAIL, address);
+     * in case %s isn't in inconf->mta) and send the headers. */
+    if ((mta = innconf->mta) == NULL)
+	return "Can't start mailer - mta not set";
+    (void)sprintf(buff, innconf->mta, address);
     if ((F = popen(buff, "w")) == NULL)
 	return "Can't start mailer";
     (void)fprintf(F, "To: %s\n", address);
@@ -644,32 +696,22 @@ ValidNewsgroups(hdr, article)
 	    continue;
 	FoundOne = TRUE;
 	DDcheck(h, p);
-        if (PERMspecified) {
-            grplist[0] = p;
-            grplist[1] = NULL;
-            if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
-                sprintf(Error,"%s: %s\r\n", NNTP_NOSUCHGROUP,p);
-                DISPOSE(groups);
-                tmpPtr = DDend(h);
-                DISPOSE(tmpPtr);
-                return Error;
-            }
-        }
-	switch (gp->Flag) {
+	switch (GPFLAG(gp)) {
 	case NF_FLAG_OK:
 	    break;
 	case NF_FLAG_MODERATED:
 	    if (!approved) {
 		DISPOSE(groups);
-                tmpPtr = DDend(h);
+		tmpPtr = DDend(h);
 		DISPOSE(tmpPtr);
-		return MailArticle(gp->Name, article);
+		return MailArticle(GPNAME(gp), article);
 	    }
 	    break;
 	case NF_FLAG_IGNORE:
 	case NF_FLAG_NOLOCAL:
-	    (void)sprintf(Error, "Postings to \"%s\" are not allowed here.",
-		    gp->Name);
+	    if (!PERMlocpost)
+		(void)sprintf(Error, "Postings to \"%s\" are not allowed here.",
+				GPNAME(gp));
 	    break;
 	case NF_FLAG_EXCLUDED:
 	    /* Do NOT return an error. */
@@ -677,9 +719,16 @@ ValidNewsgroups(hdr, article)
 	case NF_FLAG_ALIAS:
 	    (void)sprintf(Error,
 		    "The newsgroup \"%s\" has been renamed to \"%s\".\n",
-		    p, gp->Alias);
+		    p, GPALIAS(gp));
 	    break;
 	}
+        if (PERMspecified) {
+	    grplist[0] = p;
+	    grplist[1] = NULL;
+	    if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
+		sprintf(Error,"%s: %s\r\n", NNTP_NOSUCHGROUP,p);
+	    }
+        }
     } while ((p = strtok((char *)NULL, NGSEPS)) != NULL);
     DISPOSE(groups);
 
@@ -755,13 +804,12 @@ Spoolit(article, Error)
     register int	i;
     char		temp[BUFSIZ];
     char		path[BUFSIZ];
-    struct stat		Sb;
 
     /* Initialize the returned error message */
     sprintf(CANTSPOOL, "%s and can't write text to local spool file", Error);
 
     /* Try to write it to the spool dir. */
-    TempName(_PATH_SPOOLNEWS, temp);
+    TempName(innconf->pathincoming, temp);
     /* rnews -U ignores files starting with . */
     strrchr(temp, '/')[1] = '.';
     if ((F = fopen(temp, "w")) == NULL) {
@@ -799,7 +847,7 @@ Spoolit(article, Error)
     if (fclose(F))
 	return CANTSPOOL;
 
-    TempName(_PATH_SPOOLNEWS, path);
+    TempName(innconf->pathincoming, path);
     if (rename(temp, path) < 0) {
         syslog(L_FATAL, "cant rename %s %s %m", temp, path);
 	return CANTSPOOL;
@@ -823,6 +871,11 @@ ARTpost(article, idbuff)
     FILE		*FromServer;
     char		buff[NNTP_STRLEN + 2], frombuf[SMBUF];
     STRING		error;
+    char		TrackID[NNTP_STRLEN];
+    FILE		*ftd;
+    int			result;
+
+    sprintf(TrackID,"%s/trackposts/track.", innconf->pathlog);
 
     /* Set up the other headers list. */
     if (OtherHeaders == NULL) {
@@ -842,10 +895,10 @@ ARTpost(article, idbuff)
     for (i = 0, p = article; p; i++, p = next + 1)
 	if ((next = strchr(p, '\n')) == NULL)
 	    break;
-#if	defined(DO_CHECK_INCLUDED_TEXT)
-    if ((error = CheckIncludedText(article, i)) != NULL)
-	return error;
-#endif	/* defined(DO_CHECK_INCLUDED_TEXT) */
+    if (innconf->checkincludedtext) {
+	if ((error = CheckIncludedText(article, i)) != NULL)
+		return error;
+    }
     if ((error = ProcessHeaders(i)) != NULL)
 	return error;
     if (i == 0 && HDR(_control) == NULL)
@@ -861,6 +914,11 @@ ARTpost(article, idbuff)
     
     strncpy(frombuf, HDR(_from), sizeof(frombuf) - 1);
     frombuf[sizeof(frombuf) - 1] = '\0';
+    for (i = 0, p = frombuf;p < frombuf + sizeof(frombuf);)
+	if ((p = strchr(p, '\n')) == NULL)
+	    break;
+	else
+	    *p++ = ' ';
     HeaderCleanFrom(frombuf);
     p = strchr(frombuf, '@');
     if (p) {
@@ -875,30 +933,35 @@ ARTpost(article, idbuff)
      && !EQ(p, "poster")
      && (error = ValidNewsgroups(p, (char *)NULL)) != NULL)
 	return error;
-#if	LOCAL_MAX_ARTSIZE > 0
-    if (strlen(article) > LOCAL_MAX_ARTSIZE) {
-	(void)sprintf(Error,
+    if ((innconf->localmaxartsize > 0) &&
+		(strlen(article) > innconf->localmaxartsize)) {
+	    (void)sprintf(Error,
 		"Article is bigger then local limit of %ld bytes\n",
-		LOCAL_MAX_ARTSIZE);
-	return Error;
+		atoi(p));
+	    return Error;
     }
-#endif	/* LOCAL_MAX_ARTSIZE > 0 */
 
 #if defined(DO_PERL)
     /* Calls the Perl subroutine for headers management */
-    if ((p = (char *)HandleHeaders()) != NULL)
+    if ((p = (char *)HandleHeaders(article)) != NULL)
         return p;
 #endif /* defined(DO_PERL) */
 
+    if (innconf->spoolfirst)
+	return Spoolit(article, Error);
+
+    if (Offlinepost)
+         return Spoolit(article,Error);
+
     /* Open a local connection to the server. */
-    if (RemoteMaster)
-	i = NNTPconnect(RemoteMaster, &FromServer, &ToServer, buff);
+    if (innconf->nnrpdposthost != NULL)
+	i = NNTPconnect(innconf->nnrpdposthost, NNTP_PORT, &FromServer, &ToServer, buff);
     else {
-#if	defined(DO_HAVE_UNIX_DOMAIN)
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
 	i = NNTPlocalopen(&FromServer, &ToServer, buff);
 #else
-	i = NNTPremoteopen(&FromServer, &ToServer, buff);
-#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+	i = NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff);
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     }
 
     /* If we cannot open the connection, initialize the error message and
@@ -912,22 +975,22 @@ ARTpost(article, idbuff)
     }
     if (Tracing)
 	syslog(L_TRACE, "%s post_connect %s",
-	    ClientHost, RemoteMaster ? RemoteMaster : "localhost");
+	    ClientHost, innconf->nnrpdposthost ? innconf->nnrpdposthost : "localhost");
 
     /* The code below has too many (void) casts for my tastes.  At least
      * they are all inside cases that are most likely never going to
      * happen -- for example, if the server crashes. */
 
     /* Offer article to server. */
-    i = OfferArticle(buff, sizeof buff, FromServer, ToServer);
+    i = OfferArticle(buff, (int)sizeof buff, FromServer, ToServer);
     if (i == NNTP_AUTH_NEEDED_VAL) {
         /* Send authorization. */
-        if (NNTPsendpassword(RemoteMaster, FromServer, ToServer) < 0) {
+        if (NNTPsendpassword(innconf->nnrpdposthost, FromServer, ToServer) < 0) {
             (void)sprintf(Error, "Can't authorize with %s",
-                          RemoteMaster ? RemoteMaster : "innd");
+                          innconf->nnrpdposthost ? innconf->nnrpdposthost : "innd");
             return Spoolit(article,Error);
         }
-        i = OfferArticle(buff, sizeof buff, FromServer, ToServer);
+        i = OfferArticle(buff, (int)sizeof buff, FromServer, ToServer);
     }
     if (i != NNTP_SENDIT_VAL) {
         (void)strcpy(Error, buff);
@@ -973,5 +1036,30 @@ ARTpost(article, idbuff)
 	(void)strncpy(idbuff, HDR(_messageid), SMBUF - 1);
 	idbuff[SMBUF - 1] = '\0';
     }
+
+    /* Tracking */
+    if (RARTenable) {
+	strcat(TrackID,HDR(_messageid));
+	if ((ftd=fopen(TrackID,"w")) != NULL) {
+		for (hp = Table; hp < ENDOF(Table); hp++)
+			if (hp->Value)
+				(void)fprintf(ftd, "%s: %s\r\n", hp->Name, hp->Value);
+		for (i=0; i<OtherCount; i++)
+			(void)fprintf(ftd,"%s\r\n",OtherHeaders[i]);
+		(void)fprintf(ftd,"\r\n");
+		result=fputs(article,ftd);
+		fclose(ftd);
+		if (result != EOF) {
+			syslog(L_NOTICE, "%s (%s) posttrack ok %s",
+				ClientHost, Username, TrackID);
+			if (LLOGenable)
+				fprintf(locallog, "%s (%s) posttrack ok %s\n",
+					ClientHost, Username, TrackID);
+		} else 
+			syslog(L_NOTICE, "%s (%s) posttrack error 2 %s",
+				ClientHost, Username, TrackID);
+	}
+    }
+
     return NULL;
 }
