@@ -1,5 +1,5 @@
 #
-# $Id: filter_innd.pl,v 1.5 1996/11/26 15:00:56 brister Exp $
+# $Id: filter_innd.pl,v 1.1.1.1 1997/08/04 04:03:49 gpalmer Exp $
 #
 # Sample Perl filtering file for the innd hooks.
 # 
@@ -58,7 +58,7 @@
 #
 
 sub filter_art {
-	my ($rval) = "" ; # Assume we'll accept. Cannot be `0'
+	my $rval = "" ; # Assume we'll accept. Cannot be `0'
 
 ### Remove two leading '##' from the following section (and then
 ### 'ctlinnd reload filter.perl' and the filter will reject articles that
